package com.bria.common.controller.calllog

import android.content.Context
import com.bria.common.controller.IController
import com.bria.common.controller.calllog.db.*
import com.bria.common.controller.settings.ESetting
import io.reactivex.Flowable
import io.reactivex.Single
import io.reactivex.schedulers.Schedulers

class CallLogApiImpl(private val context: Context, controller: IController): CallLogApi {
    private val mCallLogDao: CallLogDao

    init {
        val isBroadworks = controller.broadworksCtrl.events.isBroadworksEnabled &&
                controller.settingsCtrl.events.getBool(ESetting.BroadWorksEnterpriseCallLog)
        mCallLogDao = if (!isBroadworks) CallLogDatabase.get(context).callLogDao() else BwCallLogDao()
    }

    override fun put(callLogEntity: CallLogEntity) =
            Single.fromCallable { mCallLogDao.put(callLogEntity) }.subscribeOn(Schedulers.io())!!

    override fun get(id: Long): Flowable<CallLogEntity> = mCallLogDao.get(id)

    override fun delete(vararg entities: CallLogEntity) =
            Single.fromCallable { mCallLogDao.delete(*entities) }.subscribeOn(Schedulers.io())!!

    override fun delete(callType: CallType) =
            Single.fromCallable { mCallLogDao.delete(callType) }.subscribeOn(Schedulers.io())!!

    override fun deleteAll() =
            Single.fromCallable { mCallLogDao.deleteAll() }.subscribeOn(Schedulers.io())!!

    override fun getAll() = mCallLogDao.getAll()

    override fun destroy() {
        CallLogDatabase.destroy()
    }
}
