package com.bria.common.controller.collaboration.rx.api

import com.bria.common.controller.collaboration.rx.entities.ConferenceConfig
import io.reactivex.Completable
import io.reactivex.Single

/**
 * Provides reactive access to collaboration features.
 */
interface ReactiveCollabApi {

    /**
     * Gets the collab conference invite URL.
     * @return A Single to wait for the URL
     */
    fun getInviteUrl(): Single<String>

    /**
     * Gets the primary conference's configuration.
     * @return A Single to wait for the [ConferenceConfig]
     */
    fun getConferenceConfig(): Single<ConferenceConfig>

    /**
     * Sets the primary conference's configuration.
     * @return A Completable to wait for the success event
     */
    fun setConferenceConfig(config: ConferenceConfig): Completable

    /**
     * Disconnects all previously connected accounts in a blocking fashion.
     */
    fun disconnectAll()

}
