package com.bria.common.controller.collaboration.rx.entities

import android.os.Parcel
import android.os.Parcelable
import com.counterpath.sdk.pb.VccsConference

data class ConferenceConfig(
        var isModerated: Boolean = false,
        var overrunMinutes: Int = 0,
        var participantPin: String = "",
        var moderatorPin: String = "",
        var dropboxToken: String = ""
) : Parcelable {

    constructor(event: VccsConference.ConferenceConfigEvent) : this(
            event.isModerated,
            event.noModeratorOverrunMinutes,
            event.participantPin,
            event.moderatorPin,
            event.dropBoxToken
    )

    fun convertToRequest(): Pair<ConferenceConfigValues, ConferenceConfigFlags> {
        val values = ConferenceConfigValues(
                isModerated = isModerated,
                overrunMinutes = overrunMinutes,
                participantPin = participantPin,
                moderatorPin = moderatorPin,
                dropBoxToken = dropboxToken
        )
        val flags = ConferenceConfigFlags(
                isModerated = true,
                overrunMinutes = true,
                participantPin = true,
                moderatorPin = true,
                dropBoxToken = false // not setting it from mobiles for now
        )
        return values to flags
    }

    // <editor-fold desc="Parcelable implementation">
    constructor(parcel: Parcel) : this(
            parcel.readByte() != 0.toByte(),
            parcel.readInt(),
            parcel.readString(),
            parcel.readString(),
            parcel.readString()
    )

    override fun writeToParcel(parcel: Parcel, flags: Int) {
        parcel.writeByte(if (isModerated) 1 else 0)
        parcel.writeInt(overrunMinutes)
        parcel.writeString(participantPin)
        parcel.writeString(moderatorPin)
        parcel.writeString(dropboxToken)
    }

    override fun describeContents(): Int = 0

    companion object CREATOR : Parcelable.Creator<ConferenceConfig> {
        override fun createFromParcel(parcel: Parcel): ConferenceConfig = ConferenceConfig(parcel)
        override fun newArray(size: Int): Array<ConferenceConfig?> = arrayOfNulls(size)
    }
    // </editor-fold>

}
