package com.bria.common.controller.collaboration.rx.entities

import com.bria.common.controller.collaboration.rx.api.ISipVccsConferenceApi.Companion.K_ERROR

/**
 * Holds a response from the collab API, allowing access to `accountId` and `conferenceId`,
 * as well as to the event itself. In case of missing `accountId` or `conferenceId`, their
 * value will be set to [K_ERROR].
 */
data class EventHolder<out T>(
        val accountId: Int = K_ERROR,
        val conferenceId: Int = K_ERROR,
        val data: T
) {
    /**
     * Gets a new [EventHolder] with the new (given) data. Account and conference IDs are copied over.
     * @return A new instance of the [EventHolder]
     */
    fun <N> copy(newData: N): EventHolder<N> = EventHolder(accountId, conferenceId, newData)
}
