package com.bria.common.controller.contacts.buddy.repository

import android.arch.persistence.room.ColumnInfo
import android.arch.persistence.room.Entity
import android.arch.persistence.room.Ignore
import android.arch.persistence.room.PrimaryKey
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import com.bria.common.util.Log

@Entity(tableName = "buddies")
data class BuddyEntity(
        @PrimaryKey
        @ColumnInfo(name = "key") val key: String,
        @ColumnInfo(name = "firstName") val firstName: String,
        @ColumnInfo(name = "lastName") val lastName: String,
        @ColumnInfo(name = "formattedName") val formattedName: String,
        @ColumnInfo(name = "type") val type: EBuddyType,
        @ColumnInfo(typeAffinity = ColumnInfo.BLOB) val image: ByteArray,
        @ColumnInfo(name = "organization") val organization: String,
        @ColumnInfo(name = "email") val email: String,
        @ColumnInfo(name = "url") val url: String,
        @ColumnInfo(name = "isActive") val isActive: Boolean
) {


    /**
     * from "XEP-0153: vCard-Based Avatars": The image height and width SHOULD be between thirty-two (32) and ninety-six (96) pixels; the
     * recommended size is sixty-four (64) pixels high and sixty-four (64) pixels wide.
     */
    companion object {
        @Ignore
        val TAG = "BuddyEntity"
        @Ignore
        val DESIRED_AVATAR_DIM = 96
    }

    @Ignore
    var phones: List<PhoneEntity> = emptyList()
    @Ignore
    var avatar: Bitmap? = null

    init {
        extractAvatar()
    }

    private fun extractAvatar() {
        try {
            if (image.isNotEmpty() && avatar != null) {
                val opts = BitmapFactory.Options()
                BitmapFactory.Options().inJustDecodeBounds = true
                opts.inJustDecodeBounds = true
                BitmapFactory.decodeByteArray(image, 0, image.size, opts)

                var scale = 1
                if (opts.outHeight > DESIRED_AVATAR_DIM || opts.outWidth > DESIRED_AVATAR_DIM) {
                    val heightRatio = opts.outHeight / DESIRED_AVATAR_DIM + if (opts.outHeight % DESIRED_AVATAR_DIM > 0) 1 else 0
                    val widthRatio = opts.outWidth / DESIRED_AVATAR_DIM + if (opts.outWidth % DESIRED_AVATAR_DIM > 0) 1 else 0
                    scale = if (heightRatio > widthRatio) heightRatio else widthRatio
                    Log.i(TAG, "xmpp avatar dimensions are too large w:" + opts.outWidth + ", h:" + opts.outHeight
                            + "; new scale" + scale)
                }

                opts.inSampleSize = scale
                opts.inJustDecodeBounds = false
                avatar = BitmapFactory.decodeByteArray(image, 0, image.size, opts)
            }
        } catch (e: Exception) {
            Log.e(TAG, "error while parsing data")
        }
    }
}

enum class EBuddyType {
    SIP, XMPP, HARDWIRED
}

