package com.bria.common.controller.im.refactoring.adapter

import android.content.Context
import android.graphics.PorterDuff
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.R
import com.bria.common.controller.im.refactoring.dataprovider.FilePickerData
import com.bria.common.controller.im.refactoring.dataprovider.FilePickerDataProvider
import com.bria.common.controller.im.refactoring.filetransfer.FileTransferUtils
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bumptech.glide.RequestManager
import com.bumptech.glide.request.RequestOptions
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable

class FilePickerListAdapter(val dataProvider: FilePickerDataProvider, val options: RequestOptions, val glide: RequestManager,
                            val eventsListener: IOnRecyclerViewItemEventsListener?, val dataProviderEvents : IDataProviderEvents,
                            val context: Context) : RecyclerView.Adapter<FilePickerListAdapter.ViewHolder>() {

    var fileList: MutableList<FilePickerData> = ArrayList<FilePickerData>()
    private val mDisposable = CompositeDisposable()

    init {
        subscribeOnProvide()
    }

    private fun subscribeOnProvide() {
        mDisposable.clear()
        dataProvider.source.observeOn(AndroidSchedulers.mainThread()).subscribe(object : Observer<List<FilePickerData>> {

            override fun onComplete() {}

            override fun onSubscribe(d: Disposable) {
                mDisposable.add(d)
            }

            override fun onError(e: Throwable) {}

            override fun onNext(t: List<FilePickerData>) {
                fileList = t.toMutableList()
                dataProviderEvents.onListLoaded(t.size)
                notifyDataSetChanged()
            }
        })
        dataProvider.reload()
    }

    override fun onBindViewHolder(holder: FilePickerListAdapter.ViewHolder, position: Int) {
        val item = fileList[position]
        holder.apply {
            this.selected.visibility = if (dataProvider.isSelected(item)) View.VISIBLE else View.GONE
            this.name.text = item.name
            if (item.type == FilePickerData.FILE) {
                @Suppress("DEPRECATION")
                holder.image.setColorFilter(context.resources.getColor(R.color.primary_light), PorterDuff.Mode.SRC_ATOP)
                this.size.text = FileTransferUtils.readableFileSize(item.size)
                this.image.setImageResource(R.drawable.ic_file_24dp)
            } else if (item.type == FilePickerData.DIRECTORY) {
                @Suppress("DEPRECATION")
                holder.image.setColorFilter(context.resources.getColor(R.color.primary_accent), PorterDuff.Mode.SRC_ATOP)
                this.size.text = context.resources.getQuantityString(R.plurals.items, item.childrenCount, item.childrenCount)
                this.image.setImageResource(R.drawable.ic_folder_black_24dp)
            }
        }

    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): FilePickerListAdapter.ViewHolder {
        val v: View = LayoutInflater.from(parent.context).inflate(R.layout.file_picker_item, parent, false)
        val vh = FilePickerListAdapter.ViewHolder(v, context)
        v.setOnClickListener {
            val pos = vh.adapterPosition
            if (pos != RecyclerView.NO_POSITION && eventsListener != null) {
                eventsListener.onItemClick(v, pos)
            }
        }
        v.setOnLongClickListener {
            val pos = vh.adapterPosition
            if (pos != RecyclerView.NO_POSITION && eventsListener != null) {
                eventsListener.onItemLongClick(v, pos)
            }
            true
        }

        return vh
    }

    override fun getItemCount(): Int = fileList.size

    fun getSelectedCount() = dataProvider.getSelectedCount()

    fun toggleSelectionByIndex(index: Int) {
        val item = fileList[index]
        dataProvider.toggleSelection(item)
        notifyItemChanged(index)
    }

    class ViewHolder(itemView: View, val context: Context) : RecyclerView.ViewHolder(itemView) {
        val bigMargin = context.resources!!.getDimensionPixelSize(R.dimen.spacing_small)
        val smallMargin = context.resources!!.getDimensionPixelSize(R.dimen.spacing_nano)
        val image: ImageView = itemView.findViewById(R.id.file_picker_item_image)
        val selected: FrameLayout = itemView.findViewById(R.id.file_picker_item_selected_container)
        val name: TextView = itemView.findViewById(R.id.file_picker_item_name)
        val size: TextView = itemView.findViewById(R.id.file_picker_item_size)
    }

    fun reload() {
        dataProvider.reload()
    }
}
