package com.bria.common.controller.im.refactoring.adapter

import android.app.Activity
import android.content.Context
import android.graphics.Bitmap
import android.graphics.PorterDuff
import android.graphics.drawable.BitmapDrawable
import android.support.annotation.IntDef
import android.support.v4.content.ContextCompat
import android.support.v7.util.DiffUtil
import android.support.v7.util.ListUpdateCallback
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.util.Patterns
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import com.bria.common.R
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.ClientConfig
import com.bria.common.controller.Controllers
import com.bria.common.controller.collaboration.utils.matcher.MatcherKey
import com.bria.common.controller.collaboration.utils.matcher.OnParticipantMatchedListener
import com.bria.common.controller.collaboration.utils.matcher.ParticipantMatch
import com.bria.common.controller.collaboration.utils.matcher.ParticipantMatcher
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.articles.ArticleResult
import com.bria.common.controller.im.articles.OnArticleExtractedListener
import com.bria.common.controller.im.refactoring.BuddyKeyUtils
import com.bria.common.controller.im.refactoring.ChatApi
import com.bria.common.controller.im.refactoring.dataprovider.InstantMessageDiffUtils
import com.bria.common.controller.im.refactoring.dataprovider.RxInstantMessageDataProvider
import com.bria.common.controller.im.refactoring.db.entities.*
import com.bria.common.modules.glide.GlideApp
import com.bria.common.uiframework.anyncbitmap.AbstractImageLoader
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bria.common.uireusable.datatypes.MessageListItemData
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_COMPOSING
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_DATE_DIVIDER
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_INCOMING
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_INCOMING_FILE_TRANSFER
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_INCOMING_IMAGE_TRANSFER
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_LOCAL
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_OUTGOING
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_OUTGOING_FILE_TRANSFER
import com.bria.common.uireusable.datatypes.MessageListItemData.Companion.MESSAGE_TYPE_OUTGOING_IMAGE_TRANSFER
import com.bria.common.util.AndroidUtils
import com.bria.common.util.AndroidUtils.Screen.TAB_PORT
import com.bria.common.util.LinkifyUtils
import com.bria.common.util.Log
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class RxInstantMessageListAdapter(val context: Context,
                                  var mImageLoader: AbstractImageLoader,
                                  private var mMatcher: ParticipantMatcher,
                                  val eventsListener: IOnRecyclerViewItemEventsListener?,
                                  internal var rxInstantMessageDataProvider: RxInstantMessageDataProvider,
                                  val screenCallback: IInstantMessageListAdapterCallback)
    : RecyclerView.Adapter<RxInstantMessageListAdapter.InstantMessageViewHolder>(),
        OnParticipantMatchedListener,
        OnArticleExtractedListener {

    private var mDefaultAvatar: Bitmap? = null
    @OutgoingBubbleColor
    private var outgoingBubbleColor: Int = 0

    private val mDisposable = CompositeDisposable()

    private var list = ArrayList<MessageListItemData>()

    private var visibleRange: Pair<Int, Int>? = null

    private var parseHyperlinks = true

    private val mLock = Any()

    init {
        mMatcher.setListener(this)
        rxInstantMessageDataProvider.articleExtractor.setListener(this)
        subscribeOnProvide()
    }

    private fun subscribeOnProvide() {
        rxInstantMessageDataProvider.source.debounce(50, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.computation())
                .map { Pair<ArrayList<MessageListItemData>, DiffUtil.DiffResult>(it, DiffUtil.calculateDiff(InstantMessageDiffUtils(list, it))) }
                .observeOn(AndroidSchedulers.mainThread()).subscribe(object : Observer<Pair<List<MessageListItemData>, DiffUtil.DiffResult>> {
                    override fun onSubscribe(d: Disposable) {
                        mDisposable.add(d)
                    }

                    override fun onNext(pair: Pair<List<MessageListItemData>, DiffUtil.DiffResult>) {
                        synchronized(mLock) {
                            //Log.d(TAG, list.joinToString(separator = "\n") { it.message })
                            val scrollToTheBottom = shouldScrollToTheBottom(list, pair.first)
                            val hasMoreMessagesBellow = !scrollToTheBottom && hasMoreMessagesBellow(pair.first)
                            list.clear()
                            list.addAll(pair.first)
                            outgoingBubbleColor = if (rxInstantMessageDataProvider.hardwiredSession) RxInstantMessageListAdapter.BUBBLE_COLOR_BLUE
                            else RxInstantMessageListAdapter.BUBBLE_COLOR_GRAY
                            if (parseHyperlinks) {
                                list.filter { !it.checkedForArticle }.forEach { parseArticle(it) }
                            }
                            pair.second.dispatchUpdatesTo(updateCallBack)
                            var desiredPosition: Int? = null
                            if (scrollToTheBottom) {
                                desiredPosition = list.size - 1
                            }
                            screenCallback.setMoreMessagesVisibility(hasMoreMessagesBellow)
                            Log.d(TAG, "List updated")
                            screenCallback.listUpdated(desiredPosition)
                        }
                    }

                    override fun onError(e: Throwable) {
                        Log.d(TAG, "onError: " + e.printStackTrace())
                    }

                    override fun onComplete() {
                        Log.d(TAG, "onComplete:")
                    }
                })
    }


    /*
     * We should not scroll to the bottom if has previous list of the same conversation and you were up in the list
     */
    fun shouldScrollToTheBottom(oldList: List<MessageListItemData>, newList: List<MessageListItemData>): Boolean {

        val newMessageArrived = (oldList.isEmpty() && newList.isNotEmpty()) || (oldList.isNotEmpty() && newList.isNotEmpty()
                && oldList[oldList.size - 1].time < newList[newList.size - 1].time)
        val listChanged = oldList.isEmpty() || newList.isEmpty() || oldList[0].conversationId != newList[0].conversationId
        val somewhereUp = visibleRange != null && visibleRange!!.second != -1 && (visibleRange!!.second < oldList.size - 5)
        return listChanged || (newMessageArrived && !somewhereUp)
    }

    fun hasMoreMessagesBellow(newList: List<MessageListItemData>): Boolean {
        if (visibleRange == null || visibleRange!!.second == -1 || visibleRange!!.second >= newList.size - 1) return false
        return IntRange(start = visibleRange!!.second, endInclusive = newList.size - 1).any {
            val m = newList[it]
            return if (m.isTransfer()) {
                m.isFileTransferUnread()
            } else {
                !newList[it].isRead
            }
        }
    }

    val updateCallBack: ListUpdateCallback = object : ListUpdateCallback {
        override fun onChanged(position: Int, count: Int, payload: Any?) {
            Log.d(TAG, "notifyItemRangeChanged(pos:$position, count $count, payload $payload)")
            this@RxInstantMessageListAdapter.notifyItemRangeChanged(position, count)
        }

        override fun onMoved(fromPosition: Int, toPosition: Int) {
            Log.d(TAG, "notifyItemMoved(fromPosition:$fromPosition, toPosition $toPosition)")
            this@RxInstantMessageListAdapter.notifyItemMoved(fromPosition, toPosition)
        }

        override fun onInserted(position: Int, count: Int) {
            Log.d(TAG, "notifyItemRangeInserted(pos:$position, count $count)")
            this@RxInstantMessageListAdapter.notifyItemRangeInserted(position, count)
        }

        override fun onRemoved(position: Int, count: Int) {
            Log.d(TAG, "notifyItemRangeRemoved(pos:$position, count $count)")
            this@RxInstantMessageListAdapter.notifyItemRangeRemoved(position, count)
        }

    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RxInstantMessageListAdapter.InstantMessageViewHolder {
        //setup clicks
        val item: Int = when (viewType) {
            MESSAGE_TYPE_INCOMING -> R.layout.im_message_item_left_group
            MESSAGE_TYPE_OUTGOING -> R.layout.im_message_item_right_group
            MESSAGE_TYPE_LOCAL -> R.layout.im_message_item_local
            MESSAGE_TYPE_COMPOSING -> R.layout.im_message_item_composing
            MESSAGE_TYPE_DATE_DIVIDER -> R.layout.im_message_item_date
            MESSAGE_TYPE_INCOMING_FILE_TRANSFER -> R.layout.im_message_item_transfer_file_left
            MESSAGE_TYPE_OUTGOING_FILE_TRANSFER -> R.layout.im_message_item_transfer_file_right
            MESSAGE_TYPE_INCOMING_IMAGE_TRANSFER -> R.layout.im_message_item_transfer_image_left
            MESSAGE_TYPE_OUTGOING_IMAGE_TRANSFER -> R.layout.im_message_item_transfer_image_right
            else -> R.layout.im_message_item_left_group
        }

        val v = LayoutInflater.from(parent.context).inflate(item, parent, false)
        val vh = InstantMessageViewHolder(v)
        v.setOnClickListener {
            val pos = vh.adapterPosition
            if (pos != RecyclerView.NO_POSITION && eventsListener != null) {
                eventsListener.onItemClick(v, pos)
            }
        }
        v.setOnLongClickListener {
            val pos = vh.adapterPosition
            if (pos != RecyclerView.NO_POSITION && eventsListener != null) {
                eventsListener.onItemLongClick(v, pos)
            }
            true
        }
        return vh
    }

    override fun onBindViewHolder(holder: InstantMessageViewHolder, position: Int) {
        val item = getItemAt(position)
        holder.messageType = item.msgType
        // timestamp has no article container
        when {
            item.msgType == MessageListItemData.MESSAGE_TYPE_INCOMING -> updateIncomingTextMessage(holder, item, position)
            item.msgType == MessageListItemData.MESSAGE_TYPE_OUTGOING -> updateOutgoingTextMessage(holder, item, position)
            item.msgType == MessageListItemData.MESSAGE_TYPE_LOCAL -> holder.message!!.text = formatLocalMessage(item.message)
            item.msgType == MessageListItemData.MESSAGE_TYPE_DATE_DIVIDER -> holder.message!!.text = item.message
            item.msgType == MessageListItemData.MESSAGE_TYPE_OUTGOING_FILE_TRANSFER
                    || item.msgType == MessageListItemData.MESSAGE_TYPE_INCOMING_FILE_TRANSFER -> updateFileTransferMessage(item, holder)
            item.msgType == MessageListItemData.MESSAGE_TYPE_INCOMING_IMAGE_TRANSFER
                    || item.msgType == MessageListItemData.MESSAGE_TYPE_OUTGOING_IMAGE_TRANSFER -> updateFileTransferMessage(item, holder)
        }
        holder.overlay?.visibility = if (item.selected) View.VISIBLE else View.GONE
        linkify(item.msgType, holder.message!!)
    }

    private fun updateFileTransferMessage(item: MessageListItemData, viewHolder: InstantMessageViewHolder) {
        Log.d(TAG, "updateFileTransfer status: " + item.getStatusDescription())
        // progress optimisation
//        if (viewHolder.bubble.tag != null && (viewHolder.bubble.tag as Long) == item.id
//                && item.isInProgress() && item.oldMessage?.isInProgress() == true) {
//            Log.d(TAG, "Updating progress only: " + item.fileTransferProgress)
//            viewHolder.acceptDeclineContainer?.visibility = View.GONE
//            viewHolder.progressBar?.visibility = View.VISIBLE
//            viewHolder.statusIcon?.visibility = View.VISIBLE
//            viewHolder.statusText?.visibility = View.VISIBLE
//            viewHolder.progressBar?.progress = item.fileTransferProgress
//            return
//        }
        viewHolder.bubble?.tag = item.id
        viewHolder.fileName?.text = item.fileName
        viewHolder.fileSize?.text = context.getString(R.string.tFileSize, item.fileSize)
        if (item.isImageTransfer()) {
            GlideApp.with(context as Activity).load(item.filePath).centerCrop().placeholder(R.color.gray_light).into(viewHolder.image)
        } else {
            viewHolder.image?.setImageResource(item.getIconByFileType())
            @Suppress("DEPRECATION")
            viewHolder.image?.setColorFilter(context.resources.getColor(R.color.white), PorterDuff.Mode.SRC_ATOP)
        }
        viewHolder.statusIcon?.apply {
            if (item.isStatusIconVisible()) {
                viewHolder.statusIcon?.setImageResource(item.getStatusIcon())
                @Suppress("DEPRECATION")
                this.setColorFilter(context.resources.getColor(if (item.isRejected()) R.color.red else R.color.white), PorterDuff.Mode.SRC_ATOP)
                this.visibility = View.VISIBLE
            } else {
                this.visibility = View.GONE
            }
        }
        viewHolder.statusText?.apply {
            if (item.isStatusTextVisible()) {
                this.text = item.getStatusText()
                this.visibility = View.VISIBLE
            } else {
                this.visibility = View.GONE
            }
        }
        viewHolder.progressBar?.apply {
            if (item.isProgressBarVisible()) {
                viewHolder.progressBar?.progress = item.fileTransferProgress
                this.visibility = View.VISIBLE
            } else {
                this.visibility = View.GONE
            }
        }
        viewHolder.bottomStatus?.apply {
            if (item.isBottomStatusVisible()) {
                this.text = item.getStatusBottomText()
                @Suppress("DEPRECATION")
                this.setTextColor(context.resources.getColor(if (item.isOutgoingFailToSend()) R.color.red else R.color.white))
                this.visibility = View.VISIBLE
            } else {
                this.visibility = View.GONE
            }
        }

        viewHolder.acceptDeclineContainer?.apply {
            if (item.isAcceptDeclineVisible()) {
                this.visibility = View.VISIBLE
                screenCallback.checkWriteStoragePermission()
                @Suppress("DEPRECATION")
                viewHolder.acceptButton?.setColorFilter(context.resources.getColor(R.color.white), PorterDuff.Mode.SRC_ATOP)
                @Suppress("DEPRECATION")
                viewHolder.declineButton?.setColorFilter(context.resources.getColor(R.color.white), PorterDuff.Mode.SRC_ATOP)
            } else {
                this.visibility = View.GONE
            }
        }

        viewHolder.transferActionInfo?.apply {
            if (item.isTransferActionInfoVisible()) {
                this.visibility = View.VISIBLE
            } else {
                this.visibility = View.GONE
            }
        }

        viewHolder.time?.apply {
            if (item.isTimeVisible()) {
                viewHolder.time?.text = item.timeString
                this.visibility = View.VISIBLE
            } else {
                this.visibility = View.GONE
            }
        }
    }

    private fun updateOutgoingTextMessage(viewHolder: InstantMessageViewHolder, item: MessageListItemData, position: Int) {
        var messageText = item.message
        if (item.isVccsInviteLink) {
            messageText = context.getString(R.string.tJoinMyConference, item.message) ?: item.message
        }
        viewHolder.message!!.text = messageText
        viewHolder.time!!.text = item.timeString
        val previousImData = if (position > 0) getItemAt(position - 1) else null
        if (previousImData != null && previousImData.msgType == MESSAGE_TYPE_OUTGOING) {
            viewHolder.setSmallerTopMargin(true)
        } else {
            viewHolder.setSmallerTopMargin(false)
        }

        val msgColor = if (item.error) R.color.red else R.color.white
        @Suppress("DEPRECATION")
        viewHolder.message!!.setTextColor(context.resources!!.getColor(msgColor))

        @Suppress("DEPRECATION")
        viewHolder.message!!.setLinkTextColor(context.resources!!.getColor(msgColor))
        setOutgoingBackgroundColor(viewHolder)
        updateArticle(item, viewHolder)
    }

    private fun setOutgoingBackgroundColor(viewHolder: InstantMessageViewHolder) {
        viewHolder.bubble?.background = ContextCompat.getDrawable(context, if (outgoingBubbleColor == BUBBLE_COLOR_GRAY)
            R.drawable.im_panel_right_briax_bkg else R.drawable.im_panel_right_bkg)
    }

    private fun updateIncomingTextMessage(viewHolder: InstantMessageViewHolder, item: MessageListItemData, position: Int) {
        var messageText = item.message
        if (item.isVccsInviteLink) {
            messageText = context.getString(R.string.tJoinMyConference, item.message)
        }
        viewHolder.message?.text = messageText
        viewHolder.time?.text = item.timeString
        val previousImData = if (position > 0) getItemAt(position - 1) else null

        if (previousImData != null && previousImData.msgType == MESSAGE_TYPE_INCOMING
                && TextUtils.equals(item.jid, previousImData.jid)) {
            viewHolder.participantName?.visibility = View.GONE
            viewHolder.avatar?.visibility = View.INVISIBLE
            viewHolder.avatar?.layoutParams?.height = 1
            viewHolder.setSmallerTopMargin(true)
        } else {
            viewHolder.participantName?.visibility = View.VISIBLE
            viewHolder.avatar?.visibility = View.VISIBLE

            viewHolder.avatar?.layoutParams?.height = viewHolder.avatar?.layoutParams?.width
            viewHolder.setSmallerTopMargin(false)

            val key = MatcherKey()
            key.xmppJid = item.jid

            if (item.isSip) key.sipNumber = ImpsUtils.removeDomainFromAddress(item.jid)

            val details = mMatcher.getMatchedInfo(key)

            var name: String?
            if (details != null) {
                name = details.displayName
                if (name == item.jid && !TextUtils.isEmpty(item.displayName)) {
                    name = item.displayName
                }
                viewHolder.setAvatar(details.photo)
            } else {
                name = item.participantName
                viewHolder.setAvatar(item.avatar)
                mMatcher.loadParticipantInfo(key, item.jid)
            }
            name = ImpsUtils.removeDomainFromAddress(name)
            viewHolder.participantName?.text = name
        }
        updateArticle(item, viewHolder)
    }

    private val isToSmallToShowPhoto: Boolean
        get() = AndroidUtils.getDeviceType(context) == TAB_PORT

    private fun formatLocalMessage(rawMessage: String): String {
        val separatorPosition = rawMessage.indexOf(ChatApi.LOCAL_MESSAGE_SEPARATOR)
        if (separatorPosition == -1) {
            return rawMessage
        }
        val eventType = rawMessage.substring(0, separatorPosition)
        val buddyKey = rawMessage.substring(separatorPosition + 1)
        val buddy = Controllers.get().buddy.getBuddyByNewKey(buddyKey)
        var displayName = buddyKey
        if (buddy != null) {
            displayName = buddy.displayName
        } else {
            val address = BuddyKeyUtils.getImAdressFromNewBuddyKey(buddyKey)
            if (address != null) {
                displayName = ImpsUtils.getUsernameFromAddress(address)
            }
        }
        return if (TextUtils.isEmpty(eventType)) {
            rawMessage
        } else {
            when (eventType) {
                ChatApi.PARTICIPANT_JOINED_EVENT_ID ->
                    context.getString(R.string.tParticipantJoined, displayName) ?: displayName+"has joined the Group Chat"
                ChatApi.PARTICIPANT_LOCAL_JOINED_EVENT_ID ->
                    context.getString(R.string.tYouWereInvitedToChatRoom)
                ChatApi.PARTICIPANT_LEFT_EVENT_ID ->
                    context.getString(R.string.tParticipantLeft, displayName) ?: displayName+"has left the Group Chat"
                ChatApi.PARTICIPANT_ADDED ->
                    context.getString(R.string.tAddedParticipant, displayName) ?: "Added $displayName"
                else -> rawMessage
            }
        }
    }

    private fun updateArticle(item: MessageListItemData, viewHolder: InstantMessageViewHolder) {
        if (item.containsArticle() && parseHyperlinks) {
            viewHolder.articleContainer!!.visibility = View.VISIBLE
            viewHolder.articleTitle!!.text = item.articleTitle
            viewHolder.articleDetails!!.text = item.articleDetails

            if (TextUtils.isEmpty(item.articleDetails)) {
                viewHolder.articleDetails!!.visibility = View.GONE
            } else {
                viewHolder.articleDetails!!.visibility = View.VISIBLE
            }

            val wrapContent = TextUtils.isEmpty(item.articleDetails) || item.articleDetails?.length ?: 0 < 80
            adaptViewSize(viewHolder.bubble!!, wrapContent)

            if (item.articlePhotoUrl == null || isToSmallToShowPhoto) {
                viewHolder.articlePhoto!!.visibility = View.GONE
            } else {
                viewHolder.articlePhoto!!.visibility = View.VISIBLE
                mImageLoader.loadImage(item.articlePhotoUrl, item.articlePhotoUrl?.hashCode(), viewHolder.articlePhoto)
            }
        } else if (viewHolder.articleContainer != null) { // timestamp has no article container
            viewHolder.articleContainer!!.visibility = View.GONE

            adaptViewSize(viewHolder.bubble!!, true)
        }
    }

    private fun adaptViewSize(view: View, wrapContent: Boolean) {
        val params = view.layoutParams as LinearLayout.LayoutParams
        if (!wrapContent) {
            params.width = context.resources.getDimensionPixelSize(R.dimen.message_max_width)
        } else {
            params.width = LinearLayout.LayoutParams.WRAP_CONTENT
        }
    }

    /**
     * Adds links to the target TextView.
     *
     * @param messageType Which message type is currently being updated
     * @param target Where to add the links
     */
    private fun linkify(messageType: Int, target: TextView) {
        if (LinkifyUtils.isLinksSupportedFor(context, messageType)) LinkifyUtils.addLinks(target)
    }

    override fun getItemCount(): Int {
        synchronized(mLock) {
            return list.size
        }
    }

    fun getItemAt(position: Int): MessageListItemData {
        synchronized(mLock) {
            return list[position]
        }
    }

    override fun getItemViewType(position: Int): Int = getItemAt(position).msgType

    override fun getItemId(position: Int): Long = getItemAt(position).id

    private val circleDefaultAvatar: Bitmap?
        get() {
            if (mDefaultAvatar == null) {
                val photo = AndroidUtils.getDrawable(context, R.drawable.default_avatar)
                mDefaultAvatar = Controllers.get().image.cropToCircle((photo as BitmapDrawable).bitmap)
            }
            return mDefaultAvatar
        }


    /* Participant matcher listener implementation goes below this line */
    override fun onParticipantMatched(matched: ParticipantMatch) {
        if (ClientConfig.get().isDebugMode) {
            Log.d(TAG, "Matched: " + matched.toString())
        }
        notifyDataSetChanged() //nvtd try better
    }

    fun cleanup() {
        mDisposable.clear()
        rxInstantMessageDataProvider.articleExtractor.setListener(null)
        //super.cleanup()
    }

    @IntDef(BUBBLE_COLOR_BLUE, BUBBLE_COLOR_GRAY)
    @Retention(AnnotationRetention.SOURCE)
    internal annotation class OutgoingBubbleColor

    fun setCurrentVisibleMessages(firstVisibleItemPosition: Int, lastVisibleItemPosition: Int) {
        visibleRange = Pair(firstVisibleItemPosition, lastVisibleItemPosition)
        rxInstantMessageDataProvider.setVisibleMessages(firstVisibleItemPosition, lastVisibleItemPosition)
    }

    fun toggleSelectionByIndex(index: Int) {
        getItemAt(index).apply {
            if (this.msgType == MESSAGE_TYPE_OUTGOING || this.msgType == MESSAGE_TYPE_INCOMING || this.isTransfer()) {
                toggleSelection(this.id)
            }
        }
        notifyItemChanged(index)
    }

    private fun toggleSelection(id: Long) {
        rxInstantMessageDataProvider.toggleSelection(id)
    }

    fun isSelected(id: Long): Boolean = rxInstantMessageDataProvider.isSelected(id)

    //fun getSelectedMessages(): List<InstantMessageData> = rxInstantMessageDataProvider.getSelectedMessages()

    fun clearSelection() {
        rxInstantMessageDataProvider.clearSelection()
        notifyDataSetChanged()
    }


    fun getSelectedMessageCount(): Int = rxInstantMessageDataProvider.getSelectedMessagesCount()

    companion object {
        const val BUBBLE_COLOR_BLUE = 0 // used by default
        const val BUBBLE_COLOR_GRAY = 1
        private const val TAG = "RxInstantMessageListAdapter"
    }

    fun loadMoreMessages() {
        rxInstantMessageDataProvider.loadMoreMessages()
    }

    private fun parseArticle(messageListItemData: MessageListItemData) {
        val messageText = messageListItemData.message
        val messageId = messageListItemData.id.toString()
        val articleId = """$messageId${BuddyKeyUtils.SMALL_SEPARATOR}$messageText"""
        val cached = rxInstantMessageDataProvider.articleExtractor.getArticleInfo(articleId)
        if (cached != null) {
            messageListItemData.addArticleResult(cached)
        }
        if (!messageListItemData.checkedForArticle) {
            messageListItemData.checkedForArticle = true
            val urlMatcher = Patterns.WEB_URL.matcher(messageText)
            var url: String? = null
            if (urlMatcher.find()) {
                Log.d(TAG, "Article extraction: found URL")
                url = messageText.substring(urlMatcher.start(0), urlMatcher.end(0))
                url = LinkifyUtils.transformUrl(url)
            }
            if (!TextUtils.isEmpty(url)) {
                rxInstantMessageDataProvider.articleExtractor.loadArticleData(url!!, articleId)
            }
        }
    }


    override fun onArticleExtracted(result: ArticleResult) {
        if (result.isSuccess) {
            val parts = result.messageId.split(BuddyKeyUtils.REGEX_SMALL_SEPARATOR.toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
            var idStr: String = ""
            if (parts.size > 1) {
                idStr = parts[0]
            }
            if (TextUtils.isEmpty(idStr)) {
                return
            }
            try {
                synchronized(mLock) {
                    val id = java.lang.Long.parseLong(idStr)
                    val messagePos = list.indices.firstOrNull { list[it].id == id }
                    if (messagePos != null) {
                        val message = list[messagePos]
                        message.addArticleResult(result)
                        this.notifyItemChanged(messagePos)
                    }
                }
            } catch (ex: NumberFormatException) {
                Log.e(TAG, "MessageId, not Correct")
            }
        }
    }

    inner class InstantMessageViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        var bubble: ViewGroup? = itemView.findViewById(R.id.im_message_item_bubble)
        var message: TextView? = itemView.findViewById(R.id.im_message_item_message_text)
        var avatar: ImageView? = itemView.findViewById(R.id.im_message_item_avatar)
        var participantName: TextView? = itemView.findViewById(R.id.im_message_item_participant_name)
        var time: TextView? = itemView.findViewById(R.id.im_message_item_message_time)


        var articleContainer: ViewGroup? = itemView.findViewById(R.id.message_article_container)
        var articleTitle: TextView? = itemView.findViewById(R.id.message_article_title)
        var articleDetails: TextView? = itemView.findViewById(R.id.message_article_details)
        var articlePhoto: ImageView? = itemView.findViewById(R.id.message_article_photo)

        var overlay: View? = itemView.findViewById(R.id.im_message_item_overlay)

        var messageType: Int = 0

        //file transfer
        var acceptDeclineContainer: ViewGroup? = itemView.findViewById(R.id.im_message_item_accept_decline)
        var acceptButton: ImageView? = itemView.findViewById(R.id.im_message_item_accept_image)
        private var acceptText: TextView? = itemView.findViewById(R.id.im_message_item_accept_text)
        var declineButton: ImageView? = itemView.findViewById(R.id.im_message_item_decline_image)
        private var declineText: TextView? = itemView.findViewById(R.id.im_message_item_decline_text)
        var image: ImageView? = itemView.findViewById(R.id.im_message_item_image)
        var fileName: TextView? = itemView.findViewById(R.id.im_message_item_file_name)
        var fileSize: TextView? = itemView.findViewById(R.id.im_message_item_file_size)
        var statusIcon: ImageView? = itemView.findViewById(R.id.im_message_item_image_status_icon)
        var statusText: TextView? = itemView.findViewById(R.id.im_message_item_image_status_text)
        var progressBar: ProgressBar? = itemView.findViewById(R.id.im_message_item_file_progress)
        var bottomStatus: TextView? = itemView.findViewById(R.id.im_message_item_file_bottom_status)
        var transferActionInfo: TextView? = itemView.findViewById(R.id.im_message_item_transfer_action_info)


        private val normalMargin = context.resources!!.getDimensionPixelSize(R.dimen.spacing_small)
        private val smallMargin = context.resources!!.getDimensionPixelSize(R.dimen.spacing_micro)

        fun clickIfNotInSelectionMode(clickHandler: () -> Unit) {
            if (rxInstantMessageDataProvider.getSelectedMessagesCount() != 0) {
                toggleSelectionByIndex(adapterPosition)
            } else {
                clickHandler()
            }
        }

        init {
            if (bubble != null) {
                message!!.setOnLongClickListener {
                    eventsListener?.onItemLongClick(itemView, adapterPosition)
                    true
                }
            }

            bubble?.setOnClickListener {
                clickIfNotInSelectionMode {
                    openFileOnClick()
                }
            }

            image?.setOnClickListener {
                clickIfNotInSelectionMode {
                    openFileOnClick()
                }
            }

            fileName?.setOnClickListener {
                clickIfNotInSelectionMode {
                    openFileOnClick()
                }
            }
            fileSize?.setOnClickListener {
                clickIfNotInSelectionMode {
                    openFileOnClick()
                }
            }

            acceptButton?.setOnClickListener {
                clickIfNotInSelectionMode {
                    BIAnalytics.get().reportUIClick("file-transfer-receive")
                    rxInstantMessageDataProvider.accept(list[adapterPosition])
                }
            }
            declineButton?.setOnClickListener {
                clickIfNotInSelectionMode {
                    rxInstantMessageDataProvider.reject(list[adapterPosition])
                }
            }
            acceptText?.setOnClickListener {
                clickIfNotInSelectionMode {
                    BIAnalytics.get().reportUIClick("file-transfer-receive")
                    rxInstantMessageDataProvider.accept(list[adapterPosition])
                }
            }
            declineText?.setOnClickListener {
                clickIfNotInSelectionMode {
                    rxInstantMessageDataProvider.reject(list[adapterPosition])
                }
            }
            statusIcon?.setOnClickListener {
                clickIfNotInSelectionMode {
                    fileStatusClicked()
                }
            }

            statusText?.setOnClickListener {
                clickIfNotInSelectionMode {
                    fileStatusClicked()
                }
            }
        }

        private fun fileStatusClicked() {
            var handled = false
            val item = list[adapterPosition]
            when {
                item.isOutgoingFailToSend() -> {
                    if (screenCallback.canSendMessage()) {
                        rxInstantMessageDataProvider.retry(item)
                        handled = true
                    }
                }
                item.isInProgress() -> {
                    rxInstantMessageDataProvider.reject(item)
                    handled = true
                }
            }
            if (!handled) {
                openFileOnClick()
            }
        }

        private fun openFileOnClick() {

            list[adapterPosition].let {
                if (it.isTransfer() && (it.isOutgoingFT() || it.isIncomingReceived())) {
                    screenCallback.openFile(it.filePath)
                }
            }
        }

        fun setAvatar(newAvatar: Bitmap?) {
            avatar?.setImageBitmap(newAvatar ?: circleDefaultAvatar)
        }

        fun setSmallerTopMargin(smaller: Boolean) {
            if (itemView.layoutParams is RecyclerView.LayoutParams) {
                val params = itemView.layoutParams as RecyclerView.LayoutParams
                params.setMargins(params.leftMargin, if (smaller) smallMargin else normalMargin, params.rightMargin, params.bottomMargin)
            }
        }
    }

    /*
     * Send screen related events
     */
    interface IInstantMessageListAdapterCallback {
        fun setMoreMessagesVisibility(visible: Boolean)
        fun listUpdated(desiredPosition: Int? = null)
        fun checkWriteStoragePermission(): Boolean
        fun canSendMessage(): Boolean
        fun openFile(filePath: String?)
    }

    fun parseHyperlinks(shouldParseHyperlinks: Boolean) {
        parseHyperlinks = shouldParseHyperlinks
    }
}




