package com.bria.common.controller.im.refactoring.dataprovider

import android.annotation.SuppressLint
import android.content.Context
import android.text.TextUtils
import com.bria.common.controller.im.refactoring.filetransfer.storagetools.getFilenameFromPath
import com.bria.common.controller.im.refactoring.filetransfer.storagetools.getParentPath
import com.bria.common.controller.im.refactoring.filetransfer.storagetools.rootStoragePath
import com.bria.common.uireusable.dataprovider.AbstractDiffDataProvider
import com.bria.common.util.Log
import com.bria.common.util.ThreadExecutors
import java.io.File


class FilePickerDataProvider(val context: Context) : AbstractDiffDataProvider<FilePickerData>() {

    private val TAG = "FilePickerDataProvider"

    var activeFolder : String = context.rootStoragePath()

    override fun reload() {
        loadAllFiles()
    }


    @SuppressLint("CheckResult")
    private fun loadAllFiles() {
        Log.d(TAG, "loadAllFiles")
        ThreadExecutors.scheduleLightWork {
            val items = ArrayList<FilePickerData>()
            val files = File(activeFolder).listFiles()?.filterNotNull()
            if (files != null) {
                for (file in files) {
                    val path = file.absolutePath
                    val name = path.getFilenameFromPath()
                    if (name.startsWith(".")) {
                        continue
                    }
                    val children = file.listFiles()?.filterNotNull()?.size ?: 0
                    val size = if (file.isDirectory) 0 else file.length()
                    val type = if (file.isDirectory) FilePickerData.DIRECTORY else FilePickerData.FILE
                    val filePickerData = FilePickerData(type, name, path, size, children)
                    items.add(filePickerData)
                }
            }
            remotlyChanged.onNext(items)
        }
    }

    fun isInRootState() : Boolean = TextUtils.equals(activeFolder, context.rootStoragePath())
            || TextUtils.equals(activeFolder, context.rootStoragePath() + "/")

    fun moveToRootState() {
        clearSelection()
        activeFolder = context.rootStoragePath()
        reload()
    }

    fun moveToFolder(folder : String) {
        clearSelection()
        activeFolder = folder
        reload()
    }

    fun moveUp() {
        Log.d(TAG, "moveUp old state: $activeFolder" )
        moveToFolder(activeFolder.getParentPath())
        Log.d(TAG, "moveUp new state: $activeFolder" )
    }

    fun getRelativeFolderPath() : String = activeFolder.removePrefix(context.rootStoragePath())
}
