package com.bria.common.controller.im.refactoring.dataprovider

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.PorterDuff
import android.graphics.PorterDuffColorFilter
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.RecyclerView.NO_POSITION
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.bria.common.R
import com.bria.common.controller.im.refactoring.adapter.IDataProviderEvents
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bumptech.glide.RequestBuilder
import com.bumptech.glide.request.RequestOptions
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable


class GalleryListAdapter(val dataProvider: GalleryListDataProvider, val options: RequestOptions, val glide: RequestBuilder<Bitmap>,
                         val eventsListener: IOnRecyclerViewItemEventsListener?, val dataProviderEvents : IDataProviderEvents,  val context: Context)
    : RecyclerView.Adapter<GalleryListAdapter.ViewHolder>() {

    private val mDisposable = CompositeDisposable()
    var galleryList: MutableList<GalleryListData> = ArrayList<GalleryListData>()

    init {
        subscribeOnProvide()
    }

    private fun subscribeOnProvide() {
        mDisposable.clear()
        dataProvider.source.observeOn(AndroidSchedulers.mainThread()).subscribe(object : Observer<List<GalleryListData>> {

            override fun onComplete() {

            }

            override fun onSubscribe(d: Disposable) {
                mDisposable.add(d)
            }

            override fun onError(e: Throwable) {

            }

            override fun onNext(t: List<GalleryListData>) {
                galleryList = t.toMutableList()
                notifyDataSetChanged()
                dataProviderEvents.onListLoaded(t.size)
            }
        })
        dataProvider.reload()
    }

    override fun getItemViewType(position: Int): Int = galleryList[position].type

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bindItems(galleryList.get(position), glide, options)
        val item = galleryList[position]
        val params = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
        if (dataProvider.isSelected(item)) {
            holder.image.setColorFilter(Color.argb(150,200,200,200))
            holder.selected.visibility = View.VISIBLE
            @Suppress("DEPRECATION")
            holder.selected.setColorFilter(context.resources.getColor(R.color.white), PorterDuff.Mode.SRC_ATOP)
            params.setMargins(holder.bigMargin, holder.bigMargin, holder.bigMargin, holder.bigMargin)
        } else {
            holder.image.colorFilter = null
            holder.selected.visibility = View.GONE
            params.setMargins(holder.smallMargin, holder.smallMargin, holder.smallMargin, holder.smallMargin)
        }
        holder.container.layoutParams = params
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val v : View
        if (viewType == GalleryListData.IMAGE) {
            v = LayoutInflater.from(parent.context).inflate(R.layout.gallery_list_image_item, parent, false)
        } else {
            v = LayoutInflater.from(parent.context).inflate(R.layout.gallery_list_album_item, parent, false)
        }
        val vh = ViewHolder(v, context)
        v.setOnClickListener{
            val pos = vh.adapterPosition
            if (pos != NO_POSITION && eventsListener != null) {
                eventsListener.onItemClick(v, pos)
            }
        }
        v.setOnLongClickListener{
            val pos = vh.adapterPosition
            if (pos != NO_POSITION && eventsListener != null) {
                eventsListener.onItemLongClick(v, pos)
            }
            true
        }

        return vh
    }

    override fun getItemCount(): Int = galleryList.size

    fun getSelectedCount() = dataProvider.getSelectedCount()

    fun toggleSelectionByIndex(index: Int) {
        val item = galleryList[index]
        dataProvider.toggleSelection(item)
        notifyItemChanged(index)
    }

    class ViewHolder(itemView: View, val context: Context) : RecyclerView.ViewHolder(itemView) {
        val bigMargin = context.resources!!.getDimensionPixelSize(R.dimen.spacing_small)
        val smallMargin = context.resources!!.getDimensionPixelSize(R.dimen.spacing_nano)
        val image: ImageView = itemView.findViewById(R.id.gallery_list_item_image)
        val selected: ImageView = itemView.findViewById(R.id.gallery_list_item_selected)
        val title: TextView= itemView.findViewById(R.id.gallery_list_item_title)
        val count: TextView= itemView.findViewById(R.id.gallery_list_item_count)
        val container: LinearLayout = itemView.findViewById(R.id.gallery_list_item_container)

        init {
            @Suppress("DEPRECATION")
            selected.drawable?.colorFilter = PorterDuffColorFilter(context.resources.getColor(R.color.white), PorterDuff.Mode.MULTIPLY)
        }

        fun bindItems(listData: GalleryListData, glide: RequestBuilder<Bitmap>, options: RequestOptions) {
            glide.load(listData.path).apply {
                @Suppress("UNUSED_EXPRESSION")
                options
            }.thumbnail(0.4f).into(image)
            if (listData.type == GalleryListData.FOLDER) {
                title.text = listData.name
                count.text = listData.count.toString()
            }
        }
    }

    fun reload() {
        dataProvider.reload()
    }

}
