package com.bria.common.controller.im.refactoring.dataprovider

import android.annotation.SuppressLint
import android.content.Context
import android.database.Cursor
import android.net.Uri
import android.provider.MediaStore
import android.text.TextUtils
import com.bria.common.permission.PermissionHandler.checkPermission
import com.bria.common.uireusable.dataprovider.AbstractDiffDataProvider
import com.bria.common.util.Log
import com.bria.common.util.ThreadExecutors


class GalleryListDataProvider(val context: Context) : AbstractDiffDataProvider<GalleryListData>() {

    private val TAG = "GalleryListDataProvider"

    var activeFolder : String? = ""

    @SuppressLint("CheckResult")
    override fun reload() {
        Log.d(TAG, "loadAllFiles")
        if (checkPermission(context, android.Manifest.permission.READ_EXTERNAL_STORAGE)) {
            ThreadExecutors.scheduleLightWork {
                if (isInRootState()) {
                    getListOfAllImagesFolders()
                } else {
                    loadAllImagesFromFolder(activeFolder)
                }
            }
        }
    }

    private fun loadAllImagesFromFolder(folderName: String?) {

        val uri: Uri = android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI
        val listOfAllImages = ArrayList<GalleryListData>()
        val selectionArgs = arrayOf("%$folderName%")
        val selection = MediaStore.Images.Media.DATA + " like ? "
        val projectionOnlyBucket = arrayOf(MediaStore.MediaColumns.DATA, MediaStore.Images.Media.BUCKET_DISPLAY_NAME, MediaStore.MediaColumns.DATE_MODIFIED)
        val cursor = context.contentResolver.query(uri, projectionOnlyBucket, selection, selectionArgs,
                MediaStore.MediaColumns.DATE_MODIFIED + " DESC ")
        val columnIndexData = cursor.getColumnIndexOrThrow(MediaStore.MediaColumns.DATA)
        val columnIndexDisplayName = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.BUCKET_DISPLAY_NAME)

        while (cursor.moveToNext()) {
            val imagePath = cursor.getString(columnIndexData)
            val displayName = cursor.getString(columnIndexDisplayName)
            if (!TextUtils.isEmpty(imagePath)) listOfAllImages.add(GalleryListData(GalleryListData.IMAGE, displayName, imagePath))
        }
        cursor.close()
        remotlyChanged.onNext(listOfAllImages)
    }

    private fun getListOfAllImagesFolders() {
        val uri: Uri = android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI
        val cursor: Cursor
        var cursorBucket: Cursor
        val columnIndexData: Int
        val columnIndexFolderName: Int
        val listOfAllImages = ArrayList<String>()
        var imagePath: String?
        val albumsList = ArrayList<GalleryListData>()
        val BUCKET_GROUP_BY = "1) GROUP BY 1,(2"
        val BUCKET_ORDER_BY = "MAX(datetaken) DESC"
        val projection = arrayOf(MediaStore.Images.ImageColumns.BUCKET_ID, MediaStore.Images.ImageColumns.BUCKET_DISPLAY_NAME,
                MediaStore.Images.ImageColumns.DATE_TAKEN, MediaStore.Images.ImageColumns.DATA)
        cursor = context.contentResolver.query(uri, projection, BUCKET_GROUP_BY, null, BUCKET_ORDER_BY)
        if (cursor != null) {
            columnIndexData = cursor.getColumnIndexOrThrow(MediaStore.MediaColumns.DATA)
            columnIndexFolderName = cursor
                    .getColumnIndexOrThrow(MediaStore.Images.Media.BUCKET_DISPLAY_NAME)
            while (cursor.moveToNext()) {
                imagePath = cursor.getString(columnIndexData)
                Log.d(TAG, "folder name:" + cursor.getString(columnIndexData))
                val selectionArgs = arrayOf("%" + cursor.getString(columnIndexFolderName) + "%")
                val selection = MediaStore.Images.Media.DATA + " like ? "
                val projectionOnlyBucket = arrayOf(MediaStore.MediaColumns.DATA, MediaStore.Images.Media.BUCKET_DISPLAY_NAME)
                cursorBucket = context.contentResolver.query(uri, projectionOnlyBucket, selection, selectionArgs, null)
                if (imagePath != "" && imagePath != null) {
                    listOfAllImages.add(imagePath)
                    albumsList.add(GalleryListData(GalleryListData.FOLDER, cursor.getString(columnIndexFolderName), imagePath, cursorBucket.count))
                }
                cursorBucket.close()
            }
            cursor.close()
        }
        remotlyChanged.onNext(albumsList)
    }

    fun isInRootState() : Boolean = TextUtils.isEmpty(activeFolder)

    fun moveToRootState() {
        clearSelection()
        activeFolder = ""
        reload()
    }

    fun moveToFolder(folder : String) {
        clearSelection()
        activeFolder = folder
        reload()
    }
}
