package com.bria.common.controller.im.refactoring.roomdb

import android.arch.persistence.room.*
import com.bria.common.controller.im.refactoring.db.entities.ImStatusData
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.im.refactoring.roomdb.entities.Message
import io.reactivex.Maybe
import io.reactivex.Single


//nvtd move all to reactive?
@Dao
interface ChatRoomDao {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    fun insertRoom(chatRoom: ChatRoom) : Long?

    @Delete
    fun deleteRoom(room: ChatRoom)

    @Query("SELECT * FROM rooms ORDER BY name")  //where accountID in list
    fun getAllRooms(): Single<List<ChatRoom>>

    @Query("SELECT * FROM rooms WHERE id IS :id")
    fun getRoom(id: Long): ChatRoom?

    @Query("SELECT * FROM rooms WHERE chatKey IS :key")
    fun getRoomByRoomKey(key: String): ChatRoom?

    @Query("SELECT * FROM messages WHERE chatId = :id ORDER BY modificationTime LIMIT 1")
    fun getLastMessageForChatRoom(id: Long): Maybe<Message>

    @Query("SELECT COUNT(status) FROM messages WHERE status = ${ImStatusData.INCOMING_UNREAD} AND read = 0 AND chatId = :id")
    fun getNumberOfUnreadMessagesForChatRoom(id: Long): Single<Int>

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    fun insertMessage(message: Message) : Long?

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    fun insertMessages(messages: List<Message>) : Array<Long?>

    @Delete
    fun deleteMessages(messages: List<Message>) : Int

    @Update
    fun updateMessages(message: List<Message>) : Int

    @Query("SELECT * FROM messages WHERE id IS :id")
    fun getMessageById(id: Long) : Message?

    @Query("UPDATE messages SET read = 1 WHERE id in (:messageIds)")
    fun markRead(messageIds : List<Long>)

    @Query("UPDATE messages SET read = 1 WHERE chatId = :chatRoomId")
    fun markReadChatRoom(chatRoomId: Long ) : Int

    @Query("SELECT * FROM messages WHERE chatId = :id ORDER BY modificationTime LIMIT :limit OFFSET :offset")
    fun getNMessageWithOffsetForChat(id: Long, offset : Int, limit : Int) : List<Message>

    @Query("UPDATE rooms SET modTime = :time WHERE id = :chatId")
    fun updateChatTime(time: Long, chatId: Long)

    @Update
    fun updateChatRoom(chatRoom: ChatRoom)

    @Query("DELETE FROM messages WHERE chatId = :id")
    fun deleteMessagesForChatRoom(id: Long)

    @Query("SELECT * FROM rooms WHERE isPrivateRoom = 1")
    fun getAllPrivateChatRooms(): Single<List<ChatRoom>>


}
