package com.bria.common.controller.im.refactoring.roomdb

import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.im.refactoring.roomdb.entities.Message
import io.reactivex.Maybe
import io.reactivex.Single
import io.reactivex.subjects.Subject

interface ChatRoomRepo {
    fun addChatRoom(chatRoom : ChatRoom)

    fun deleteChatRoom(chatRoom: ChatRoom)

    fun changeOwner(owner : String)

    fun getOnRoomAddedObservable() : Subject<List<ChatRoom>>
    fun getOnRoomUpdatedObservable() : Subject<List<ChatRoom>>
    fun getOnRoomDeletedObservable() : Subject<List<ChatRoom>>

    fun getAllChatRooms() : Single<List<ChatRoom>>

    fun getLastMessageForChatRoom(id : Long) : Maybe<Message>

    fun getNumberOfUnreadMessagesForChatRoom(id: Long): Single<Int>

    fun addMessage(message: Message)

    fun getNMessageWithOffsetForChat(id: Long, offset: Int, limit: Int) : List<Message>

    fun getOnMessagesAddedObservable(): Subject<List<Message>>

    fun getOnMessagesUpdatedObservable() : Subject<List<Message>>

    fun getOnServerRoomListChangedObservable() : Subject<List<LocalRoomInfo>>

    fun getOnMessagesRemovedObservable() : Subject<List<Message>>
    fun getOnMessagesReadObservable(): Subject<List<Message>>
    fun addMessages(messages: List<Message>)
    fun updateMessages(messages: List<Message>)
    fun deleteMessages(messages: List<Message>)
    fun markRead(messages: List<Message>)
    fun markReadChatRoom(id: Long)
    fun getRoomByRoomKey(key: String): ChatRoom?
    fun getChatRoom(id: Long): ChatRoom?
    fun getMessageById(id: Long): Message?
    fun updateChatRoom(chatRoom: ChatRoom)

    fun addRoomInfo(info: LocalRoomInfo)
    fun removeRoomInfosForAcc(accId: String)
    fun roomNameInTheList(name: String): Boolean

    fun destroy()
    fun getAllPrivateChatRooms() : Single<List<ChatRoom>>
    fun getServerRoomList(query: String): MutableList<LocalRoomInfo>
    fun addRoomInfos(infos: List<LocalRoomInfo>)
}
