package com.bria.common.controller.im.refactoring.roomdb.entities

import android.arch.persistence.room.ColumnInfo
import android.arch.persistence.room.Entity
import android.arch.persistence.room.PrimaryKey
import android.arch.persistence.room.TypeConverter
import com.bria.common.controller.settings.core.upgrade.v1.types.SettingsStringEncoder_v1
import com.bria.common.util.im.ParticipantsSet

@Entity(tableName = "rooms")
data class ChatRoom(
        @PrimaryKey(autoGenerate = true)
        @ColumnInfo(name = "id") var id: Long = 0,
        @ColumnInfo(name = "type") var type: Long = 0,
        @ColumnInfo(name = "chatKey") var chatKey: String = "",
        @ColumnInfo(name = "jid") var jid: String = "",
        @ColumnInfo(name = "name") var name: String = "",
        @ColumnInfo(name = "description") var description: String = "",
        @ColumnInfo(name = "owners") var owners: ParticipantsSet = ParticipantsSet(),
        @ColumnInfo(name = "members") var members: ParticipantsSet = ParticipantsSet(),
        @ColumnInfo(name = "invited") var invited: ParticipantsSet = ParticipantsSet(),
        @ColumnInfo(name = "password") var password: ChatRoomPassword = ChatRoomPassword(""),
        @ColumnInfo(name = "color") var color: Int = -1,
        @ColumnInfo(name = "accountId") var accountId: String = "",
        @ColumnInfo(name = "isPrivateRoom") var privateRoom: Int = 0,
        @ColumnInfo(name = "modTime") var modTime: Long = 0,
        @ColumnInfo(name = "creationTime") var creationTime: Long = 0,
        @ColumnInfo(name = "state") var state: ChatRoomState =  ChatRoomState.ACTIVE
) {
    fun isUserOwner() : Boolean {
        return owners.containsRemoteAddress(accountId)
    }
}

data class ChatRoomPassword(val plainPassword: String)

class ChatRoomTypeConverters {

    @TypeConverter
    fun toEncryptedPassword(plain: ChatRoomPassword): String {
        if (plain.plainPassword.isEmpty()) return ""
        return SettingsStringEncoder_v1.encrypt(plain.plainPassword ?: "") ?: ""
    }

    @TypeConverter
    fun toPlainPassword(encrypted: String): ChatRoomPassword {
        if (encrypted.isEmpty()) return ChatRoomPassword("")
        return ChatRoomPassword(SettingsStringEncoder_v1.decrypt(encrypted) ?: "")
    }

    @TypeConverter
    fun toParticipantsSet(participants: String): ParticipantsSet {
        return ParticipantsSet(participants)
    }

    @TypeConverter
    fun toParticipantsString(participantsSet: ParticipantsSet): String {
        return participantsSet.serializedParticipantKeys
    }

    @TypeConverter
    fun toChatRoomState(string : String) : ChatRoomState = ChatRoomState.valueOf(string)

    @TypeConverter
    fun toStateString(state : ChatRoomState) : String = state.name

}

fun String.getInitials(): String {
    val initials = this.split(" ").map { if (it.isNotEmpty()) it.first().toString() else it }
    return when (initials.size) {
        0 -> ""
        1 -> if (initials[0].isNotEmpty()) initials[0].toUpperCase() else ""
        2 -> "${initials[0]}${initials[1]}".toUpperCase()
        else -> "${initials[0]}${initials[1]}".toUpperCase()
    }
}

enum class ChatRoomState {
    ACTIVE, LEFT
}


