package com.bria.common.controller.im.refactoring.roomdb.entities

import android.arch.persistence.room.*
import com.bria.common.controller.im.refactoring.db.entities.ImStatusData


//nvtd fix file transfer extenstions

@Entity(tableName = "messages", indices = [Index(value = arrayOf("externalId"), unique = true)])
data class Message(
        @PrimaryKey(autoGenerate = true)
        @ColumnInfo(name = "id") var id: Long = 0,
        @ColumnInfo(name = "chatId") var chatId: Long = 0,
        @ColumnInfo(name = "status") var status: Int = 0,
        @ColumnInfo(name = "creationTime") var creationTime: Long = 0,
        @ColumnInfo(name = "modificationTime") var modificationTime: Long = 0,
        @ColumnInfo(name = "text") var text: String = "",
        @ColumnInfo(name = "externalId") var externalId: String = "",
        @ColumnInfo(name = "xmppThreadId") var xmppThreadId: Long = 0,
        @ColumnInfo(name = "error") var error: Int = 0,
        @ColumnInfo(name = "deleted") var deleted: Boolean = false,
        @ColumnInfo(name = "read") var read: Boolean = false,
        @ColumnInfo(name = "serverId") var serverId: Long = 0,
        @ColumnInfo(name = "syncRev") var syncRev: Long = 0,
        @ColumnInfo(name = "remoteAddress") var remoteAddress: String = "",
        @ColumnInfo(name = "filePath") var filePath: String = "",
        @ColumnInfo(name = "fileSize") var fileSize: Long = 0,
        @ColumnInfo(name = "fileProgress") var fileProgress: Int = 0 //should not be in database
) {
    @Ignore
    private val outgoingList = listOf(ImStatusData.OUTGOING_SENT, ImStatusData.OUTGOING_SENDING_FAILED, ImStatusData.OUTGOING_SEEN,
            ImStatusData.OUTGOING_PENDING, ImStatusData.OUTGOING_DELIVERED, ImStatusData.OUTGOING_NOT_DELIVERED)

    @Ignore
    private val incomingList = listOf(ImStatusData.INCOMING_READ, ImStatusData.INCOMING_UNREAD, ImStatusData.INCOMING_COLLAB_UNREAD)

    fun isOutgoing(): Boolean = status in outgoingList

    fun isIncoming(): Boolean = status in incomingList

    fun isLocal(): Boolean {
        return status == ImStatusData.LOCAL
    }

    fun isSyncable(): Boolean {
        return isIncoming() || status == ImStatusData.OUTGOING_DELIVERED || status == ImStatusData.OUTGOING_SENT || status == ImStatusData.OUTGOING_SEEN
    }

    fun isCollab(): Boolean {
        return status == ImStatusData.INCOMING_COLLAB_UNREAD
    }

    fun isAlreadySynced(): Boolean {
        return !(serverId == 0L || syncRev == 0L)
    }

}
