package com.bria.common.controller.im.refactoring.xmpp

import com.bria.common.controller.im.refactoring.ChatApi
import com.bria.common.util.Utils
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import io.reactivex.subjects.PublishSubject
import java.util.concurrent.TimeUnit

class SyntheticComposingObservable(private val notifyNoLongerComposingCallback: Consumer<XmppChatInfo>) {
    val TAG = "SyntheticComposingObservable"

    val d = mutableMapOf<XmppChatInfo, PublishSubject<Boolean>>()

    var disposables = CompositeDisposable()

    fun notifyOfComposingState(xmppChatInfo: XmppChatInfo, isComposing: Boolean) {
        val subject = d.getOrPut(xmppChatInfo) {

            // create the subject for this chat session
            val result = PublishSubject.create<Boolean>()

            //subscribe immediately
            val disposable = result
                // wait 15 seconds
                .debounce(ChatApi.COMPOSING_IDLE_INTERVAL.toLong(), TimeUnit.SECONDS, Schedulers.computation())

                // if last one was composing
                .filter { it }

                // then let set composing flag to false
                .subscribe({
                    notifyNoLongerComposingCallback.accept(xmppChatInfo)
                }, {
                    Utils.crashInDebug(TAG, it)
                })

            disposables.add(disposable)

            result
        }

        subject.onNext(isComposing)
    }

    fun destroy() {
        disposables.dispose()
    }
}
