package com.bria.common.controller.presence

import com.bria.common.controller.accounts.*
import com.bria.common.controller.presence.refactoring.EPresenceStatus
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.ISettingsReader
import com.bria.common.util.IObservable
import com.bria.common.util.SyncObservableDelegate

class PresenceStatuses(
        private val settings: ISettingsReader<ESetting>,
        private val accountsController: IAccountsCtrlActions,
        private val accountEvents: IObservable<IAccountsCtrlObserver>
) {
    private var mPresenceStatuses: List<EPresenceStatus> = listOf()
    private val mObservable = SyncObservableDelegate<IPresenceStatusesObserver>()

    private val mAccountsObserver = object : IAccountsCtrlObserver {
        override fun onAccountsChanged(changes: AccountsChangeInfo?) {
            updatePresenceStatuses()
        }

        override fun onAccountStatusChanged(account: Account?, isPushTunnel: Boolean) {
            updatePresenceStatuses()
        }

        override fun onPrimaryAccountChanged(primaryAccount: Account?) {}
        override fun onAccountVMNumberChanged(voiceMail: VoiceMail?) {}
        override fun onVpnConnectionFailed(errCode: Int, msg: String?) {}
    }

    init {
        updatePresenceStatuses()

        accountEvents.attachObserver(mAccountsObserver)
    }

    fun shutdown() {
        accountEvents.detachObserver(mAccountsObserver)
    }


    val list: List<EPresenceStatus>
        get() = mPresenceStatuses

    val observable: IObservable<IPresenceStatusesObserver>
        get() = mObservable

    interface IPresenceStatusesObserver {
        fun onPresenceListUpdated()
    }

    private fun updatePresenceStatuses() {

        val genbandEnabled = settings.getBool(ESetting.FeatureGenband)
        val passivePresence = settings.getBool(ESetting.FeaturePassivePresence)
        val noImpsAccs = accountsController.getActiveImAccounts().isEmpty()

        if (genbandEnabled) {
            mPresenceStatuses = listOf(EPresenceStatus.Connected, EPresenceStatus.Away, EPresenceStatus.OutToLunch,
                    EPresenceStatus.Busy, EPresenceStatus.OnVacation, EPresenceStatus.DoNotDisturb,
                    EPresenceStatus.AppearOffline)
        } else if (passivePresence) {
            mPresenceStatuses = listOf(EPresenceStatus.Available, EPresenceStatus.Away, EPresenceStatus.Busy,
                    EPresenceStatus.DoNotDisturb, EPresenceStatus.AppearOffline)
        } else if (noImpsAccs) {
            mPresenceStatuses = listOf(EPresenceStatus.Offline, EPresenceStatus.DoNotDisturb)
        } else {
            mPresenceStatuses = listOf(EPresenceStatus.Available, EPresenceStatus.Away, EPresenceStatus.Busy,
                    EPresenceStatus.DoNotDisturb, EPresenceStatus.OnThePhone, EPresenceStatus.AppearOffline)
        }

        observable.notifyObserver { it.onPresenceListUpdated() }
    }
}
