package com.bria.common.controller.presence.refactoring

import android.content.Context
import android.graphics.drawable.Drawable
import android.support.annotation.DrawableRes
import android.support.annotation.StringRes
import com.bria.common.R
import com.bria.common.util.AndroidUtils

enum class EPresenceStatus constructor(id: Int, @DrawableRes imageResource: Int,
                                       @param:StringRes @field:StringRes internal val stringResource: Int) {
    Available(0, R.drawable.ic_presence_available, R.string.pr_status_available),
    //This is same as Connected, for Genband we use Connected
    Away(1, R.drawable.ic_presence_away, R.string.pr_status_away),
    Busy(2, R.drawable.ic_presence_busy, R.string.pr_status_busy),
    DoNotDisturb(3, R.drawable.ic_presence_disturb, R.string.pr_status_do_not_disturb),
    OnThePhone(4, R.drawable.ic_presence_on_phone, R.string.pr_status_on_the_phone),
    AppearOffline(5, R.drawable.ic_presence_offline, R.string.pr_status_appear_offline),
    Offline(6, R.drawable.ic_presence_offline, R.string.pr_status_offline),
    Unknown(7, R.drawable.ic_presence_unknown, R.string.pr_status_unknown),
    Connected(8, R.drawable.ic_presence_available, R.string.pr_status_connected),
    OutToLunch(9, R.drawable.ic_presence_away, R.string.pr_status_out_to_lunch),
    BeRightBack(10, R.drawable.ic_presence_away, R.string.pr_status_be_right_back),
    OnVacation(11, R.drawable.ic_presence_busy, R.string.pr_status_on_vacation),
    OnHoliday(12, R.drawable.ic_presence_busy, R.string.pr_status_on_holiday),
    Idle(13, R.drawable.ic_presence_away, R.string.pr_status_idle),
    Blocked(14, R.drawable.ic_presence_offline, R.string.pr_status_blocked),
    Other(999, R.drawable.ic_presence_offline, R.string.pr_status_unknown),
    // Presence Watcher Authorization - Pending Authorization status
    PendingAuthorization(1999, R.drawable.ic_presence_watch, R.string.pr_status_pendingauth),
    // The following is for display of remote contacts only, Telefonica specific
    OfflinePending(2000, R.drawable.ic_presence_pending_authorization, R.string.pr_status_pendingauth),
    OfflineForbidden(2001, R.drawable.ic_presence_offline, R.string.pr_status_notauthorized),
    Indeterminate(2002, R.drawable.ic_presence_offline, R.string.pr_status_undetermined);
    //IMPORTANT: any id change of the above enums should be synched with getEPresenceStatusById below
    // IMPORTANT: check ToolbarCustomizer to sync there as well

    var id: Int = 0
        internal set
    @DrawableRes
    @get:DrawableRes
    var iconResourceId: Int = 0
        internal set

    init {
        this.id = id
        iconResourceId = imageResource
    }

    fun getIcon(context: Context): Drawable? {
        return AndroidUtils.getDrawable(context, iconResourceId)
    }

    fun getString(context: Context): String {
        return context.getString(stringResource)
    }
}
