package com.bria.common.controller.presence.refactoring

import android.text.TextUtils
import com.bria.common.controller.contacts.buddy.BuddySubscription
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.contacts.buddy.repository.BuddyRepository
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.BuddyKeyUtils
import com.bria.common.controller.presence.BuddyPresence
import com.bria.common.controller.settings.branding.EAccountType
import com.bria.common.sdkwrapper.SipStackManager
import com.bria.common.util.ThreadExecutors
import com.counterpath.sdk.XmppApiVCard
import com.counterpath.sdk.XmppRoster
import com.counterpath.sdk.handler.XmppRosterHandler
import com.counterpath.sdk.pb.Xmpproster

class XmppRostersApi(val buddyRepository: BuddyRepository): XmppRosterHandler {

    private var stackManager: SipStackManager? = null

    private val vCardMap = HashMap<Int, Int>()// key = xmpp account handle; value = vCard handle


    override fun onXmppRosterUpdateEvent(roster: XmppRoster, evt: Xmpproster.XmppRosterEvents.XmppRosterUpdateEvent) {
        val xmppAccount = roster.api.account
        val xmppApiVcard = XmppApiVCard.get(xmppAccount)

        //if (!xmppApiVcard.getHandlers().contains(this)) xmppApiVcard.addHandler(this) nvtd add this

        var vCardHandle = vCardMap[xmppAccount.handle().get()]
        val xmppVcard = if (vCardHandle == null) {
            val tempVcard = xmppApiVcard.newVCard()
            vCardHandle = tempVcard.handle()
            vCardMap.put(xmppAccount.handle().get(), vCardHandle)
            tempVcard
        } else {
            xmppApiVcard.getVCard(vCardHandle)
        }
        stackManager?.let { manager ->
            val account = manager.getRegistrationManager().getAccount(xmppAccount) ?: return
            xmppVcard?.fetchVCard(ImpsUtils.getAddressWithDomain(account)) // fetch my Vcard info
            for (i in 0..(evt.addedCount - 1)) {
                val item = evt.getAdded(i).item
                val buddy = XmppBuddy()
                buddy.subscription = item.subscription
                buddy.accountNickname = ImpsUtils.getAddressWithDomain(account)
                buddy.imAddress = ImpsUtils.getRemoteAddress(item.address)
                buddy.displayName = item.displayName
                val presence = BuddyPresence(if (buddy.isSubscribed) EPresenceStatus.Offline else EPresenceStatus.Blocked)
                buddy.presence = presence
                if (!TextUtils.isEmpty(buddy.imAddress) && buddy.subscription != BuddySubscription.S10nNone) {
                    xmppVcard.fetchVCard(buddy.imAddress)
                }
                buddyRepository.addBuddy(buddy)
            }
            for (i in 0..(evt.removedCount - 1)) {
                buddyRepository.removeBuddyByKey(BuddyKeyUtils.getNewBuddyKey(EAccountType.Xmpp,
                        ImpsUtils.getAddressWithDomain(account), ImpsUtils.getRemoteAddress(evt.getRemoved(i).address)))
            }
            for (i in 0..(evt.updatedCount - 1)) {
                val item = evt.getUpdated(i).item
                val jid = ImpsUtils.getRemoteAddress(item.address)
                buddyRepository.getBuddyByKey(BuddyKeyUtils.getNewBuddyKey(EAccountType.Xmpp, ImpsUtils.getAddressWithDomain(account), jid))?.let { buddy ->
                    val presence = BuddyPresence(if (buddy.isSubscribed) EPresenceStatus.Offline else EPresenceStatus.Blocked)
                }

            }
        }



    }

    override fun onXmppRosterErrorEvent(p0: XmppRoster?, p1: Xmpproster.XmppRosterEvents.XmppRosterErrorEvent?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onXmppRosterUpdateFailedEvent(p0: XmppRoster?, p1: Xmpproster.XmppRosterEvents.XmppRosterUpdateFailedEvent?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onXmppRosterUnsubscriptionRequestEvent(p0: XmppRoster?, p1: Xmpproster.XmppRosterEvents.XmppRosterUnsubscriptionRequestEvent?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onXmppRosterSubscriptionRequestEvent(p0: XmppRoster?, p1: Xmpproster.XmppRosterEvents.XmppRosterSubscriptionRequestEvent?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onXmppRosterPresenceEvent(p0: XmppRoster?, p1: Xmpproster.XmppRosterEvents.XmppRosterPresenceEvent?) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    private fun offload(task: Runnable): Boolean {
        return ThreadExecutors.scheduleLightWork(task)
    }
}
