package com.bria.common.controller.remotedebug

import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.ISettingsObserver
import com.bria.common.controller.settings.core.Settings
import com.bria.common.notification.NotificationsHandler
import com.bria.common.util.IObservable

class RemoteDebugNotifications(
        remoteDebugCtrlObservable: IObservable<IRemoteDebugCtrlObserver>,
        private val notificationsHandler: NotificationsHandler,
        settings: Settings) {

    companion object {
        private var instance: RemoteDebugNotifications? = null

        @JvmStatic
        fun initialize(remoteDebugController: RemoteDebugController, notificationsHandler: NotificationsHandler, settings: Settings) {
            instance = RemoteDebugNotifications(remoteDebugController.observable, notificationsHandler, settings)
        }
    }

    private var mNotificationsRemoteDebugEnabled: Boolean

    init {
        mNotificationsRemoteDebugEnabled = settings.getBool(ESetting.NotificationsRemoteDebugEnabled)

        remoteDebugCtrlObservable.attachObserver(object : IRemoteDebugCtrlObserver {
            override fun onStatusChanged(remoteDebugStatusMessage: RemoteDebugStatusMessage, status: EHdaConnectionStatus) {
                if (remoteDebugStatusMessage.category == RemoteDebugStatusMessage.ECategory.REMOTE_DEBUG) {
                    updateRemoteDebugNotification(remoteDebugStatusMessage.notifBar)
                }
            }
        })

        settings.attachObserver(object : ISettingsObserver {
            override fun onOwnerChanged() {}

            override fun onSettingsChanged(changedSettings: MutableSet<ESetting>) {

                if (changedSettings.contains(ESetting.NotificationsRemoteDebugEnabled)) {
                    mNotificationsRemoteDebugEnabled = settings.getBool(ESetting.NotificationsRemoteDebugEnabled)
                    if (!mNotificationsRemoteDebugEnabled) {
                        notificationsHandler.cancelRemoteDebugNotification()
                    }
                }
            }
        }, setOf(ESetting.NotificationsRemoteDebugEnabled))
    }

    private fun updateRemoteDebugNotification(show: Boolean) {
        if (!mNotificationsRemoteDebugEnabled) {
            return
        }

        notificationsHandler.updateRemoteDebugNotification(show)
    }
}
