package com.bria.common.controller.remotesync

import com.bria.common.controller.accounts.Account
import com.bria.common.controller.im.refactoring.IImData
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData
import com.bria.common.controller.im.refactoring.db.entities.InstantMessageData
import java.util.*
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.CopyOnWriteArrayList

class RemoteSyncQueue(private var remoteSync: RemoteSync) {

    private val listOfUpdateItems: CopyOnWriteArrayList<Long> = CopyOnWriteArrayList()
    private val listOfUpdateConversations: CopyOnWriteArrayList<Long> = CopyOnWriteArrayList()
    private val mapOfFetchRangeRevision: ConcurrentHashMap<Long, Int> = ConcurrentHashMap()
    private val listOfFetchConversations: CopyOnWriteArrayList<Long> = CopyOnWriteArrayList()
    private val listOfGetMessageCount: CopyOnWriteArrayList<Account> = CopyOnWriteArrayList()
    private val listOfSyncItem: CopyOnWriteArrayList<Long> = CopyOnWriteArrayList()
    private val listOfFetchConversationsOffset: CopyOnWriteArrayList<Int> = CopyOnWriteArrayList()
    private val setOfAccounts: MutableSet<Account> = mutableSetOf()

    fun addPendingUpdateItem(messageId : Long) {
        listOfUpdateItems.add(messageId)
    }

    fun addPendingUpdateConversation(conversationId: Long) {
        listOfUpdateConversations.add(conversationId)
    }

    fun addPendingFetchRangeRevision(conversationId: Long, offset: Int) {
        mapOfFetchRangeRevision[conversationId] = offset
    }

    fun addPendingFetchConversations(conversations: List<ImConversationData>) {
        conversations.forEach {
            listOfFetchConversations.add(it.id)
        }
    }

    fun addPendingGetMessagesCount(account: Account) {
        listOfGetMessageCount.add(account)
    }

    fun addPendingSyncItem(im: InstantMessageData) {
        listOfSyncItem.add(im.id)
    }

    fun addPendingFetchConversations(offset: Int) {
        listOfFetchConversationsOffset.add(offset)
    }

    fun addPendingSyncItems(listOfMessages: List<InstantMessageData>) {
        listOfMessages.forEach {
            listOfSyncItem.add(it.id)
        }
    }

    fun addPendingSetAccounts(listOfAccounts: List<Account>) {
        val map: ConcurrentHashMap<Account, Boolean> = ConcurrentHashMap()
        listOfAccounts.forEach {
            map[it] = true
        }
        setOfAccounts.addAll(Collections.newSetFromMap(map))
    }

    fun flushPendingItems() {
        val imData: IImData = remoteSync.imData
        if (setOfAccounts.isNotEmpty()) {
            remoteSync.setAccounts(setOfAccounts.toList())
            setOfAccounts.clear()
        }
        if (listOfSyncItem.isNotEmpty()) {
            val listOfSyncItems: MutableList<InstantMessageData> = mutableListOf()
            listOfSyncItem.forEach {id ->
                imData.getMessageById(id)?.let {
                    listOfSyncItems.add(it)
                }
            }
            remoteSync.syncNewMessages(listOfSyncItems)
            listOfSyncItem.clear()
        }
        if (listOfUpdateItems.isNotEmpty()) {
            listOfUpdateItems.forEach {
                remoteSync.updateMessage(imData.getMessageById(it))
            }
            listOfUpdateItems.clear()
        }
        if (listOfUpdateConversations.isNotEmpty()) {
            listOfUpdateConversations.forEach { id ->
                imData.getConversationById(id)?.let {
                    remoteSync.updateImConversation(it)
                }
            }
            listOfUpdateConversations.clear()
        }
        if (listOfFetchConversations.isNotEmpty()) {
            val listOfConversations = arrayListOf<ImConversationData>()
            listOfFetchConversations.forEach { id ->
                imData.getConversationById(id)?.let {
                    listOfConversations.add(it)
                }
            }
            remoteSync.fetchConversations(listOfConversations)
            listOfFetchConversations.clear()
        }
        if (listOfFetchConversationsOffset.isNotEmpty()) {
            val offset: Int = listOfFetchConversationsOffset.min() ?: 0
            remoteSync.fetchConversations(offset)
            listOfFetchConversationsOffset.clear()
        }
        if (mapOfFetchRangeRevision.isNotEmpty()) {
            mapOfFetchRangeRevision.forEach {
                (key, value) -> remoteSync.fetchRangeRevision(key, value)
            }
            mapOfFetchRangeRevision.clear()
        }
        if (listOfGetMessageCount.isNotEmpty()) {
            listOfGetMessageCount.forEach {
                remoteSync.getMessageCount(it)
            }
            listOfGetMessageCount.clear()
        }
    }

}
