package com.bria.common.modules

import android.content.Context
import android.os.Build
import android.os.Process
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.IController
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.Settings
import com.bria.common.uiframework.helpers.ProximityManager
import com.counterpath.sdk.android.SipPhoneAndroid
import kotlin.concurrent.thread

object NoncriticalComponentsInitializer {
    fun initialize(context: Context, controller:IController, sipPhoneAndroid: SipPhoneAndroid) {
        thread(
                // true is default, but just to be explicit
                start = true,

                // don't prevent shutdown
                // if `true`, the thread is created as a daemon thread. The Java Virtual Machine exits when the only threads running are all daemon threads.
                isDaemon = true,

                // nice to see in profiler
                name = "NoncriticalComponentsInitializer") {

            /**
             * Lowest available thread priority.  Only for those who really, really
             * don't want to run if anything else is happening.
             * Use with {@link #setThreadPriority(int)} and
             * {@link #setThreadPriority(int, int)}, <b>not</b> with the normal
             * {@link java.lang.Thread} class.
             */
            Process.setThreadPriority(Process.THREAD_PRIORITY_LOWEST)


            // now start components

            BriaGraph.networkModule.init()
            BriaGraph.androidSoundInfo.dumpHWAudioEffectsSupportedByDeviceToLog()
            ProximityManager.create(context, controller.getPhoneCtrl().getEvents().getCallsApi(), controller.getSettingsCtrl().getEvents())
            BriaGraph.goodReviewDriver.initialize()
            BIAnalytics.initializeBIEvent(context, sipPhoneAndroid, controller.getSettingsCtrl().getEvents())
            BriaGraph.cpcCallingSchemesPattern
            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && Settings.get(context).getBool(ESetting.FeatureShortcutManager)) {
                BriaGraph.shortcutManager
            }
        }
    }
}
