package com.bria.common.modules

import android.os.Bundle
import io.reactivex.subjects.BehaviorSubject

/**
 * An object that dispatches push messages received from push broadcast receivers to all subscribers.
 */
object PushMessageDispatcher {

    private val publisher = BehaviorSubject.createDefault(Bundle())

    /**
     * Returns the observable for a stream of push messages that arrived to one of the broadcast receivers.
     */
    fun getObservable() = publisher.share()!!

    /**
     * Dispatches the given push message to all subscribers.
     * @param message The message to publish, must not be `null`
     */
    fun dispatch(message: Bundle) = publisher.onNext(message)

}
