package com.bria.common.modules.android

import android.content.Context
import android.content.pm.PackageManager
import android.media.audiofx.AcousticEchoCanceler
import android.media.audiofx.AutomaticGainControl
import android.media.audiofx.NoiseSuppressor
import com.bria.common.util.Log

class AndroidSoundInfo(private val context: Context) {
    val LOG_TAG = "AndroidSoundInfo"

    val isNoiseSuppressorAvailable by lazy {
        // it's kind of expensive, so we cache

        try {
            return@lazy NoiseSuppressor.isAvailable()
        } catch (e: Exception) {
            // this was empty before also
        }

        return@lazy false
    }

    val isAutomaticGainControlAvailable by lazy {
        try {
            return@lazy AutomaticGainControl.isAvailable()
        } catch (e: Exception) {
            //Log.e(e);
        }

        return@lazy false
    }

    val isAcousticEchoCancelerAvailable by lazy {
        try {
            return@lazy AcousticEchoCanceler.isAvailable()
        } catch (e: Exception) {
            //Log.e(e);
        }

        return@lazy false
    }

    val isLowLatencyFeatureAvailable by lazy {
        val pm = context.packageManager
        val claimsFeature = pm.hasSystemFeature(PackageManager.FEATURE_AUDIO_LOW_LATENCY)

        return@lazy claimsFeature
    }

    fun dumpHWAudioEffectsSupportedByDeviceToLog() {
        Log.i(LOG_TAG, "HW AEC: " + if (isAcousticEchoCancelerAvailable) "supported" else "unsupported")
        Log.i(LOG_TAG, "HW NS: " + if (isNoiseSuppressorAvailable) "supported" else "unsupported")
        Log.i(LOG_TAG, "HW AGC: " + if (isAutomaticGainControlAvailable) "supported" else "unsupported")
        Log.i(LOG_TAG, "HW LowLatency: " + if (isLowLatencyFeatureAvailable) "supported" else "unsupported")
    }
}
