package com.bria.common.modules.android

import android.app.NotificationManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.os.Build
import com.bria.common.controller.Controller
import com.bria.common.controller.settings.ESetting
import com.bria.common.modules.ModuleClassFinder
import com.bria.common.util.AndroidUtils
import com.bria.common.util.GlobalConstants
import com.bria.common.util.Log

class DoNotDisturbReceiver(var controller: Controller) : BroadcastReceiver() {

    companion object {
        private val TAG = "DoNotDisturbReceiver"
    }

    override fun onReceive(context: Context, intent: Intent?) {
        Log.d(TAG, "Do Not Disturb state changed, intent = $intent")
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            val notificationManager = context.applicationContext.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager?
            notificationManager?.let {
                // don't react if already granted
                if (it.isNotificationPolicyAccessGranted) {
                    Log.i(TAG, "DND access is already granted, ignoring")
                    return
                }

                // if user hasn't checked never to show it again, display the dialog
                val hasActiveCalls = controller.phoneCtrl.events.callsApi.calls.size > 0
                if (hasActiveCalls && intent != null && !controller.settingsCtrl.events.getBool(ESetting.DoNotDisturbDialogShow)) {
                    val serviceIntent = Intent(GlobalConstants.INTENT_DND_STATE_CHANGED, null, context, ModuleClassFinder.instance.serviceClass)
                    Log.i(TAG, "Starting service to display the DND prompt (if not already started), intent = $serviceIntent")
                    // this triggers onStartCommand
                    AndroidUtils.startServiceCompat(context, serviceIntent)
                }
            }
        }
    }

}
