package com.bria.common.modules.android

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import com.bria.common.modules.ModuleClassFinder
import com.bria.common.modules.framework.ComponentLifecycle
import com.bria.common.modules.meta.ComponentIds
import com.bria.common.util.AndroidUtils
import com.bria.common.util.Log

class ScreenOnOffReceiver : BroadcastReceiver() {

    companion object {
        private val TAG = "ScreenOnOffReceiver"
        const val ACTION_INTERACTIVE_STATE_CHANGED = "ACTION_INTERACTIVE_STATE_CHANGED"
        const val INTERACTIVE_STATE_VALUE = "INTERACTIVE_STATE_VALUE"
    }

    override fun onReceive(context: Context, intent: Intent?) {
        // check current state from intent
        var isInteractive: Boolean? = null
        if (intent?.action == Intent.ACTION_SCREEN_OFF) {
            isInteractive = false
        } else if (intent?.action == Intent.ACTION_SCREEN_ON) {
            isInteractive = true
        }
        Log.d(TAG, "Screen state changed, is device interactive: $isInteractive")

        // send intent to service only if interactive state has changed and service is fully created
        val isServiceRunning = ComponentLifecycle.getLastState(ComponentIds.SERVICE_MAIN) == ComponentLifecycle.STATE_CREATED
        if (isInteractive != null && isServiceRunning) {
            val serviceIntent = Intent(context, ModuleClassFinder.instance.serviceClass)
            serviceIntent.action = ACTION_INTERACTIVE_STATE_CHANGED
            serviceIntent.putExtra(INTERACTIVE_STATE_VALUE, isInteractive)
            AndroidUtils.startServiceCompat(context, serviceIntent)
        }
    }

}
