package com.bria.common.uireusable.dataprovider

import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import io.reactivex.subjects.Subject

abstract class AbstractDiffDataProvider<T>() {

    val selectedItems = ArrayList<T>()

    var list = ArrayList<T>()

    private val sourceOfManuallyChangedList = PublishSubject.create<ArrayList<T>>()

    var remotlyChanged: Subject<List<T>> = PublishSubject.create()

    var source : Observable<List<T>>

    init {
        source = remotlyChanged.mergeWith(sourceOfManuallyChangedList).map{ newList ->
            list.clear()
            list.addAll(newList)
            newList
        }.replay(1).autoConnect()
    }

    private fun cleanData() {
        sourceOfManuallyChangedList.onNext(ArrayList())
    }

    abstract fun reload()

    /*
    * clear all selected messages and fire list update
    */
    fun clearSelection() {
        selectedItems.clear()
        sourceOfManuallyChangedList.onNext(list)
    }

    fun isSelected(t: T): Boolean {
        return selectedItems.contains(t)
    }

    fun toggleSelection(t: T) {
        if (selectedItems.contains(t)) {
            selectedItems.remove(t)
        } else {
            selectedItems.add(t)
        }
    }


    /*
     * Returns list of selected items
     */
    fun getSelectedItems(): List<T> {
        return selectedItems
    }


    fun getSelectedCount(): Int = selectedItems.size

}
