package com.bria.common.uireusable.datatypes

import android.content.Context
import android.text.TextUtils
import com.bria.common.R
import com.bria.common.controller.Controllers
import com.bria.common.controller.accounts.Account
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData.*
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoom
import com.bria.common.controller.im.refactoring.roomdb.entities.ChatRoomState
import com.bria.common.controller.settings.branding.EAccountType
import com.bria.common.util.im.Participant
import com.bria.common.util.im.ParticipantsSet
import java.util.*

class ExtendedConversationData(val context: Context) {

    var participantSet: ParticipantsSet = ParticipantsSet()
    var imConversationData: ImConversationData? = null
        internal set
    internal var chatRoom: ChatRoom? = null
    internal var mConversationTitle: String = ""
    var roomId: String? = null

    /*
    We are using this only until conversation is created
     */
    private var mTemporaryConversationType: Int = 0

    fun getRemoteKey(): String {
        return when {
            chatRoom != null -> chatRoom!!.chatKey
            imConversationData != null -> imConversationData!!.remoteKey
            else -> participantSet.serializedParticipantKeys
        }
    }

    fun isGroupChat(): Boolean {
        return (chatRoom == null) && ((imConversationData != null && imConversationData!!.isGroupChat) || (participantSet.size > 1))
    }

    fun getId(): Long {
        return when {
            imConversationData != null -> imConversationData!!.id
            chatRoom != null -> chatRoom!!.id
            else -> ImConversationData.INVALID_CONVERSATION_ID
        }
    }

    fun isImType(): Boolean = if (imConversationData != null) {
        imConversationData!!.isImType
    } else mTemporaryConversationType == IM_XMPP || mTemporaryConversationType == IM_SIP
            || mTemporaryConversationType == GROUPCHAT_SIP || mTemporaryConversationType == GROUPCHAT_XMPP

    fun isSmsType(): Boolean = if (imConversationData != null) {
        imConversationData!!.isSMSType
    } else mTemporaryConversationType == SMS || mTemporaryConversationType == GROUPCHAT_SMS || mTemporaryConversationType == SMS_API

    fun isCreated(): Boolean = getId() != ImConversationData.INVALID_CONVERSATION_ID

    fun getType(): Int {
        if (chatRoom != null) {
            return CHAT_ROOM_TYPE
        } else if (imConversationData != null) {
            return imConversationData!!.type
        } else {
            return mTemporaryConversationType
        }
    }

    fun setType(type: Int) {
        mTemporaryConversationType = type
    }

    fun getFirstParticipant(): Buddy? = if (!participantSet.isEmpty()) participantSet.firstParticipant else null

    fun getAccount(): Account? {
        if (Controllers.isDestroyed() || !isCreated()) {
            return null
        } else if (chatRoom != null) {
            return Controllers.get().accounts.getAccountForUserAtDomain(chatRoom!!.accountId, EAccountType.Xmpp)
        } else if (imConversationData != null) {
            return Controllers.get().accounts.getAccountForUserAtDomain(imConversationData!!.accountId,
                    ImpsUtils.getAccountTypeFromConversationType(imConversationData!!.type))
        }
        return null
    }

    fun isActive(): Boolean = Controllers.get().im.chatApi.isActive(imConversationData!!.id!!)

    fun getModTime(): Long = if (isCreated()) {
        imConversationData!!.modTime
    } else {
        -1
    }

    fun isSipType(): Boolean = getType() == IM_SIP || getType() == GROUPCHAT_SIP

    fun isXMPPType(): Boolean = isCreated() && (getType() == IM_XMPP || getType() == GROUPCHAT_XMPP || getType() == GROUPCHAT_COLLAB)

    fun getConversationTitle(): String {
        return if (!TextUtils.isEmpty(mConversationTitle)) {
            mConversationTitle
        } else {
            participantSet.participantsString
        }
    }

    fun setConversation(conversation: ImConversationData?) {
        chatRoom = null
        mConversationTitle = ""
        imConversationData = conversation
        if (conversation != null) {
            participantSet = ParticipantsSet(imConversationData!!.participants)
        }
    }

    fun setChatRoom(chatRoom: ChatRoom) {
        imConversationData = null
        mConversationTitle = if (chatRoom.state == ChatRoomState.LEFT) {
            "${context.getString(R.string.tLeftChatRoom)} ${chatRoom.name}"
        } else {
            chatRoom.name
        }
        this.chatRoom = chatRoom
    }

    fun getChatRoom(): ChatRoom? {
        return chatRoom
    }

    fun isChatRoom() = this.chatRoom != null


    fun createConversation(buddyKeyList: Iterable<String>, type: Int) {
        mTemporaryConversationType = type
        participantSet = ParticipantsSet()
        for (buddyKey in buddyKeyList) participantSet.add(Participant(buddyKey))
        val imConversationData = Controllers.get().im.imData.getConversationByRemoteKeyAndType(participantSet.serializedParticipantKeys, mTemporaryConversationType)
        setConversation(imConversationData)
    }

    fun createConversation(buddyKey: String, type: Int) {
        val buddyKeyList = ArrayList<String>()
        buddyKeyList.add(buddyKey)
        createConversation(buddyKeyList, type)
    }

    fun isParticipantActive(participant: Participant): Boolean {
        val isActiveInConference = Controllers.get().im.chatApi.isParticipantActive(imConversationData, participant)
        val isOffline = participant.isOffline
        return isActiveInConference && !isOffline
    }

    fun setConversationText(conversationTitle: String) {
        mConversationTitle = conversationTitle
    }

    fun isPrivateChatRoom(): Boolean {
        return isChatRoom() && chatRoom?.privateRoom == 1
    }

    fun isLeft(): Boolean {
        return isChatRoom() && chatRoom?.state == ChatRoomState.LEFT
    }

    companion object {
        const val CHAT_ROOM_TYPE = -10;
    }
}
