package com.bria.common.uireusable.datatypes

import android.content.Context
import android.graphics.Bitmap
import android.text.TextUtils
import android.text.format.DateUtils
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.articles.ArticleResult
import com.bria.common.controller.im.refactoring.db.entities.ImStatusData
import com.bria.common.controller.im.refactoring.db.entities.InstantMessageData
import com.bria.common.controller.im.refactoring.db.entities.getStatusPrefix
import com.bria.common.controller.im.refactoring.filetransfer.FileTransferUtils
import com.bria.common.controller.im.refactoring.roomdb.entities.Message
import com.bria.common.util.Log
import java.io.File
import java.text.SimpleDateFormat
import java.util.*

class MessageListItemData : Comparable<MessageListItemData> {

    var id: Long = 0
    var msgType: Int = 0
    var message: String
    var jid: String = ""
    var participantName: String = ""
    var displayName: String? = null
    var dateString: String = ""
    var timeString: String = ""
    var avatar: Bitmap? = null
    var articleUrl: String = ""
    var articleTitle: String? = null
    var articleDetails: String? = null
    var articlePhotoUrl: String? = null
    var isVccsInviteLink: Boolean = false
    var checkedForArticle: Boolean = false
    var isRead = true
    var isSip: Boolean = false
    var error: Boolean = false
    var selected: Boolean = false
    var externalId : String = ""
    var modTime: Long = 0
    var time: Long = 0
    var conversationId: Long = -1
    var buddyVisible: Boolean = false
    var filePath: String? = ""
    var fileName: String? = ""
    var fileSize: String? = ""
    var fileExtension : String? = ""
    var fileTransferProgress: Int = 0
    var status: Int = -1
    var oldMessage: MessageListItemData? = null
    var deleted = false
    var context: Context

    constructor(imData: InstantMessageData, context: Context) : super() {
        id = imData.id!!
        conversationId = imData.conversationId!!
        status = imData.status
        msgType = getTypeFromStatus(imData.status)
        this.context = context
        time = imData.time
        modTime = imData.modTime
        message = imData.message
        participantName = imData.remoteAddress
        jid = imData.remoteAddress
        error = (imData.status == ImStatusData.OUTGOING_SENDING_FAILED) || (imData.status == ImStatusData.OUTGOING_NOT_DELIVERED)
        isRead = imData.isRead
        externalId = imData.externalId
        filePath = imData.filePath
        if (!TextUtils.isEmpty(filePath)) {
            val file = File(filePath)
            fileName = file.name
            fileExtension = file.extension
            fileSize = FileTransferUtils.readableFileSize(imData.fileSize)
            checkedForArticle = true
        }
        deleted = imData.deleted
        fileTransferProgress = imData.fileTransferState
        calculateDateFormat()
    }

    constructor(imData : Message,context: Context) {
        id = imData.id
        conversationId = imData.chatId
        status = imData.status
        msgType = getTypeFromStatus(imData.status)
        this.context = context
        time = imData.creationTime
        modTime = imData.modificationTime
        message = imData.text
        participantName = imData.remoteAddress
        jid = imData.remoteAddress
        error = (imData.status == ImStatusData.OUTGOING_SENDING_FAILED) || (imData.status == ImStatusData.OUTGOING_NOT_DELIVERED)
        isRead = imData.read
        externalId = imData.externalId
        filePath = imData.filePath
        if (!TextUtils.isEmpty(filePath)) {
            val file = File(filePath)
            fileName = file.name
            fileExtension = file.extension
            fileSize = FileTransferUtils.readableFileSize(imData.fileSize)
            checkedForArticle = true
        }
        deleted = imData.deleted
        fileTransferProgress = imData.fileProgress
        calculateDateFormat()
    }

    /**
     * Constructor used for date separator
     *
     * @param id
     * @param time
     * @param messageType
     * @param context
     */
    constructor(id: Long, time: Long, messageType: Int, context: Context) {
        this.id = id
        this.time = time
        this.msgType = messageType
        calculateDateFormat()
        this.message = this.dateString
        this.isRead = true
        this.context = context
    }

    private fun calculateDateFormat() {
        val sdf = SimpleDateFormat("HH:mm")
        dateString = DateUtils.getRelativeTimeSpanString(time, System.currentTimeMillis(), DateUtils.DAY_IN_MILLIS, DateUtils.FORMAT_SHOW_DATE) as String
        timeString = sdf.format(Date(time))
    }

    fun containsArticle(): Boolean = !TextUtils.isEmpty(articleUrl) && !TextUtils.isEmpty(articleTitle)

    private fun getTypeFromStatus(status: Int): Int {
        Log.d(TAG, "getTypeFromStatus: $status")
        return when {
            status < 10 -> MESSAGE_TYPE_OUTGOING
            status in 10..19 -> MESSAGE_TYPE_INCOMING
            status == 20 -> MESSAGE_TYPE_LOCAL
            this.getStatusPrefix() == ImStatusData.OUTGOING_FT_PREFIX -> MESSAGE_TYPE_OUTGOING_FILE_TRANSFER
            this.getStatusPrefix() == ImStatusData.INCOMING_FT_PREFIX -> MESSAGE_TYPE_INCOMING_FILE_TRANSFER
            this.getStatusPrefix() == ImStatusData.OUTGOING_IMAGE_TRANSFER_PREFIX -> MESSAGE_TYPE_OUTGOING_IMAGE_TRANSFER
            this.getStatusPrefix() == ImStatusData.INCOMING_IMAGE_TRANSFER_PREFIX -> MESSAGE_TYPE_INCOMING_IMAGE_TRANSFER
            else -> MESSAGE_TYPE_INCOMING
        }
    }

    fun addArticleResult(result: ArticleResult) {
        if (result.isVccs) {
            this.isVccsInviteLink = true
        }
        this.articleDetails = result.details
        this.articlePhotoUrl = result.photoUrl
        this.articleTitle = result.title
        this.articleUrl = result.url
        this.checkedForArticle = true
    }

    override fun compareTo(other: MessageListItemData): Int {
        return when {
            this.time > other.time -> 1
            this.time == other.time -> 0
            else -> if (this.time < other.time) -1 else 0
        }
    }

    companion object {

        private val TAG = "MessageListItemData"
        const val MESSAGE_TYPE_OUTGOING = 1
        const val MESSAGE_TYPE_INCOMING = 0
        const val MESSAGE_TYPE_LOCAL = 3
        const val MESSAGE_TYPE_COMPOSING = 4
        const val MESSAGE_TYPE_DATE_DIVIDER = 5
        const val MESSAGE_TYPE_INCOMING_FILE_TRANSFER = 6
        const val MESSAGE_TYPE_OUTGOING_FILE_TRANSFER = 7
        const val MESSAGE_TYPE_INCOMING_IMAGE_TRANSFER = 8
        const val MESSAGE_TYPE_OUTGOING_IMAGE_TRANSFER = 9

        /**
         * @param time UTC time in ms,
         *
         * @return Returns Date Divider Item for conversation based on time. - creation time is midnight of the current date - type is date divider
         */
        fun buildDateDivider(time: Long, context: Context): MessageListItemData {
            val timeMidnight = ImpsUtils.getLocalMidnightTime(time)
            return MessageListItemData(timeMidnight, timeMidnight, MESSAGE_TYPE_DATE_DIVIDER, context)
        }
    }
}


