package com.bria.common.util

import android.app.Application
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.SettingsObservable
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers

object LogConfigurator {

    private val TAG = "LogConfigurator"

    private var mDisposable: Disposable? = null

    // workaround for null context in subscribe callback. could be kotlin compiler issue
    private lateinit var mContext: Application

    fun applyBuildConfigurationAndSettings(context: Application) {

        mContext = context

        AndroidLog.d(TAG, "initialize logging [start]") // NOTE: AndroidLog required at this place

        // app_disable_logs feature
        val disableLogs = Utils.canDisableLogs(context)
        Log.setEnabled(!disableLogs)

        // initial log
        AndroidLog.d(TAG, "onCreate(), pid = " + android.os.Process.myPid()) // NOTE: AndroidLog required at this place

        Log.initialize(context)

        val logFileName = Utils.Build.getProjectName(context) + ".log" // Bria.log
        Log.setFileName(logFileName)
        Log.setDestination(Log.ANDROIDLOG) // enable Android log (logcat) by default
        if (!Utils.Build.isGoodDynamicsBuild(context)) {
            Log.addDestination(Log.LOGFILE) // enable log file (Bria.log)
        }

        Log.d(TAG, "init() called, appContext = " + this + ", pid = " + android.os.Process.myPid())

        LogUtils.logGeneralInfo(context)

        Log.d(TAG, "initialize logging [end]")

        mDisposable = SettingsObservable
            .getObservable()
            .observeOn(Schedulers.computation())
            .subscribe({
                if (it.getBool(ESetting.FeatureSimplifiedClientLog)) {
                    Log.addDestination(Log.SIMPLIFIEDLOGFILE)
                }

                // Determine whether all logging should be enabled or disabled
                // To disable logging, app_disable_logs must be set to 1 (build-time)
                val enableLogging = !Utils.canDisableLogs(mContext) || it.getBool(ESetting.VerboseLogging)
                Log.setEnabled(enableLogging) // turn Java logging on or off

            }, {
                Utils.crashInDebug(TAG, it)
            })
    }

    fun shutdown(){
        mDisposable?.dispose()
    }
}
