package com.bria.common.util.im

import android.text.Editable

/*
* This class is needed because SDK relies on Protocol buffers and Protocol buffers library
* does not tolerate unpaired surrogates.
* */
object SdkStringCutter {
    fun truncate(s: String, maxSize: Int): String {
        if (s.length < maxSize) {
            return s
        }

        if (maxSize == 0) {
            return ""
        }

        val lastChar = s[maxSize - 1]

        if (lastChar.isHighSurrogate()) {
            return s.substring(0, maxSize - 1)
        } else {
            return s.substring(0, maxSize)
        }
    }

    // modifies editable inplace
    fun truncate(s: Editable, maxSize: Int): Unit {
        if (s.length < maxSize) {
            return
        }

        if (maxSize == 0) {
            s.clear()
            return
        }

        val lastChar = s[maxSize - 1]

        if (lastChar.isHighSurrogate()) {
            s.delete(maxSize - 1, s.length)
        } else {
            s.delete(maxSize, s.length)
        }
    }

    /*
    * Splits string s to parts each long maxSize characters or less.
    * */
    fun splitWithMaxSize(s: String?, maxSize: Int): List<String> {
        if (s.isNullOrEmpty()) {
            return listOf()
        }

        if (s!!.length <= maxSize) {
            return listOf(s)
        }

        fun splitToTwo(s: String, maxSize: Int): Pair<String, String> {
            if (s.length <= maxSize) {
                return Pair(s, "")
            }

            val lastChar = s[maxSize - 1]
            if (lastChar.isHighSurrogate()) {
                return Pair(s.substring(0, maxSize - 1), s.substring(maxSize - 1))
            } else {
                return Pair(s.substring(0, maxSize), s.substring(maxSize))
            }
        }

        val result = mutableListOf<String>()

        var pair = splitToTwo(s, maxSize)
        do {
            result.add(pair.first)
            pair = splitToTwo(pair.second, maxSize)
        } while (pair.second.isNotEmpty())

        result.add(pair.first)

        return result
    }
}
