package com.bria.common.util.reactive.api

import io.reactivex.Scheduler
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import io.reactivex.schedulers.TestScheduler

/**
 * Provides RxJava schedulers to other classes as an external source.
 * For tests, all functions should probably return [Schedulers.trampoline].
 */
interface SchedulerProvider {

    /**
     * Production: [Schedulers.io], test: [Schedulers.trampoline].
     */
    fun io(): Scheduler

    /**
     * Production: [Schedulers.computation], test: [Schedulers.trampoline].
     */
    fun computation(): Scheduler

    /**
     * Production: [AndroidSchedulers.mainThread], test: [Schedulers.trampoline].
     */
    fun main(): Scheduler

    /**
     * Always a new [TestScheduler].
     */
    fun test(): TestScheduler

}
