package com.bria.voip.ui.main.calllog.refactoring

import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import android.text.format.DateUtils
import com.bria.common.controller.calllog.db.CallType
import com.bria.common.uireusable.datatypes.IdentifiableDataItem

abstract class CallLogItemModel : IdentifiableDataItem {
    open var isGroup = false
        protected set
}

data class CallLogGroupItem(
        val id: String = "",
        var contactId: String = "",
        val number: String = "",
        var numberType: String = "",
        var displayName: String = "",
        var contactPhoto: Bitmap? = null,
        var contactPresence: Drawable? = null,
        var isContactPresenceVisible: Boolean = false,
        val date: Long = System.currentTimeMillis(),
        val callTypes: MutableList<CallType> = mutableListOf(),
        val items : MutableList<CallLogContentItem> = mutableListOf(),
        // Below not used in adapter directly
        val accountNickname: String = "",
        val accountDomain: String = "",
        val remoteName: String = "",
        val remoteDomain: String = "",
        val hostedCollab: Boolean = false,
        var externalId: String = "" // Optional. Can be used to store Genband Key or Broadworks remote log ID
) : CallLogItemModel() {
    override var isGroup = true
    var isExpanded = false

    override fun getUniqueIdentifier() = id

    fun getFormattedNumberTypeAndData() : String {
        val relativeTime = DateUtils.getRelativeTimeSpanString(date, System.currentTimeMillis(),
                DateUtils.MINUTE_IN_MILLIS, DateUtils.FORMAT_ABBREV_RELATIVE).toString()
        val typeAndDate = if (numberType.isNotEmpty()) "$numberType, $relativeTime" else relativeTime
        return if (items.size > callTypes.size) "(${items.size}) $typeAndDate" else typeAndDate
    }

    fun getEntityId() = items[0].id.toLong() // Not empty list expected here, so let it crash otherwise

    fun isRecordIndicatorVisible() = items.find { it.isRecordInfoVisible } != null
}

data class CallLogContentItem(
        val id: String,
        val callType: CallType,
        val callInfo: String,
        val callDate: String,
        val callDuration: String,
        val isCallDurationVisible: Boolean,
        val isRecordInfoVisible: Boolean,
        val recordInfo: String,
        val recordPath: String
) : CallLogItemModel() {
    override var isGroup = false

    override fun getUniqueIdentifier() = id
}
