package com.bria.voip.ui.main.calllog.refactoring

import android.support.v7.widget.CardView
import android.support.v7.widget.RecyclerView
import android.view.ContextMenu
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.uiframework.lists.recycler.AbstractRecyclerAdapter
import com.bria.common.uireusable.dataprovider.ISimpleDataProvider
import com.bria.common.util.kotlin.setGone
import com.bria.common.util.kotlin.setInvisible
import com.bria.common.util.kotlin.setVisible
import com.bria.voip.R
import de.hdodenhof.circleimageview.CircleImageView

private const val TYPE_ITEM_GROUP = 0
private const val TYPE_ITEM_CONTENT = 1
private const val TYPE_ITEM_CONTENT_RECORD = 2

class CallLogListAdapter(recyclerView: RecyclerView, dataProvider: ISimpleDataProvider<CallLogItemModel>) :
        AbstractRecyclerAdapter<CallLogItemModel, AbstractRecyclerAdapter.AbstractViewHolder>(recyclerView, dataProvider,
                R.layout.calllog_list_item_group,
                R.layout.calllog_list_item_content,
                R.layout.calllog_list_item_content_record) {

    var onCreateContextMenuListener: ((ContextMenu, View, ContextMenu.ContextMenuInfo?, Int) -> Any)? = null
    var onItemClickListener: ((View, Int) -> Any)? = null
    var onItemContactButtonClickListener: ((View, Int) -> Any)? = null
    var onItemCallButtonClickListener: ((View, Int) -> Any)? = null
    var onItemDeleteRecordButtonClickListener: ((View, Int) -> Any)? = null
    var onItemSaveRecordButtonClickListener: ((View, Int) -> Any)? = null
    var onItemShareRecordButtonClickListener: ((View, Int) -> Any)? = null
    var onItemPlayRecordButtonClickListener: ((View, Int) -> Any)? = null

    class GroupViewHolder(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val cardView: CardView = itemView.findViewById(R.id.calllog_list_item_group_card)
        val contactPhoto: CircleImageView = itemView.findViewById(R.id.calllog_list_item_group_contact_photo)
        val contactPresence: ImageView = itemView.findViewById(R.id.calllog_list_item_group_contact_presence)
        val recordIndicator: ImageView = itemView.findViewById(R.id.calllog_list_item_group_record_indicator)
        val displayName: TextView = itemView.findViewById(R.id.calllog_list_item_group_display_name)
        val callTypeIcons: CallTypeView = itemView.findViewById(R.id.calllog_list_item_group_call_type_icons)
        val numberTypeAndDate: TextView = itemView.findViewById(R.id.calllog_list_item_group_number_type_and_date)
        val callButton: ImageView = itemView.findViewById(R.id.calllog_list_item_group_call_button)

        override fun hasDivider() = false
        override fun isClickable() = false
        override fun isLongClickable() = false
    }

    class ContentViewHolder(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val callTypeIcon: CallTypeView = itemView.findViewById(R.id.calllog_list_item_content_call_type_icon)
        val callInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_info)
        val callDuration: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_duration)
        val callDate: TextView = itemView.findViewById(R.id.calllog_list_item_content_call_date)

        override fun hasDivider() = false
        override fun isClickable() = false
        override fun isLongClickable() = false
    }

    class ContentRecordViewHolder(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val callTypeIcon: CallTypeView = itemView.findViewById(R.id.calllog_list_item_content_record_call_type_icon)
        val callInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_call_info)
        val callDuration: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_call_duration)
        val callDate: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_call_date)
        val recordInfo: TextView = itemView.findViewById(R.id.calllog_list_item_content_record_file_info)

        override fun hasDivider() = false
        override fun isClickable() = false
        override fun isLongClickable() = false
    }

    override fun getItemViewType(position: Int): Int {
        val item = getItemAt(position)!!
        return if (item.isGroup)
            TYPE_ITEM_GROUP
        else if (item is CallLogContentItem && item.isRecordInfoVisible)
            TYPE_ITEM_CONTENT_RECORD
        else
            TYPE_ITEM_CONTENT
    }

    override fun provideNewViewHolder(inflatedView: View, viewType: Int) : AbstractRecyclerAdapter.AbstractViewHolder? {
        return when (viewType) {
            TYPE_ITEM_GROUP -> CallLogListAdapter.GroupViewHolder(inflatedView)
            TYPE_ITEM_CONTENT -> CallLogListAdapter.ContentViewHolder(inflatedView)
            TYPE_ITEM_CONTENT_RECORD -> CallLogListAdapter.ContentRecordViewHolder(inflatedView)
            else -> null
        }
    }

    override fun onNewViewHolderReady(viewHolder: AbstractRecyclerAdapter.AbstractViewHolder, viewType: Int) { }

    override fun onUpdateNeeded(viewHolder: AbstractRecyclerAdapter.AbstractViewHolder, position: Int) {
        when (viewHolder.itemViewType) {
            TYPE_ITEM_GROUP -> updateGroupViewHolder(
                    viewHolder as CallLogListAdapter.GroupViewHolder, getItemAt(position) as CallLogGroupItem)
            TYPE_ITEM_CONTENT -> updateContentViewHolder(
                    viewHolder as CallLogListAdapter.ContentViewHolder, getItemAt(position) as CallLogContentItem)
            TYPE_ITEM_CONTENT_RECORD -> updateContentRecordViewHolder(
                    viewHolder as CallLogListAdapter.ContentRecordViewHolder, getItemAt(position) as CallLogContentItem)
        }
    }

    private fun updateGroupViewHolder(viewHolder : CallLogListAdapter.GroupViewHolder, dataItem : CallLogGroupItem) {
        viewHolder.displayName.text = dataItem.displayName
        viewHolder.callTypeIcons.apply {
            clear()
            dataItem.callTypes.forEach { add(it) }
            requestLayout()
        }

        viewHolder.numberTypeAndDate.text = dataItem.getFormattedNumberTypeAndData()

        viewHolder.contactPhoto.apply {
            when {
                dataItem.contactPhoto != null -> setImageBitmap(dataItem.contactPhoto)
                else -> setImageResource(R.drawable.default_avatar)
            }
        }

        viewHolder.contactPresence.apply {
            setImageDrawable(dataItem.contactPresence)
            if (dataItem.isContactPresenceVisible) setVisible() else setInvisible()
        }

        viewHolder.recordIndicator.apply { if (dataItem.isRecordIndicatorVisible()) setVisible() else setInvisible() }

        viewHolder.cardView.setOnCreateContextMenuListener { menu, view, menuInfo ->
            onCreateContextMenuListener?.invoke(menu, view, menuInfo, viewHolder.adapterPosition) }

        viewHolder.cardView.setOnClickListener { view -> onItemClickListener?.invoke(view, viewHolder.adapterPosition) }
        viewHolder.contactPhoto.setOnClickListener { view -> onItemContactButtonClickListener?.invoke(view, viewHolder.adapterPosition) }
        viewHolder.callButton.setOnClickListener { view -> onItemCallButtonClickListener?.invoke(view, viewHolder.adapterPosition) }
    }

    private fun updateContentViewHolder(viewHolder : CallLogListAdapter.ContentViewHolder, dataItem : CallLogContentItem) {
        viewHolder.callTypeIcon.apply {
            clear()
            add(dataItem.callType)
            requestLayout()
        }
        viewHolder.callInfo.text = dataItem.callInfo
        viewHolder.callDate.text = dataItem.callDate
        viewHolder.callDuration.apply {
            text = dataItem.callDuration
            if (dataItem.isCallDurationVisible) setVisible() else setGone()
        }
    }

    private fun updateContentRecordViewHolder(viewHolder : CallLogListAdapter.ContentRecordViewHolder, dataItem : CallLogContentItem) {
        viewHolder.callTypeIcon.apply {
            clear()
            add(dataItem.callType)
            requestLayout()
        }
        viewHolder.callInfo.text = dataItem.callInfo
        viewHolder.callDate.text = dataItem.callDate
        viewHolder.callDuration.apply {
            text = dataItem.callDuration
            if (dataItem.isCallDurationVisible) setVisible() else setGone()
        }
        viewHolder.recordInfo.text = dataItem.recordInfo
    }
}
