package com.bria.voip.ui.main.calllog.refactoring

import android.content.Context
import android.graphics.Canvas
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.View
import com.bria.common.controller.calllog.db.CallType
import com.bria.voip.R

/**
 * View that draws one or more symbols for different types of calls (missed calls, outgoing etc).
 * As this view doesn't create subviews, it is better suited for recycling that a regular LinearLayout using ImageViews.
 */
class CallTypeView(context: Context, attrs: AttributeSet?) : View(context, attrs) {
    private var mCallTypes: MutableList<CallType> = ArrayList(3)
    private var mWidth: Int = 0
    private var mHeight: Int = 0

    private class Resources(context: Context) {
        val incoming: Drawable = context.resources!!.getDrawable(R.drawable.call_incoming)
        val outgoing: Drawable = context.resources!!.getDrawable(R.drawable.call_outgoing)
        val missed: Drawable = context.resources!!.getDrawable(R.drawable.call_missed)
//        val incoming: Drawable = context.resources!!.getDrawable(R.drawable.call_log_incoming)
//        val outgoing: Drawable = context.resources!!.getDrawable(R.drawable.call_log_outgoing)
//        val missed: Drawable = context.resources!!.getDrawable(R.drawable.call_log_missed)
        val voiceMail: Drawable = context.resources!!.getDrawable(R.drawable.ic_voice_mail)
        val conference: Drawable = context.resources!!.getDrawable(R.drawable.call_log_vccsconference)
        val iconMargin: Int = context.resources!!.getDimensionPixelSize(R.dimen.spacing_micro)
    }

    companion object {
        private var sResources: Resources? = null
    }

    init {
        if (sResources == null) {
            sResources = Resources(context)
        }
    }

    fun clear() {
        mCallTypes.clear()
        mWidth = 0
        mHeight = 0
        invalidate()
    }

    fun add(callType: CallType) {
        mCallTypes.add(callType)
        val drawable = getCallTypeDrawable(callType)
        mWidth += drawable.intrinsicWidth + sResources!!.iconMargin
        mHeight = Math.max(mHeight, drawable.intrinsicHeight)
        invalidate()
    }

    private fun getCallTypeDrawable(callType: CallType) : Drawable {
        return when (callType) {
            CallType.INCOMING -> sResources!!.incoming
            CallType.OUTGOING -> sResources!!.outgoing
            CallType.MISSED -> sResources!!.missed
            CallType.VOICE_MAIL -> sResources!!.voiceMail
            CallType.CONFERENCE -> sResources!!.conference
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        setMeasuredDimension(mWidth, mHeight)
    }

    override fun onDraw(canvas: Canvas?) {
        var left = 0
        for (callType in mCallTypes) {
            val drawable = getCallTypeDrawable(callType)
            val right = left + drawable.intrinsicWidth
            drawable.setBounds(left, 0, right, drawable.intrinsicHeight)
            drawable.draw(canvas)
            left = right + sResources!!.iconMargin
        }
    }
}
