package com.bria.voip.ui.main.contacts

import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.ContactDataProviderV2
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.common.uireusable.dataprovider.ISimpleDataProvider
import com.bria.common.util.GlobalConstants

open class V2ContactListPresenter : AbstractPresenter() {
    enum class Events : IPresenterEventTypeEnum {
        FILTER_TYPE_CHANGED;


        override fun getType(): IPresenterEventTypeEnum {
            return this
        }
    }

    fun initContact() {
        initContactsDataProvider()
    }


    private lateinit var mContactsDataProvider: ContactDataProviderV2

    private fun initContactsDataProvider() {
        mContactsDataProvider = ContactDataProviderV2(context)
        mContactsDataProvider.attachListener(object : IFilterableDataProvider.DataProviderListener {

            override fun onDataLoaded(isEmpty: Boolean) {
                firePresenterEvent(Events.FILTER_TYPE_CHANGED)

            }

            override fun onDataLoading() {
                //Implement status loading
            }
        })
        mContactsDataProvider.initialize()
    }

    private var mActiveTab = GlobalConstants.ETab.CONTACTS

    fun setActiveTab(activeTab: GlobalConstants.ETab) {
        mActiveTab = activeTab
    }

    fun getActiveTab(): GlobalConstants.ETab {
        return mActiveTab
    }

    fun changedTabSelected(activeTab: GlobalConstants.ETab) {
        setActiveTab(activeTab)
        firePresenterEvent(Events.FILTER_TYPE_CHANGED)
    }

    fun getDataProvider(): ISimpleDataProvider<out Any>? {
        if (mActiveTab == GlobalConstants.ETab.CONTACTS) {
            return mContactsDataProvider

        }
        if (mActiveTab == GlobalConstants.ETab.BUDDIES) {
            return mControllers.buddy.getDataProvider();
        }


        if (mActiveTab == GlobalConstants.ETab.FAVORITES) {
            return null
        }
        return null
    }


}
