package com.bria.voip.ui.main.contacts

import android.Manifest
import android.content.pm.PackageManager
import android.os.Bundle
import android.support.annotation.MainThread
import android.support.design.widget.TabLayout
import android.support.v7.widget.AppCompatSpinner
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.widget.Button
import android.widget.LinearLayout
import android.widget.TextView
import com.bria.common.controller.settings.ESetting
import com.bria.common.customelements.internal.views.indexer.IndexLettersView
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.branding.AbstractCustomizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.lists.recycler.AbstractRecyclerAdapter
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uireusable.CustomSearchView
import com.bria.common.uireusable.adapters.GenericPersonAdapter
import com.bria.common.util.GlobalConstants
import com.bria.common.util.LoaderFactory
import com.bria.common.util.Log
import com.bria.voip.R
import com.bria.voip.ui.main.dialer.MultiStateViewHelper
import com.kennyc.view.MultiStateView
import java.util.*

@Layout(R.layout.v2_contact_list_screen)
@com.bria.common.uiframework.annotations.Menu(R.menu.contact_list_menu)
open class V2ContactListScreen<Presenter : V2ContactListPresenter> : AbstractScreen<Presenter>() {
    private val TAG = "ContactsListScreen"

    @Inject(id = R.id.v2_contacts_list_screen_search_view, col = false)
    protected var mSearchView: CustomSearchView? = null

    @Inject(id = R.id.screen_toolbar_left_spinner, col = false)
    private val mSpinner: AppCompatSpinner? = null

    @Inject(id = R.id.v2_contacts_list_screen_tabs)
    private lateinit var mTabWidget: TabLayout

    protected lateinit var mPersonsList: RecyclerView

    @Inject(id = R.id.v2_contacts_list_screen_requests_buttons, col = false)
    private val mBuddyRequestsButtons: LinearLayout? = null

    @Inject(id = R.id.v2_contacts_list_screen_accept_all, back = ESetting.ColorBrandTint, inv = true, tag = AbstractCustomizer.Tag.FCS)
    @Clickable
    private val mButtonAcceptAll: TextView? = null

    @Clickable
    @Inject(id = R.id.v2_contacts_list_screen_decline_all, back = ESetting.ColorBrandTint, inv = true, tag = AbstractCustomizer.Tag.FCS)
    private val mButtonDeclineAll: TextView? = null

    private var mIndexLetterScroller: IndexLettersView? = null

    @Inject(id = R.id.v2_contacts_list_screen_touched_letter, back = ESetting.ColorBrandTint, tag = AbstractCustomizer.Tag.FCS, inv = true)
    private val mTouchedIndexLetter: TextView? = null

    protected lateinit var mAdapter: AbstractRecyclerAdapter<*, *>


    @Suppress("UNCHECKED_CAST")
    override fun getPresenterClass(): Class<Presenter> = V2ContactListPresenter::class.java as Class<Presenter>

    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {

        if (event.type is V2ContactListPresenter.Events) {
            val type = event.type as V2ContactListPresenter.Events
            Log.i(TAG, "Presenter Event: " + type.name)
            when (type) {
                V2ContactListPresenter.Events.FILTER_TYPE_CHANGED -> {
                    reloadDataScreen()

                }
            }
        }
    }

    fun reloadDataScreen() {
        setDataProvider()
        mPersonsList.adapter = mAdapter
        post{mAdapter.notifyDataChanged()}
    }

    private fun setDataProvider() {
        val dataProvider = presenter.getDataProvider()
        if (dataProvider == null) {
            Log.w(TAG, "setupAdapter: data provider is null")
            return
        }
        mAdapter.dataProvider = dataProvider
    }

    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
        presenter.subscribe(this)
    }

    override fun onStop(finishing: Boolean) {
        super.onStop(finishing)
        presenter.unsubscribe()
    }

    override fun getTitle(): String? = "NEW CONTACT LIST SCREEN"

    private lateinit var mMultiStateHelper: MultiStateViewHelper

    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)
        mMultiStateHelper = MultiStateViewHelper(layout)
        mPersonsList = mMultiStateHelper.getRecyclerView()
        val mLinearLayoutManager = LinearLayoutManager(activity)
        mPersonsList.setLayoutManager(mLinearLayoutManager)

        updateTabs()
        initAdapter()
        presenter.initContact()


    }

    private fun initRequestPermissionButton() {
        val multiStateView = mMultiStateHelper.multiStateView

        multiStateView.setViewForState(R.layout.missing_permission_error_state, MultiStateView.VIEW_STATE_ERROR)
        val permissionErrorView = multiStateView.getView(MultiStateView.VIEW_STATE_ERROR)
        if (permissionErrorView != null) {
            val textView = permissionErrorView.findViewById<TextView>(R.id.missing_permission_error_state_text_view)
            textView.text = getString(R.string.tContactsListPermissionError)
            val button = permissionErrorView.findViewById<Button>(R.id.missing_permission_error_state_request_button)
            button.setOnClickListener { v ->
                requestPermission(
                        Manifest.permission.READ_CONTACTS,
                        GlobalConstants.CP_PERMISSION_INIT_CONTACTS,
                        getString(R.string.tContactsListPermissionRequestExplanation))
            }
        }

    }

    fun initAdapter() {
        mAdapter = GenericPersonAdapter(mPersonsList, LoaderFactory.createContactImageLoader(activity), LoaderFactory
                .createPresenceLoader(activity))
    }


    fun createTabs(): List<TabLayout.Tab> {
        val tabList = ArrayList<TabLayout.Tab>(5)
        val contactsTab = mTabWidget
                .newTab()
                .setText(getString(R.string.tSwitchToContacts))
                .setTag(GlobalConstants.ETab.CONTACTS)

        tabList.add(contactsTab)
        val buddiesTab = mTabWidget
                .newTab()
                .setText(getString(R.string.tSwitchToBuddies))
                .setTag(GlobalConstants.ETab.BUDDIES)

        tabList.add(buddiesTab)

        val favoritesTab = mTabWidget
                .newTab()
                .setText(getString(R.string.tFavorites))
                .setTag(GlobalConstants.ETab.FAVORITES)

        tabList.add(favoritesTab)

        return tabList
    }


    private fun updateTabs() {
        mTabWidget.removeAllTabs()
        mTabWidget.removeOnTabSelectedListener(mTabListener)

        val tabList = createTabs()

        for (tab in tabList) {
            mTabWidget.addTab(tab)
        }

        recolorTabs()
        updateSelectedTab() // set initial selection

        mTabWidget.addOnTabSelectedListener(mTabListener)
    }

    private fun recolorTabs() {
        val customizer = Coloring.get().getCustomizer(TabLayout::class.java)
        customizer.setTarget(mTabWidget)
        customizer.applyChanges()
    }

    private fun updateSelectedTab() {
        val activeTab = presenter.getActiveTab()
        for (i in 0 until mTabWidget.getTabCount()) {
            val tab = mTabWidget.getTabAt(i)
            if (tab != null && tab.getTag() === activeTab && !tab.isSelected()) {
                tab.select()
            }
        }
    }


    private val mTabListener = object : TabLayout.OnTabSelectedListener {
        override fun onTabSelected(tab: TabLayout.Tab) {
            if (tab.tag != null) {
                //TODO Implemented
                // saveLayoutState()
                presenter.changedTabSelected(tab.tag as GlobalConstants.ETab)

            }
        }

        override fun onTabUnselected(tab: TabLayout.Tab) {}

        override fun onTabReselected(tab: TabLayout.Tab) {}
    }

    override fun onResume() {
        super.onResume()
        setVisibilities()
    }

    private fun setVisibilities() {

        val filterType = presenter.getActiveTab()
        val noPermission = (filterType == GlobalConstants.ETab.CONTACTS || filterType == GlobalConstants.ETab.FAVORITES) && !checkPermission(Manifest.permission.READ_CONTACTS)
        if (noPermission) {
            debug("setVisibilities: no Permission")
            mMultiStateHelper.multiStateView.viewState = MultiStateView.VIEW_STATE_ERROR
            initRequestPermissionButton()
        } else

        /* mBuddyRequestsButtons.setVisibility(if (presenter.areRequestsButtonsVisible()) View.VISIBLE else View.GONE)
         mSpinner.setVisibility(if (presenter.isSpinnerVisible()) View.VISIBLE else View.GONE)

         val filterType = presenter.getActiveFilterType()
         val noPermission = (filterType == CONTACTS || filterType == FAVORITES) && !checkPermission(Manifest.permission.READ_CONTACTS)
         if (noPermission) {
             debug("setVisibilities: no Permission")
             mMultiStateHelper.getMultiStateView().setViewState(MultiStateView.VIEW_STATE_ERROR)
         } else if (presenter.isLoading()) {
             // still loading data
             debug("setVisibilities: Still Loading")
             mMultiStateHelper.getMultiStateView().setViewState(MultiStateView.VIEW_STATE_LOADING)
             mMultiStateHelper.getNoDataView().setText(presenter.getLoadingString())
         } else if (presenter.getItemsCount() == 0) {
             // loaded, but no contacts
             debug("setVisibilities: Loaded, empty")
             mMultiStateHelper.getMultiStateView().setViewState(MultiStateView.VIEW_STATE_EMPTY)

             if (filterType == LDAP) {
                 mMultiStateHelper.getNoDataView().setText(presenter.getExtendedErrorDescription())
             } else {
                 mMultiStateHelper.getNoDataView().setText(presenter.getNoDataString())
             }
         } else {
             // contacts loaded and present
             debug("setVisibilities: Loaded and Present")*/

            mMultiStateHelper.getMultiStateView().setViewState(MultiStateView.VIEW_STATE_CONTENT)
        /*      val indexVisible = mIndexLetterScroller.isSectionsReady() && TextUtils.isEmpty(presenter.getSearchQuery())
              mIndexLetterScroller.setVisibility(if (indexVisible) View.VISIBLE else View.GONE)
              if (indexVisible) {
                  mIndexLetterScroller.bringToFront()
              }*/
    }

    @MainThread
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        when (requestCode) {
            GlobalConstants.CP_PERMISSION_INIT_CONTACTS -> if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                debug("CP_PERMISSION_INIT_CONTACTS Granted")
                presenter.initContact()
                reloadDataScreen()
                mMultiStateHelper.getMultiStateView().setViewState(MultiStateView.VIEW_STATE_CONTENT)
            }
            GlobalConstants.CP_PERMISSION_WRITE_CONTACTS -> if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                debug("CP_PERMISSION_WRITE_CONTACTS Granted")
                //presenter.screenAction(ContactsListPresenter.EScreenActions.WRITE_CONTACTS_PERMISSION_GRANTED)
            } else if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_DENIED) {
                debug("CP_PERMISSION_WRITE_CONTACTS Not Granted")
                //presenter.screenAction(ContactsListPresenter.EScreenActions.WRITE_CONTACTS_PERMISSION_DENIED)
            }
        }
    }
}
