package com.bria.voip.ui.main.im

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Intent
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.provider.ContactsContract
import android.speech.RecognizerIntent
import android.support.annotation.MainThread
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.TextUtils
import com.bria.common.R
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.ClientConfig
import com.bria.common.controller.Controllers
import com.bria.common.controller.Observables
import com.bria.common.controller.accounts.*
import com.bria.common.controller.collaboration.rx.impl.ParticipantManager
import com.bria.common.controller.contacts.buddy.VCard
import com.bria.common.controller.contacts.buddy.XmppBuddy
import com.bria.common.controller.contacts.local.Contact
import com.bria.common.controller.contacts.local.data.PhoneNumber
import com.bria.common.controller.im.ImController.MAX_GROUP_CHAT_PARTICIPANTS
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.*
import com.bria.common.controller.im.refactoring.adapter.IDataProviderEvents
import com.bria.common.controller.im.refactoring.dataprovider.RxInstantMessageDataProvider
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData.GROUPCHAT_XMPP
import com.bria.common.controller.im.refactoring.db.entities.isTransferValidForForwarding
import com.bria.common.controller.presence.IPresenceCtrlObserver
import com.bria.common.controller.presence.OwnPresence
import com.bria.common.controller.presence.refactoring.EPresenceStatus
import com.bria.common.controller.provisioning.core.EProvisioningState
import com.bria.common.controller.remotesync.RemoteSyncUtil
import com.bria.common.controller.settings.EAccountSetting
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.branding.EAccountType
import com.bria.common.controller.settings.core.ISettingsObserver
import com.bria.common.mdm.MdmUtils
import com.bria.common.modules.BriaGraph
import com.bria.common.network.INetworkObserver
import com.bria.common.network.NetworkModule
import com.bria.common.sdkwrapper.CallData
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.RoundedBackgroundSpan
import com.bria.common.uireusable.datatypes.ExtendedConversationData
import com.bria.common.uireusable.datatypes.ExtendedConversationData.Companion.CHAT_ROOM_TYPE
import com.bria.common.util.*
import com.bria.common.util.GlobalConstants.*
import com.bria.common.util.im.Participant
import com.bria.common.util.im.ParticipantsSet
import com.bria.common.util.im.SdkStringCutter
import com.bria.voip.ui.main.contacts.AddXmppBuddyPresenter
import com.bria.voip.ui.main.contacts.ContactEditPresenter
import com.bria.voip.ui.main.im.ConversationListDataProvider.Companion.FILTER_TYPE_IM
import com.bria.voip.ui.main.im.ConversationListDataProvider.Companion.FILTER_TYPE_SMS
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import java.util.*


open class ConvPresenter : AbstractPresenter(), IPresenceCtrlObserver, IAccountsCtrlObserver, INetworkObserver,
        ISettingsObserver, IDataProviderEvents {

    companion object {
        val KEY_CHAT_ACTION = "ACTION"
        val KEY_CHAT_MESSAGE = "MESSAGE"
        val ACTION_FORWARD = "FORWARD"
    }

    private val TAG = "ConvPresenter"

    val dataProvider = RxInstantMessageDataProvider(context, this)
    internal val conversationData = ExtendedConversationData(context)
    var contact: Contact? = null

    val mNetworkModule: NetworkModule?= BriaGraph.networkModule

    private val observedSettings = EnumSet.of(ESetting.ImPresence, ESetting.Sms)

    private var filesToSend : Array<String> = emptyArray()

    fun initConversation(bundle : Bundle) {
        ThreadExecutors.scheduleLightWork {
            var prepared = false
            if (isValidBundle(bundle)) {
                if (bundle.containsKey(GlobalConstants.KEY_CHAT_JID)) {
                    prepared = prepareCollabChat(bundle)
                } else if (bundle.containsKey(GlobalConstants.KEY_CONVERSATION_ID)) {
                    prepared = prepareConversationById(bundle)
                } else if (bundle.containsKey(GlobalConstants.KEY_CHAT_ROOM_ID)) {
                    prepared = prepareChatRoomById(bundle)
                } else if (bundle.containsKey(GlobalConstants.KEY_BUDDY_IDS) || (bundle.containsKey(GlobalConstants.KEY_CHAT_RAW_SMS_NUMBERS)) ||
                        bundle.containsKey(GlobalConstants.KEY_CHAT_SMS_NUMBERS)){
                    prepared = prepareConversationByParticipants(bundle)
                }
                if (!prepared) {
                    cleanAndGoUp()
                }
            } else {
                Log.d(TAG, "bundle: " + bundle.toString())
                //Utils.crashInDebug(TAG, "Invalid bundle: " +bundle.toString(), IllegalStateException::class.java)

            }
            if (prepared) {
                if (conversationData.isSmsType()) {
                    val participantKey = conversationData.participantSet.firstParticipantKey
                    val number = BuddyKeyUtils.getImAdressFromNewBuddyKey(participantKey)
                    if (!TextUtils.isEmpty(number)) {
                        findContactByNumberAsync(number)
                    }
                }
                firePresenterEvent(Events.USER_TYPING, getChatApi().isTyping(conversationData.imConversationData))
                Log.d(TAG, "Init Conversation")
                publishActiveConversation()

                dataProvider.setImConversationData(conversationData.getId(), isHardwiredSession(), conversationData.isChatRoom())
                firePresenterEvent(Events.CONVERSATION_UPDATED)
                firePresenterEvent(Events.SET_INPUT_TEXT, getImData().getUnsentText(conversationData.getId()))
                if (bundle.containsKey(GlobalConstants.KEY_CHAT_INPUT_TEXT)) {
                    val message = bundle.getString(GlobalConstants.KEY_CHAT_INPUT_TEXT)
                    Log.d(TAG, "Trying to send message")
                    sendMessage(message)
                }
                if (bundle.containsKey(GlobalConstants.KEY_FILES_TO_SEND)) {
                    if (conversationData.isCreated()) {
                        val files = bundle.getStringArray(GlobalConstants.KEY_FILES_TO_SEND)
                        if (files.isNotEmpty()) {
                            getChatApi().sendFiles(files, conversationData.getId())
                        }
                    } else {
                        filesToSend = bundle.getStringArray(GlobalConstants.KEY_FILES_TO_SEND)
                    }
                }
            }
        }

    }

    private fun prepareChatRoomById(bundle: Bundle): Boolean {
        val chatRoomId = bundle.getLong(GlobalConstants.KEY_CHAT_ROOM_ID)
        val chatRoom = getImData().chatRoomRepo.getChatRoom(chatRoomId) ?: return false
        conversationData.setChatRoom(chatRoom)
        return true
    }

    private fun isValidBundle(bundle: android.os.Bundle?): Boolean {
        return bundle != null && (bundle.containsKey(KEY_BUDDY_IDS) || bundle.containsKey(KEY_CONVERSATION_ID)
                || bundle.containsKey(KEY_CHAT_ROOM_ID) || bundle.containsKey(KEY_CHAT_SMS_NUMBERS) ||
                bundle.containsKey(KEY_CHAT_JID) || bundle.containsKey(KEY_CHAT_RAW_SMS_NUMBERS))
    }

    @SuppressLint("WrongConstant")
    @MainThread
    override fun onCreate() {
        super.onCreate()
        Observables.get().presence.attachObserver(this)
        Observables.get().accounts.attachObserver(this)
        Controllers.get().settings.attachObserver(this, observedSettings)
        mNetworkModule?.attachObserver(this)
        getImData().attachObserver(mImDataObserverAdapter)
    }

    @MainThread
    override fun onSubscribe() {
        super.onSubscribe()
        getChatApi().attachObserver(mChatApiObserver)
    }

    override fun onUnsubscribe() {
        super.onUnsubscribe()
        getChatApi().detachObserver(mChatApiObserver)
    }

    @MainThread
    override fun onDestroy() {
        super.onDestroy()

        Observables.get().presence.detachObserver(this)
        Observables.get().accounts.detachObserver(this)
        Controllers.get().settings.detachObserver(this)
        mNetworkModule?.detachObserver(this)
        getImData().detachObserver(mImDataObserverAdapter)
        dataProvider.clean()
        disposables.clear()
    }

    fun getTitle(): String = if (conversationData.isGroupChat()) getString(R.string.tGroupChat) else conversationData.getConversationTitle()

    fun isPresenceIconVisible() : Boolean{
        return !conversationData.isChatRoom()
    }

    @Suppress("DEPRECATION")
    open fun getPresenceIcon() : Drawable {
        if (conversationData.isChatRoom()) {
            return resources.getDrawable(R.drawable.icon_group_available)
        } else {
            if (conversationData.isGroupChat()) {
                return if (getChatApi().isActive(conversationData.getId())) {
                    resources.getDrawable(R.drawable.icon_group_available)
                } else {
                    resources.getDrawable(R.drawable.icon_group_disabled)
                }
            } else if (conversationData.isSmsType()) {
                return resources.getDrawable(R.drawable.im_sms_icon)
            } else if (!conversationData.participantSet.isEmpty() && conversationData.getFirstParticipant() != null) {
                return conversationData.getFirstParticipant()!!.getPresenceIcon(context)
            }
            return resources.getDrawable(R.drawable.ic_presence_unknown)
        }
    }

    open fun getPresenceNote(): String  = if (conversationData.getFirstParticipant() != null)
        conversationData.getFirstParticipant()!!.presence.getPresenceNote(context) else ""

    fun sendMessage(message: String): Boolean {
        var trimmedMessage = message.trim { it <= ' ' }
        if (!TextUtils.isEmpty(trimmedMessage)) {
            if (Controllers.get().settings.getBool(ESetting.FeatureInsertUnicodeEmoticons)) {
                trimmedMessage = EmoticonUtils.injectEmoticons(message)
            }
            if (!conversationData.isCreated()) {
                getChatApi().sendMessage(conversationData.participantSet, trimmedMessage, conversationType, getRemoteKey())
            } else {
                if (isChatRoom()) {
                    getChatApi().sendMessage(conversationData.getChatRoom(), trimmedMessage)
                } else {
                    getChatApi().sendMessage(conversationData.imConversationData, trimmedMessage)
                }
            }
            return true
        } else {
            return false
        }
    }

    fun shouldShowWifiWarningDialog() =
            mControllers.settings.getBool(ESetting.FeatureWifiOnlyRegistration) && BriaGraph.networkModule.connectionType != INetworkObserver.ENetworkType.WIFI

    val conversationType: Int get() = conversationData.getType()

    fun isSMS() = conversationData.isSmsType()

    fun getRemoteKey() = conversationData.getRemoteKey()

    val canCopy : Boolean get() = !Utils.Build.isGoodDynamicsBuild(context) && !MdmUtils.isDataLeakagePolicyEnforced()

    val isImType: Boolean get() = conversationData.isImType()

    fun getPhoneList(): java.util.ArrayList<String> = ArrayList<String>(conversationData.participantSet.setOfBuddyKeys)

    // add chat room flag
    private fun publishActiveConversation() {
        if (dataProvider.isEmpty() && !dataProvider.loading.get()) {
            Log.d(TAG, "Publish1: " + "data is empty")
            val bundle = Bundle()
            bundle.putString(ConversationListPresenter.KEY_ACTION, ConversationListPresenter.KEY_DESELECT_ALL_CONVERSATIONS)
            firePresenterEvent(Events.DESELECT_ALL_CONVERSATIONS, bundle)
            return
        }
        val bundle = Bundle()
        Log.d(TAG, "Publish2: " + "have conversation")
        if (conversationData.isCreated()) {
            Log.d(TAG, "Publish2: " + "conversation created")
            bundle.putLong(GlobalConstants.KEY_CONVERSATION_ID, conversationData.getId())
            bundle.putString(ConversationListPresenter.KEY_ACTION, ConversationListPresenter.KEY_SELECT_ACTIVE_CONVERSATION)
            val type = if (conversationData.isSmsType()) FILTER_TYPE_SMS else FILTER_TYPE_IM
            bundle.putInt(ConversationListPresenter.KEY_SHOW_SESSION_TYPE, type)
        }
        Log.d(TAG, "Publish2: " + "bundle is: " + bundle.toString())
        firePresenterEvent(Events.PUBLISH_ACTIVE_CONVERSATION, bundle)
    }

    private var disposables = CompositeDisposable()

    private fun findContactByNumberAsync(number: String) {
        disposables.add(mControllers.contacts
                .contactsApi
                .contact
                .byNumber(number)
                .withAllData()
                .asMaybe()
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ contact1 ->
                    contact = contact1
                    contact1?.displayName?.let {
                        conversationData.setConversationText(it)
                        firePresenterEvent<Events, String>(Events.TITLE_UPDATED, it, 100)
                    }
                }) { throwable -> Log.d(TAG, "Error ", throwable) })
    }

    val isGroupChat: Boolean get() = conversationData.isGroupChat()

    fun isTyping() : Boolean = getChatApi().isTyping(conversationData.imConversationData)

    private fun getChatApi(): IChatApi = mControllers.im.chatApi

    private fun getImData(): IImData = mControllers.im.imData

    private fun prepareCollabChat(bundle : Bundle) : Boolean {
        val roomId = bundle.getString(GlobalConstants.KEY_CHAT_JID)
        val username = bundle.getString(GlobalConstants.KEY_CHAT_USERNAME)
        val xmppAccounts = mControllers.accounts.getAccounts(IAccountsFilter.ACTIVE_XMPP)
        if (!xmppAccounts.isEmpty()) {
            val conversationId = getChatApi().initCollabChat(roomId, username, xmppAccounts[0])
            conversationData.roomId = roomId
            if (conversationId != -1L) {
                val imConversationData = getImData().getConversationById(conversationId)
                conversationData.setConversation(imConversationData)
            } else {
                if (ClientConfig.get().isDebugMode) {
                    //throw new IllegalStateException("Conversation already started, you didn't write any code for this case");
                } else {
                    Log.e(TAG, "initializeConversation: Conversation already started!")
                }
            }
        }
        return true
    }

    private fun prepareConversationById(bundle: Bundle) :Boolean {
        val conversationId = bundle.getLong(GlobalConstants.KEY_CONVERSATION_ID)
        val imConversationData = getImData().getConversationById(conversationId) ?: return false
        conversationData.setConversation(imConversationData)
        return true
    }

    fun canSendMessages(): Int {
        if (!conversationData.isCreated()) {
            return 0
        }
        val account = conversationData.getAccount()
        return when {
            account == null || !account.isEnabled -> R.string.tNoAccountActive
            !account.isRegistered && conversationData.isImType() -> R.string.tNoBuddiesToastAccountsIM
            !account.isRegistered && conversationData.isSmsType() -> R.string.tSMSandIMDisabled
            (conversationData.isImType() && account.type != EAccountType.Xmpp && !account.getBool(EAccountSetting.IsIMPresence)) ||
                    (conversationData.isSmsType() && !account.getBool(EAccountSetting.IsSMS)) -> R.string.tSMSandIMDisabled
            conversationData.isChatRoom() && conversationData.isLeft() -> R.string.tUserLeftChatRoom
            else -> 0
        }
    }

    private fun prepareConversationByParticipants(bundle: Bundle) : Boolean {
        val type = bundle.getInt(GlobalConstants.KEY_CONVERSATION_TYPE, ImConversationData.IM_XMPP)
        if (type == ImConversationData.IM_XMPP || type == ImConversationData.GROUPCHAT_XMPP || type == ImConversationData.IM_SIP) {
            conversationData.createConversation(bundle.getStringArrayList(GlobalConstants.KEY_BUDDY_IDS), type)
            if (!conversationData.participantSet.isEmpty() && !getChatApi().initImConversation(conversationData.participantSet)) {
                Log.e(TAG, "Unable to init conversation")
                return false
            }

        } else if (type == ImConversationData.SMS) {
            val accountNickname = bundle.getString(GlobalConstants.KEY_CHAT_ACCOUNT)
            val account = mControllers.accounts.getAccount(accountNickname)
            var remoteAddress = ""
            if (bundle.containsKey(GlobalConstants.KEY_CHAT_RAW_SMS_NUMBERS)) {
                val phoneNumberList = bundle.getParcelableArrayList<PhoneNumber>(GlobalConstants.KEY_CHAT_RAW_SMS_NUMBERS)
                remoteAddress = ImpsUtils.getSanitizedSMSNumber(phoneNumberList!![0], account)
            } else if (bundle.containsKey(GlobalConstants.KEY_CHAT_SMS_NUMBERS)) {
                remoteAddress = bundle.getStringArrayList(GlobalConstants.KEY_CHAT_SMS_NUMBERS)?.get(0).toString()
            }
            if (TextUtils.isEmpty(remoteAddress)) {
                return false
            }
            if (!remoteAddress.contains("@")) {
                remoteAddress = String.format("%s@%s", remoteAddress, account.getSettingValue(EAccountSetting.Domain))
            }
            val participantKey = BuddyKeyUtils.getNewBuddyKey(EAccountType.Sip, ImpsUtils.getUserAtDomainForAccount(account), remoteAddress)
            conversationData.createConversation(participantKey, type)
            getChatApi().initSMSConversation(conversationData.participantSet)
        } else if (type == ImConversationData.SMS_API) {
            val accountNickname = bundle.getString(GlobalConstants.KEY_CHAT_ACCOUNT)
            val account = mControllers.accounts.getAccount(accountNickname)
            var remoteAddress = ""
            if (bundle.containsKey(GlobalConstants.KEY_CHAT_RAW_SMS_NUMBERS)) {
                val phoneNumberList = bundle.getParcelableArrayList<PhoneNumber>(GlobalConstants.KEY_CHAT_RAW_SMS_NUMBERS)
                // if phoneNumberList is an empty list, don't try to get the first element, just keep remoteAddress as empty
                if (phoneNumberList != null && !phoneNumberList.isEmpty()) {
                    remoteAddress = ImpsUtils.getSanitizedSMSNumber(phoneNumberList!![0], account)
                }
            } else if (bundle.containsKey(GlobalConstants.KEY_CHAT_SMS_NUMBERS)) {
                remoteAddress = bundle.getStringArrayList(GlobalConstants.KEY_CHAT_SMS_NUMBERS)?.get(0).toString()
            }
            if (TextUtils.isEmpty(remoteAddress)) {
                return false
            }
            val participantKey = BuddyKeyUtils.getNewBuddyKey(EAccountType.SmsApi, ImpsUtils.getUserAtDomainForAccount(account), remoteAddress)
            conversationData.createConversation(participantKey, type)
            getChatApi().initSMSConversation(conversationData.participantSet)
        }
        return true
    }

    fun isHardwiredSession() : Boolean {
        if (mControllers.settings.getBool(ESetting.FeatureX)) {
            conversationData.getAccount()?.let {
                it.accountSettings[EAccountSetting.Hardwired]
            }
        }
        return false
    }

    fun getLastUnsentMessage(): String = if (conversationData.isCreated()) getImData().getUnsentText(conversationData.getId()) else ""

    fun setLastUnsentMessage(lastUnsentMessage: String) {
        if (conversationData.isCreated()) {
            getImData().setUnsentText(conversationData.getId(), lastUnsentMessage)
        }
    }

    fun getSMSCounterValue(length: Int): String = "$length / ${GlobalConstants.SMS_MAX_MESSAGE}"

    fun sendTypingNotification(typing: Boolean) {
        if (conversationData.isCreated()) {
            getChatApi().setComposingMessage(conversationData.getId(), typing)
        }
    }

    fun isSameId(bundle: Bundle): Boolean = bundle.containsKey(KEY_CONVERSATION_ID) && bundle.getLong(KEY_CONVERSATION_ID) == conversationData.getId()


    @Suppress("unused")
    fun showCounterVisibility(): Boolean =
            conversationData.isCreated() && conversationData.isSmsType() && mControllers.settings.getBool(ESetting.FeatureLimitSMSCompositionLength)

    fun canAddMoreParticipants(): Boolean {
        val account = conversationData.getAccount()
        val type = conversationData.getType()
        return when {
            (!conversationData.isCreated()) -> false
            (account == null || !account.isRegistered) -> false
            type == ImConversationData.IM_SIP -> false
            type == ImConversationData.IM_XMPP && getChatApi().isGroupChatCapable(conversationData.participantSet.firstParticipantKey) -> true
            type == ImConversationData.GROUPCHAT_XMPP && conversationData.participantSet.size < MAX_GROUP_CHAT_PARTICIPANTS
                    && getChatApi().isActive(conversationData.getId()) -> true
            type == CHAT_ROOM_TYPE -> false
            else -> false
        }
    }

    open fun canCall(): Boolean {
        val activeSipAccounts = mControllers.accounts.getActiveAccounts(EAccountType.Sip)
        return when {
            !conversationData.isCreated() -> false
            activeSipAccounts == null || activeSipAccounts.isEmpty() -> false
            conversationData.isGroupChat() -> false
            conversationData.isChatRoom() -> false
            conversationData.isXMPPType() -> {
                val buddy = conversationData.getFirstParticipant()
                if (buddy != null) {
                    val vCard = (buddy as XmppBuddy).vCard
                    return vCard != null &&
                            (!TextUtils.isEmpty(buddy.vCard.softphone) || (vCard.phoneList != null && !vCard.phoneList.isEmpty()))
                }
                return false
            }
            else -> true
        }
    }

    fun collabEnabled () : Boolean {
        return mControllers.collaboration.isCollabAvailable
    }

    fun cannedEnabled(): Boolean {
        if (!conversationData.isCreated()) {
            return false
        }
        val acc = conversationData.getAccount() ?: return false
        val isIm = conversationData.isImType() && (acc.type == EAccountType.Xmpp || acc.getBool(EAccountSetting.IsIMPresence))
        val isSms = conversationData.isSmsType() && acc.getBool(EAccountSetting.IsSMS)
        return  acc.isEnabled && acc.isRegistered && (isIm || isSms)
    }

    fun isDeleteMenuItemVisible(): Boolean = conversationData.isCreated()

    fun canSaveBuddy(): Boolean = !conversationData.isGroupChat() && conversationData.getFirstParticipant() == null && !conversationData.isSmsType()
            && (conversationData.getAccount()?.isRegistered ?: false)
            && (conversationData.getAccount()?.getBool(EAccountSetting.IsIMPresence) ?: false || conversationData.getAccount()?.type == EAccountType.Xmpp)

    fun handleSaveBuddy() {
        if (mControllers.presence.presence?.isStatusOffline == true && conversationData.getAccount()?.type == EAccountType.Xmpp) {
            firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tBuddyAddUnabledWhenOffline))
            return
        }
        offloadLightWork {
            val bundle = Bundle()
            if (conversationData.participantSet.isEmpty()) {
                firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tAddXMPPBuddyFailure))
            } else {
                val participant = conversationData.participantSet.list.get(0)
                val account = conversationData.getAccount()
                if (account != null) {
                    bundle.putString(ContactEditPresenter.KEY_ACCOUNT_NICKNAME, account.getStr(EAccountSetting.Nickname))
                }
                if (participant.type === Participant.EParticipantType.SIP) {
                    bundle.putSerializable(ContactEditPresenter.KEY_SCREEN_ACTION, ContactEditPresenter.ACTION_ADD)
                    bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.BUDDIES)
                    bundle.putString(ContactEditPresenter.KEY_CONTACT_PHONE_NUMBER, participant.remoteAddress)
                    bundle.putInt(ContactEditPresenter.KEY_PHONE_TYPE, PhoneNumber.SOFTPHONE_CODE)
                    firePresenterEvent(Events.GO_TO_CONTACT_EDIT_SCREEN, bundle)
                } else if (participant.type === Participant.EParticipantType.XMPP) {
                    bundle.putString(AddXmppBuddyPresenter.KEY_PHONE_NUMBER, participant.remoteAddress)
                    if (account != null) {
                        bundle.putString(AddXmppBuddyPresenter.KEY_ACCOUNT_NICKNAME, account.getStr(EAccountSetting.Nickname))
                    }
                    firePresenterEvent(Events.GO_TO_ADD_XMPP_BUDDY_SCREEN, bundle)
                }
            }
        }
    }

    fun canSaveContact(): Boolean = conversationData.isSmsType() && contact == null

    fun canAddToExisting(): Boolean = conversationData.participantSet.list.isNotEmpty() &&
            conversationData.participantSet.list[0]?.type === Participant.EParticipantType.SIP &&
            (canSaveBuddy() || canSaveContact())

    fun handleSaveContact() {
        val participant = conversationData.participantSet.list.get(0)
        if (participant != null && conversationData.isSmsType()) {
            val bundle = Bundle(4)
            bundle.putSerializable(ContactEditPresenter.KEY_SCREEN_ACTION, ContactEditPresenter.ACTION_ADD)
            bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.CONTACTS)
            bundle.putString(ContactEditPresenter.KEY_CONTACT_PHONE_NUMBER, participant.remoteAddress)
            bundle.putInt(ContactEditPresenter.KEY_PHONE_TYPE, getPhoneTypeForContacts(participant.remoteAddress))

            firePresenterEvent(Events.GO_TO_CONTACT_EDIT_SCREEN, bundle)
        }
    }

    fun handleAddToExisting(contactId: String) {
        val participant = conversationData.participantSet.list[0]
        if (participant?.type === Participant.EParticipantType.SIP) {
            val bundle = Bundle(7)
            val account = Controllers.get().accounts.getAccountForUserAtDomain(participant.account, EAccountType.Sip)
            if (account != null) {
                bundle.putString(ContactEditPresenter.KEY_ACCOUNT_NICKNAME, account.getStr(EAccountSetting.Nickname))
            }
            bundle.putSerializable(ContactEditPresenter.KEY_SCREEN_ACTION, ContactEditPresenter.ACTION_EDIT)
            bundle.putString(GlobalConstants.KEY_CONTACT_ID, contactId)
            bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.BUDDIES)
            bundle.putString(ContactEditPresenter.KEY_CONTACT_PHONE_NUMBER, participant.remoteAddress)
            bundle.putInt(ContactEditPresenter.KEY_PHONE_TYPE, PhoneNumber.SOFTPHONE_CODE)

            firePresenterEvent(Events.GO_TO_CONTACT_EDIT_SCREEN, bundle)
        }
    }

    private fun getPhoneTypeForContacts(phoneNumber: String): Int {
        val haveSoftphone = mControllers.settings.getBool(ESetting.HaveSoftphoneType)
        return if (haveSoftphone && !Validator.isNumeric(phoneNumber)) {
            PhoneNumber.SOFTPHONE_CODE
        } else ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE

    }

    fun getParticipantBundle(): Bundle? {
        if (conversationData.isGroupChat()) {
            return null
        }
        return if (conversationData.isImType()) {
            conversationData.getFirstParticipant()?.toBundle()
        } else {
            if (contact != null) {
                getContactBundle(contact);
            } else return null
        }
    }

    private fun getContactBundle(contact: Contact?): Bundle {
        val bundle = Bundle()
        bundle.putSerializable(GlobalConstants.KEY_CONTACT_TYPE, GlobalConstants.EContactsFilterType.BUDDIES)
        bundle.putString(GlobalConstants.KEY_CONTACT_SUB_TYPE, GlobalConstants.BUDDY_SUBTYPE_SIP)
        bundle.putString(GlobalConstants.KEY_CONTACT_ID, contact?.getId())
        return bundle
    }

    fun getParticipantList(): java.util.ArrayList<String> = ArrayList<String>(conversationData.participantSet.setOfBuddyKeys)

    fun call() {
        if (mControllers.settings.getBool(ESetting.FeatureProvisioning) && mControllers.provisioning.loginState != EProvisioningState.LoggedIn) {
            Log.w(TAG, "call - Trying to call when logged out")
            return
        }

        if (!canCall()) {
            Log.w(TAG, "call - Trying to call with no registered accounts")
            return
        }

        if (conversationData.isSipType() || conversationData.isSmsType()) {
            callSipNumber()
        } else if (conversationData.getType() == ImConversationData.IM_XMPP) {
            callXmppBuddy()
        }
    }

    open fun callSipNumber() {
        val buddyKey = conversationData.participantSet.firstParticipantKey
        if (TextUtils.isEmpty(buddyKey)) {
            return
        }
        val contactAddress = BuddyKeyUtils.getImAdressFromNewBuddyKey(buddyKey)
        val displayNameStr = conversationData.getConversationTitle()
        var account = conversationData.getAccount()
        if (conversationType == ImConversationData.SMS_API) {
            account = Controllers.get().accounts.primaryAccount
        }
        var accNickName: String? = null
        if (account == null) {
            account = Controllers.get().accounts.primaryAccount
        }
        if (account != null) {
            accNickName = account.getStr(EAccountSetting.Nickname)
        }
        if (TextUtils.isEmpty(accNickName)) {
            return
        }
        BIAnalytics.get().reportUIClick("dial-from-IM");
        if (!mControllers.phone.call(contactAddress, accNickName, displayNameStr, CallData.ECallType.Generic)) {
            val error = mControllers.phone.lastError
            if (error != null) firePresenterEvent(Events.ON_IM_ERROR, error)
        }
    }

    private fun callXmppBuddy() {
        val vCardNumbers: List<VCard.PhoneNumberType> = getPhoneListForBuddy()
        if (vCardNumbers.isEmpty()) {
            Log.e(TAG, "callXmppBuddy - unexpected case: xmpp buddy has no phone numbers")
        } else if (vCardNumbers.size == 1) {
            callXmppNumber(vCardNumbers[0])
        } else if (vCardNumbers.size > 1) {
            firePresenterEvent(Events.SHOW_NUMBER_CHOOSER_4CALL)
        }
    }

    fun getPhoneListForBuddy() : List<VCard.PhoneNumberType> {
        var vCardNumbers: List<VCard.PhoneNumberType> = ArrayList<VCard.PhoneNumberType>()
        val buddy = conversationData.getFirstParticipant() ?: return vCardNumbers
        try {
            vCardNumbers = (buddy as XmppBuddy).vCard.phoneList
            if (!TextUtils.isEmpty(buddy.vCard.softphone)) {
                val softphone = buddy.vCard.PhoneNumberType()
                softphone.number = buddy.vCard.softphone
                softphone.phoneType = VCard.EPhoneType.Voice
                softphone.isPreferred = true
                vCardNumbers.add(softphone)
            }
        } catch (ignored: NullPointerException) {
        }
        return vCardNumbers
    }

    fun callXmppNumber(numberWrapper: VCard.PhoneNumberType) {
        val nickname = mControllers.accounts.primaryAccount.getStr(EAccountSetting.Nickname)
        val displayName = conversationData.getFirstParticipant()?.displayName ?: ""
        mControllers.phone.call(numberWrapper.number, nickname, displayName, CallData.ECallType.Generic)
    }

    internal fun validateConversation() {
        if (conversationData.isImType() && !mControllers.settings.getBool(ESetting.ImPresence)) {
            cleanAndGoUp()
        } else if (conversationData.isSmsType() && !mControllers.settings.getBool(ESetting.Sms)) {
            cleanAndGoUp()
        }
    }

    fun deleteSelectedMessages() {
        dataProvider.getSelectedFileTransferMessages().forEach {message ->
            getChatApi().rejectMessage(message)
        }
        dataProvider.deleteSelectedMessages()

    }

    fun handleDeleteConversation() {
        offloadLightWork {
            if (conversationData.isCreated()) {
                sendTypingNotification(false)
                setLastUnsentMessage("")
                conversationData.imConversationData?.let {
                    getImData().removeConversation(it)
                }
                conversationData.getChatRoom()?.let {
                    getImData().chatRoomRepo.deleteChatRoom(it)
                }
                firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tSessionDeleted))
                cleanAndGoUp()
            }
        }
    }

    fun forwardSelectedMessagesTo(buddyList: java.util.ArrayList<String>) {
        offloadLightWork {
            if (!isBuddyListValid(buddyList)) {
                firePresenterEvent(Events.SHOW_TOAST, Integer.valueOf(R.string.tUnableToCreateSessionForBuddies))
            } else {
                val selectedFiles: List<String> = dataProvider.getSelectedFileTransferMessages().filter { it.isTransferValidForForwarding() }.map { it.filePath }
                //go to window
                var tempMessage = dataProvider.getSelectedMessagesAsString()
                val bundle = Bundle()
                bundle.putStringArrayList(KEY_BUDDY_IDS, buddyList)
                if (selectedFiles.isNotEmpty()) {
                    bundle.putStringArray(GlobalConstants.KEY_FILES_TO_SEND, selectedFiles.toTypedArray())
                }
                if (tempMessage.length > GlobalConstants.MAX_IM_MESSAGE_LENGTH) {
                    tempMessage = SdkStringCutter.truncate(tempMessage, GlobalConstants.MAX_IM_MESSAGE_LENGTH)
                    firePresenterEvent(Events.SHOW_TOAST, Integer.valueOf(R.string.tMaxMeassage))
                }
                var conversationType = ImConversationData.IM_XMPP
                if (!buddyList.isEmpty()) {
                    if (buddyList.size > 1) {
                        conversationType = GROUPCHAT_XMPP
                    } else {
                        val participant = Participant(buddyList[0])
                        val type = participant.type
                        when {
                            type === Participant.EParticipantType.SIP -> conversationType = ImConversationData.IM_SIP
                            type == Participant.EParticipantType.SMSAPI -> conversationType = ImConversationData.SMS_API
                        }
                    }
                }
                if (tempMessage.isNotBlank()) {
                    bundle.putString(KEY_CHAT_INPUT_TEXT, tempMessage)
                }
                bundle.putInt(KEY_CONVERSATION_TYPE, conversationType)
                firePresenterEvent(Events.FORWARD_MESSAGES, bundle)
            }
        }
    }

    private fun isBuddyListValid(buddyList: java.util.ArrayList<String>): Boolean =
            buddyList.none { BuddyKeyUtils.getAccountFromNewBuddyKey(it) == BuddyKeyUtils.NO_ACCOUNT }

    fun forwardSMSTo(numbers: ArrayList<PhoneNumber>) {
        var smsAccount: Account? = null
        val accounts = mControllers.accounts.getActiveAccountsWithEnabledSMS(EAccountType.Sip)
        if (accounts.size == 1) {
            smsAccount = accounts[0] //show account chooser if more than one sms account
        }
        if (smsAccount == null) {
            firePresenterEvent(Events.CLEAR_SELECTION)
            firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tNoActiveSmsAccount))
            return
        }
        val message = dataProvider.getSelectedMessagesAsString()
        val bundle = Bundle()
        if (smsAccount.getType() == EAccountType.SmsApi) {
            bundle.putInt(KEY_CONVERSATION_TYPE, ImConversationData.SMS_API)
        } else {
            bundle.putInt(KEY_CONVERSATION_TYPE, ImConversationData.SMS)
        }
        bundle.putString(KEY_CHAT_ACCOUNT, smsAccount.getStr(EAccountSetting.Nickname))
        bundle.putParcelableArrayList(KEY_CHAT_RAW_SMS_NUMBERS, numbers)
        bundle.putString(KEY_CHAT_INPUT_TEXT, message)
        firePresenterEvent(Events.FORWARD_MESSAGES, bundle)
    }

    fun addParticipants(buddyList: java.util.ArrayList<String>) {
        if (!canAddMoreParticipants()) {
            firePresenterEvent(Events.SHOW_TOAST, Integer.valueOf(R.string.tAddParticipantFailed))
            return
        }
        val participantsSet = ParticipantsSet(conversationData.participantSet)
        for (buddyKey in buddyList) {
            participantsSet.add(Participant(buddyKey))
        }
        if (!conversationData.isGroupChat()) {
            val bundle = Bundle()
            bundle.putInt(KEY_CONVERSATION_TYPE, GROUPCHAT_XMPP)
            bundle.putStringArrayList(KEY_BUDDY_IDS, ArrayList<String>(participantsSet.setOfBuddyKeys))
            firePresenterEvent(Events.START_CONVERSATION, bundle)
        } else {
            getChatApi().addParticipants(conversationData.imConversationData, participantsSet)
        }
    }

//    fun createMsgFilters(): Array<InputFilter?> {
//        if (conversationData.isCreated) {
//            if (conversationData.isImType) {
//                return arrayOf(object : InputFilter.LengthFilter(GlobalConstants.MAX_IM_MESSAGE_LENGTH + 1) {})
//            } else if (conversationData.isSmsType && mControllers.settings.getBool(ESetting.FeatureLimitSMSCompositionLength)) {
//                return arrayOf(InputFilter.LengthFilter(GlobalConstants.SMS_MAX_MESSAGE))
//            }
//        }
//        return arrayOfNulls(0)
//    }

    fun remoteSyncEnabled(): Boolean {
        return conversationData.isCreated() && conversationData.getType() == ImConversationData.IM_XMPP
                && RemoteSyncUtil.remoteSyncEnabled(conversationData.getAccount())
    }

    private fun cleanAndGoUp() {
        conversationData.setConversation(null)
        dataProvider.clean()
        firePresenterEvent(Events.GO_UP)
    }

    private val mImDataObserverAdapter = object : ImDataObserverAdapter() {

        override fun onConversationAdded(conversation: ImConversationData) {

            var conversationMatched = false
            if (conversation.type == ImConversationData.GROUPCHAT_COLLAB && TextUtils.equals(conversationData.roomId, conversation.remoteKey)) {
                conversationMatched = true
            } else if (isGroupChat && conversationData.participantSet.serializedParticipantKeys == conversation.remoteKey) {
                conversationMatched = true
            } else if (conversationData.participantSet.serializedParticipantKeys == conversation.participants) {
                conversationMatched = true
            }
            if (conversationMatched) {
                Log.d(TAG, "Conversation matched")
                conversationData.setConversation(conversation)
                dataProvider.setImConversationData(conversation.getId()!!, isHardwiredSession(), conversationData.isChatRoom())
                if (filesToSend.isNotEmpty()) {
                    sendFiles(filesToSend.clone())
                    filesToSend = emptyArray()
                }
                firePresenterEvent(Events.CONVERSATION_UPDATED)
                publishActiveConversation()
            }

        }

        override fun onConversationRemoved(conversation: ImConversationData) {
            super.onConversationRemoved(conversation)
            if (conversationData.getId() == conversation.id) {
                cleanAndGoUp()
            }
        }

        override fun onComposingEvent(conversationId: Long?, isComposing: Boolean) {
            if (conversationData.isCreated()) {
                if (conversationId == conversationData.getId()) {
                    firePresenterEvent(Events.USER_TYPING, isComposing)
                }
            }
        }

        override fun onOwnerIsChanged() {
            cleanAndGoUp()
        }

        override fun onMessagesFetched() {
            firePresenterEvent(Events.MESSAGES_FETCHED)
        }

        override fun onConversationUpdated(conversationId: Long?) {
            var fireScreenUpdate = true
            if (conversationData.getId() == conversationId) {
                val newConversationData = getImData().getConversationById(conversationId)
                if (newConversationData == null) {
                    cleanAndGoUp()
                } else {
                    if (newConversationData.participants == conversationData.participantSet.serializedParticipantKeys && conversationData
                            .getRemoteKey() == newConversationData.remoteKey && conversationData.getModTime() != newConversationData.modTime) {
                        fireScreenUpdate = false
                    }
                    conversationData.setConversation(newConversationData)
                    if (fireScreenUpdate) { // do not update on only conversation mod time change
                        firePresenterEvent(Events.CONVERSATION_UPDATED)
                    }
                }
                publishActiveConversation()
            }
        }
    }

    override fun onSettingsChanged(changedSettings: MutableSet<ESetting>?) {
        if (!conversationData.isCreated() || changedSettings == null) {
            return
        }
        if (changedSettings.contains(ESetting.ImPresence) && conversationData.isImType()) {
            if (!mControllers.settings.getBool(ESetting.ImPresence)) {
                cleanAndGoUp()
            }
        } else if (changedSettings.contains(ESetting.Sms) && conversationData.isSmsType()) {
            if (!mControllers.settings.getBool(ESetting.Sms)) {
                cleanAndGoUp()
            }
        }
    }

    private val mChatApiObserver = object : IChatApiObserver {
        //nvtd implement this for chat room
        override fun onComposing(conversationId: Long, isComposing: Boolean) {
            if (conversationData.isCreated() && conversationId == conversationData.getId()) {
                firePresenterEvent(Events.USER_TYPING, isComposing)
            }
        }

        override fun onRoomStatusChange(remoteKey: String?, isActive: Boolean) {
            if (conversationData.isCreated() && conversationData.isGroupChat() && conversationData.getRemoteKey() == remoteKey) {
                Log.d(TAG, "onRoomStatusChange: IS_ACTIVE " + isActive)
                firePresenterEvent(Events.MCU_ROOM_STATE_CHANGED, isActive)
            }
        }

        override fun onMessageDeliveryFailed(errorCode: Int, errorMessage: String, conversationId: Long) {
            if (conversationData.getId() == conversationId) {
                firePresenterEvent(Events.SHOW_TOAST,  getString(R.string.tMessageDeliveryFailed) + ": $errorCode - " + errorMessage)
            }
        }
    }

    override fun onAccountStatusChanged(account: Account?, isPushTunnel: Boolean) { firePresenterEvent(Events.ACCOUNTS_CHANGED) }

    override fun onRemoteEndPresenceChanged(accId: String?, remoteAddr: String?, presenceStatus: EPresenceStatus?, presenceMsg: String?) {
        if (conversationData.isCreated() && conversationData.participantSet.containsRemoteAddress(remoteAddr)) {
            firePresenterEvent(if (isGroupChat) Events.UPDATE_PARTICIPANTS_BAR else Events.UPDATE_ACTION_BAR)
        }
    }

    override fun onPrimaryAccountChanged(primaryAccount: Account?) {}

    override fun onAccountsChanged(changes: AccountsChangeInfo?) {firePresenterEvent(Events.ACCOUNTS_CHANGED)}

    override fun onPresenceUpdate(presence: OwnPresence, previousPresence: OwnPresence?) {}

    override fun onAccountVMNumberChanged(voiceMail: VoiceMail?) {}

    override fun onSubscriptionRequest() {}

    override fun onVpnConnectionFailed(errCode: Int, msg: String?) {}

    override fun onBuddyRequestNumberChanged(newRequestCount: Int, allRequestsCount: Int) {}

    override fun onNetworkConnected(conType: INetworkObserver.ENetworkType?, cellType: INetworkObserver.EMobileType?) {
        firePresenterEvent(Events.INVALIDATE_MENU)
    }

    override fun onNetworkDisconnected() {
        firePresenterEvent(Events.INVALIDATE_MENU)
    }

    override fun onPhoneStateChanged(status: INetworkObserver.EPhoneMonitoring?) {
    }

    override fun onActivityResult(a: Activity?, requestCode: Int, resultCode: Int, intent: Intent?) {
        super.onActivityResult(a, requestCode, resultCode, intent)
        if (requestCode == INTENT_CODE_VOICE_INPUT_IM) {
            if (resultCode == Activity.RESULT_OK) {
                // populate the wordsList with the String values the recognition engine thought it heard
                val matches = intent!!.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
                if (matches != null && matches.size > 0) {
                    Log.d(TAG, "Voice Input got this: " + matches[0])
                    firePresenterEvent(Events.ADD_MESSAGE_WORD, matches[0])
                }


            }

        }
    }

    override fun onOwnerChanged() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    enum class Events : IPresenterEventTypeEnum {
        GO_UP,
        PUBLISH_ACTIVE_CONVERSATION,
        DESELECT_ALL_CONVERSATIONS,
        USER_TYPING,
        CONVERSATION_UPDATED,
        SET_INPUT_TEXT,
        TITLE_UPDATED,
        GO_TO_CONTACT_EDIT_SCREEN,
        GO_TO_ADD_XMPP_BUDDY_SCREEN,
        SHOW_NUMBER_CHOOSER_4CALL,
        ON_IM_ERROR,
        SHOW_TOAST,
        FORWARD_MESSAGES,
        START_CONVERSATION,
        MCU_ROOM_STATE_CHANGED,
        ACCOUNTS_CHANGED,
        UPDATE_ACTION_BAR,
        MESSAGES_FETCHED,
        CANCELED_LOADING,
        ADD_MESSAGE_WORD,
        CLEAR_SELECTION,
        UPDATE_PARTICIPANTS_BAR,
        SEND_TO_CLIPBOARD,
        INVALIDATE_MENU;
        override fun getType(): IPresenterEventTypeEnum = this
    }

    fun isVoiceInputEnabled(): Boolean {
        val manager = context.packageManager
        val activities = manager.queryIntentActivities(Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0)
        return activities.size > 0 && mControllers.settings.getBool(ESetting.FeatureVoiceInput)
    }

    fun hasParticipants(): Boolean {
        return conversationData.participantSet.isNotEmpty()
    }

    fun getParticipantsText(): CharSequence {
        val ret = SpannableStringBuilder(context.getString(R.string.tTo))
        val participants = conversationData.participantSet.list
        for (participant in participants) {
            val participantName = participant.name
            val spannableString = SpannableString(" $participantName ")
            val buddyAvailable = conversationData.isParticipantActive(participant)
            // If the buddy is not offline and the group chat is created, only then will he appear present, i.e. his text color will be black
            val textColor = R.color.black
            val backColor = if (buddyAvailable && conversationData.isActive()) R.color.participant_background_active_color
            else R.color.participant_background_inactive_color
            spannableString.setSpan(RoundedBackgroundSpan(context, backColor, textColor), 0, spannableString.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            ret.append("  ") // we need two spaces because of the extra wide span
            ret.append(spannableString)
        }
        return ret
    }

    fun getParticipantNames(): String {
        return conversationData.participantSet.getShortParticipantList(context)
    }

    fun isCollab(): Boolean { return conversationData.getType() == ImConversationData.GROUPCHAT_COLLAB}

    fun shouldParseHyperlinks(): Boolean {
        return mControllers.settings.getBool(ESetting.ImHyperlinkPreview)
    }

    fun sendImages(selectedImages: Array<out String>?) {
        if (selectedImages != null) {
            offloadHeavyWork {
                if (getChatApi().sendImages(selectedImages, conversationData.getId()) == false) {
                    firePresenterEvent(Events.SHOW_TOAST, R.string.tSendFileUnable)
                }
            }

        }
    }

    fun isFileTransferEnabled(): Boolean = conversationData.isXMPPType() && !conversationData.isGroupChat()
            && mControllers.settings.getBool(ESetting.FeatureFileTransfer)
    fun getMessageMaxLength(): Int = GlobalConstants.MAX_IM_MESSAGE_LENGTH
    fun startHostWithParticipantInvite() {
        mControllers.collaboration.startHostFromChat(conversationData.imConversationData)
    }

    fun isConferenceLive(): Boolean{
        return mControllers.collaboration.isConferenceLive
    }

    fun isConferenceHosted(): Boolean {
        return ParticipantManager.isCurrentUserModerator();
    }

    fun inviteParticipantToJoinConf() {
        mControllers.collaboration.fetchConferenceInviteFromChat(conversationData.imConversationData)
    }
    fun sendFiles(selectedFiles: Array<String>?) {
        if (selectedFiles != null) {
            offloadHeavyWork {
                if (!getChatApi().sendFiles(selectedFiles, conversationData.getId())) {
                    firePresenterEvent(Events.SHOW_TOAST, getString(R.string.tSendFileUnable))
                }
            }

        }
    }

    override fun onListLoaded(size: Int) {
        publishActiveConversation()
    }

    override fun onListLoading() {
    }

    fun isChatRoom(): Boolean = conversationData.isChatRoom()

    fun isPrivateChatRoom(): Boolean = conversationData.isPrivateChatRoom()

    fun showMenuChatRoomMembersItem(): Boolean = ClientConfig.get().isDebugMode

    fun prepareSelectedMessagesForCopying() {
        offloadLightWork {
            firePresenterEvent(Events.SEND_TO_CLIPBOARD, dataProvider.getSelectedMessagesAsString())
        }
    }
}
