package com.bria.voip.ui.main.im

import android.Manifest
import android.annotation.SuppressLint
import android.app.Dialog
import android.content.*
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.graphics.drawable.AnimationDrawable
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Parcelable
import android.speech.RecognizerIntent
import android.support.annotation.MainThread
import android.support.constraint.ConstraintLayout
import android.support.v4.content.FileProvider
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.app.AlertDialog
import android.support.v7.widget.DefaultItemAnimator
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.TooltipCompat
import android.text.Editable
import android.text.InputType
import android.text.TextUtils
import android.text.TextWatcher
import android.util.DisplayMetrics
import android.view.*
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.inputmethod.EditorInfo
import android.widget.*
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.Controllers
import com.bria.common.controller.collaboration.utils.matcher.ParticipantMatcher
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.local.data.PhoneNumber
import com.bria.common.controller.im.ImpsUtils
import com.bria.common.controller.im.refactoring.adapter.RxInstantMessageListAdapter
import com.bria.common.controller.im.refactoring.db.entities.ImConversationData
import com.bria.common.controller.settings.ESetting
import com.bria.common.mdm.MdmUtils
import com.bria.common.ui.util.EditTextUtils
import com.bria.common.uiframework.activities.Orion
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.anyncbitmap.AbstractImageLoader
import com.bria.common.uiframework.branding.AbstractCustomizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.dialogs.ScreenHolderDialog
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bria.common.uiframework.lists.recycler.ScrollLinearLayoutManager
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uiframework.screens.IScreenEnum
import com.bria.common.util.BriaError
import com.bria.common.util.GlobalConstants
import com.bria.common.util.LocalString
import com.bria.common.util.Log
import com.bria.voip.BriaSipGraph
import com.bria.voip.R
import com.bria.voip.ui.main.im.chatroom.ChatRoomMentionsAdapter
import com.bria.voip.ui.main.im.chatroom.ChatRoomMentionsTokenizer
import com.bria.voip.ui.main.misc.EPhoneActivityList
import com.bria.voip.ui.main.misc.EScreenList
import com.bria.voip.ui.shared.pickers.BuddyPickerScreen
import com.bria.voip.ui.shared.pickers.BuddyPickerScreen.CHOSEN_BUDDY_IDS
import com.bria.voip.ui.shared.pickers.ContactPickerScreen
import com.bria.voip.ui.shared.pickers.SmsPickerScreen
import java.io.File

@Layout(R.layout.conversation_screen)
@com.bria.common.uiframework.annotations.Menu(R.menu.chat_menu)
open class ConvScreen<Presenter : ConvPresenter> : AbstractScreen<Presenter>(), View.OnKeyListener {

    private companion object {
        private val TAG = "ConvScreen"
        private val KEY_LAYOUT_STATE_LIST = "LAYOUT_CONVERSATION_STATE"
        private val KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID = "LAYOUT_STATE_SAVED_CONVERSATION_ID"
        private val KEY_FILE_CONTAINER_VISIBLE = "FILE_CONTAINER_VISIBLE"
        private val KEY_LAYOUT_STATE_FOCUSED = "LAYOUT_STATE_FOCUSED"
        private val MESSAGE_SELECTION_START_KEY = "MESSAGE_SELECTION_START_KEY"
        private val MESSAGE_SELECTION_END_KEY = "MESSAGE_SELECTION_END_KEY"
        private val KEY_LAYOUT_STATE_NEW_MESSAGES_FLAG = "LAYOUT_NEW_MESSAGES_STATE"
        //DIALOGS
        private val CONTACT_EDIT_OVERLAY = 0xD0001
        private val DIALOG_DELETE_IMS = 0xD00002
        private val DELETE_CONTACT_DIALOG = 0xD00003
        private val NUMBER_CHOOSER_DIALOG = 0xD00004
        private val SHOW_WIFI_WARNING_DIALOG = 0xD00005

        val COLLAB_SCREEN_SMALL_SIZE = 2000
    }

    @Inject(R.id.conversation_screen_toolbar_container)
    private lateinit var mFakeToolbar: ConstraintLayout

    @Inject(R.id.conversation_screen_ivPresenceImage)
    private lateinit var mPresenceIcon: ImageView

    @Inject(id = R.id.conversation_screen_tvContactName, back = ESetting.ColorNavBar, inv = true, tag = AbstractCustomizer.Tag.OSO)
    private lateinit var mTitleView: TextView

    @Inject(id = R.id.conversation_screen_tvPresenceNote, back = ESetting.ColorNavBar, inv = true, tag = AbstractCustomizer.Tag.OSO)
    private lateinit var mSubtitleView: TextView

    @Clickable
    @Inject(id = R.id.compose_message_send_message, fore = ESetting.ColorBrandTint, fade = true)
    private lateinit var mSendButton: ImageView

    @Inject(R.id.conversation_screen_messages_container)
    private lateinit var mListContainer: FrameLayout

    @Inject(R.id.conversation_screen_progress)
    private lateinit var mProgressBar: ProgressBar

    @Inject(R.id.conversation_screen_swipe_refresh)
    private lateinit var mSwipeRefresh: SwipeRefreshLayout

    @Inject(R.id.conversation_screen_recycler_view)
    private lateinit var recyclerList: RecyclerView

    @Inject(R.id.conversation_screen_participants_container)
    private lateinit var mParticipantsShortContainer: ViewGroup

    @Inject(R.id.conversation_screen_participants_panel_container)
    private lateinit var mParticipantsContainer: ViewGroup

    @Inject(R.id.conversation_screen_participants_panel_text_view)
    private lateinit var mParticipantsPanelShortList: TextView

    @Clickable
    @Inject(id = R.id.conversation_screen_participant_panel_details_button, fore = ESetting.ColorBrandTint, tag = AbstractCustomizer.Tag.FCS)
    private lateinit var mParticipantsPanelDetailsButton: TextView

    @Inject(R.id.conversation_screen_expanded_participants_panel_container)
    private lateinit var mParticipantsExpandedPanelContainer: ViewGroup

    @Inject(R.id.conversation_screen_expanded_participants_panel_text_view)
    private lateinit var mParticipantsExpandedPanelText: TextView

    @Clickable
    @Inject(id = R.id.conversation_screen_expanded_participant_panel_hide_button, fore = ESetting.ColorBrandTint, tag = AbstractCustomizer.Tag.FCS)
    private lateinit var mParticipantsExpandedPanelHideButton: TextView

    @Inject(R.id.conversation_screen_compose_container)
    private lateinit var mComposeContainer: View

    @Inject(R.id.compose_message_text)
    private lateinit var mMessageText: MultiAutoCompleteTextView

    @Inject(R.id.compose_message_counter)
    private lateinit var letterCounter: TextView

    @Inject(R.id.conversation_screen_ivTyping)
    private lateinit var typingImage: ImageView

    @Inject(R.id.conversation_screen_more_messages_container)
    private lateinit var mMoreMessagesContainer: LinearLayout

    @Inject(R.id.conversation_screen_more_messages)
    @Clickable
    private lateinit var mMoreMessagesText: TextView

    @Inject(R.id.compose_message_text_voice)
    @Clickable
    private lateinit var voiceInput: ImageView

    @Inject(id = R.id.conversation_screen_msel_toolbar, back = ESetting.ColorNavBar, tag = AbstractCustomizer.Tag.OSO)
    private lateinit var mSelectToolbar: LinearLayout

    @Inject(R.id.conversation_screen_holder)
    private lateinit var mScreenHolder: ConstraintLayout

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_back, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectBack: ImageView

    @Inject(id = R.id.conversation_screen_msel_number, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectNumber: TextView

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_del, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectDelete: ImageView

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_copy, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectCopy: ImageView

    @Clickable
    @Inject(id = R.id.conversation_screen_msel_forward, back = ESetting.ColorNavBar, inv = true)
    private lateinit var mSelectForward: ImageView

    @Clickable
    @Inject(id = R.id.compose_message_send_file)
    private lateinit var mSendFile: ImageView

    @Clickable
    @Inject(id = R.id.compose_message_send_doc)
    private lateinit var mSendDoc: ImageView

    @Clickable
    @Inject(id = R.id.compose_message_send_image)
    private lateinit var mSendImage: ImageView

    @Inject(id = R.id.conversation_screen_send_file_container)
    private lateinit var mSendFileContainer: ConstraintLayout

//    @Inject(id = R.id.compose_message_container)
//    private lateinit var mComposeMessageContainer: ConstraintLayout

    private lateinit var adapter: RxInstantMessageListAdapter
    private var animationDrawable: AnimationDrawable? = null
    private lateinit var layoutManager: LinearLayoutManager
    private var menuItemClickEnabled: Boolean = true

    private var sendFileContainerVisible = false

    private val chatRoomMentionsAdapter = BriaSipGraph.chatRoomMentionsAdapter()

    @MainThread
    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)
        val imageLoader = object : AbstractImageLoader(activity, activity.resources.getDimensionPixelSize(R.dimen.article_thumbnail_width)) {
            override fun processBitmap(data: Any?): Bitmap? = Controllers.get().image.loadPhotoFromUri(data as String)
        }
        imageLoader.setImageFadeIn(false)
        (imageLoader as AbstractImageLoader).setLoadingImage(R.drawable.article_photo_placeholder)
        typingImage.setBackgroundResource(R.drawable.chat_typing)
        animationDrawable = typingImage.background as AnimationDrawable
        adapter = RxInstantMessageListAdapter(activity, imageLoader, ParticipantMatcher(50, activity), clickListener, presenter.dataProvider, callback)

        layoutManager = ScrollLinearLayoutManager(activity, LinearLayoutManager.VERTICAL, false)
        layoutManager.stackFromEnd = true
        layoutManager.supportsPredictiveItemAnimations()
        recyclerList.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView?, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                //Log.d(TAG, "onScrolled: from " + (mLayoutManager.findFirstVisibleItemPosition() + " to: " + mLayoutManager.findLastVisibleItemPosition()));
                updateVisibleMessages()
                if (isScrolledToTheBottom()) {
                    mMoreMessagesContainer.visibility = View.GONE
                }
            }
        })
        //Collab Chat Patch
        if (isInsideCallScreen && !isInTabletMode) {
            val metrics = DisplayMetrics()
            activity.windowManager.defaultDisplay.getMetrics(metrics)
            if (metrics.heightPixels < COLLAB_SCREEN_SMALL_SIZE) {
                mScreenHolder.minHeight = resources.getDimensionPixelSize(R.dimen.collab_conversation_screen_minheight)
            } else {
                mScreenHolder.minHeight = resources.getDimensionPixelSize(R.dimen.popup_window_default_height)
            }
        }
        recyclerList.adapter = adapter
        recyclerList.layoutManager = layoutManager
        recyclerList.visibility = View.VISIBLE
        recyclerList.itemAnimator = DefaultItemAnimator()
        mSubtitleView.text = ""
        mSubtitleView.visibility = View.GONE
        mPresenceIcon.setImageDrawable(null)
        mPresenceIcon.visibility = View.GONE
        mMoreMessagesContainer.visibility = View.GONE
        mMessageText.imeOptions = EditorInfo.IME_ACTION_SEND;
        mMessageText.setRawInputType(InputType.TYPE_CLASS_TEXT);
        mMessageText.setOnEditorActionListener { _, actionId, _ ->
            var reacted = false
            if (actionId == EditorInfo.IME_ACTION_SEND) {
                sendMessage()
                reacted = true
            }
            reacted
        }
        TooltipCompat.setTooltipText(mSelectForward, getString(R.string.tForward))
        TooltipCompat.setTooltipText(mSelectCopy, getString(R.string.tCopy))
        TooltipCompat.setTooltipText(mSelectDelete, getString(R.string.tDelete))

    }

    @MainThread
    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
        presenter.subscribe(this)
        if (bundle != null) {
            presenter.initConversation(bundle)
        }
        mMessageText.setOnKeyListener(this)
        mMessageText.addTextChangedListener(mMessageTextWatcher)
        typingImage.visibility = View.GONE
        letterCounter.visibility = View.GONE
        // mMessageText.filters = presenter.createMsgFilters()
        updateSwipeRefresh()
        typingImage.setBackgroundResource(R.drawable.chat_typing)
        animationDrawable = typingImage.background as AnimationDrawable
        typingImage.visibility = View.GONE
        voiceInput.visibility = if (presenter.isVoiceInputEnabled()) View.VISIBLE else View.GONE
        //presenter.setupVoiceInputButton(voiceInput, mMessageText, false)
        updateLastUnsentMessage()
    }

    @MainThread
    override fun onNewConfig(bundle: Bundle?) {
        super.onNewConfig(bundle)
        if (bundle != null && !presenter.isSameId(bundle)) {
            hideExtendedParticipantsPanel()
            sendFileContainerVisible = false
            updateSendFileMenu()
        }
        if (mMessageText.isEnabled) {
            presenter.setLastUnsentMessage(mMessageText.text.toString())
        }
        if (bundle != null) {
            presenter.initConversation(bundle)
        }
    }

    @MainThread
    @SuppressLint("SetTextI18n")
    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
        val type = event.type as ConvPresenter.Events
        Log.d(TAG, "OnPresenterEvent ${type.name}")
        when (type) {
            ConvPresenter.Events.GO_UP -> {
                presenter.sendTypingNotification(false)
                if (isVisible) post({ coordinator.flow().goUp() }, 500)
            }
            ConvPresenter.Events.PUBLISH_ACTIVE_CONVERSATION -> publishActiveConversation(event.data as Bundle)
            ConvPresenter.Events.DESELECT_ALL_CONVERSATIONS -> post({ sendMessage(event.data as Bundle, EScreenList.CONVERSATION_LIST) }, 200)
            ConvPresenter.Events.USER_TYPING -> animateTyping(event.data != null && event.data as Boolean)
            ConvPresenter.Events.CONVERSATION_UPDATED -> updateScreen()
            ConvPresenter.Events.SET_INPUT_TEXT -> setInputText(event.data as String)
            ConvPresenter.Events.TITLE_UPDATED -> {
                mTitleView.text = if (event.data != null) event.data as String else presenter.getTitle()
                invalidateMenu()
            }
            ConvPresenter.Events.INVALIDATE_MENU -> invalidateMenu()
            ConvPresenter.Events.GO_TO_CONTACT_EDIT_SCREEN -> {
                if (event.data != null) {
                    val bundle = event.data as Bundle
                    if (isInTabletMode) {
                        showDialog(CONTACT_EDIT_OVERLAY, bundle)
                    } else {
                        mCoordinator.flow(bundle).goTo(EScreenList.CONTACT_EDIT)
                    }
                }
            }
            ConvPresenter.Events.GO_TO_ADD_XMPP_BUDDY_SCREEN -> {
                if (event.data != null) mCoordinator.flow(event.data as Bundle).goTo(EScreenList.XMPP_BUDDY_ADD)
            }
            ConvPresenter.Events.ON_IM_ERROR -> toastLong((event.data as BriaError).description)
            ConvPresenter.Events.SHOW_NUMBER_CHOOSER_4CALL -> showDialog(NUMBER_CHOOSER_DIALOG)
            ConvPresenter.Events.SHOW_TOAST -> {
                if (event.data is Int) toastShort(event.data as Int)
                if (event.data is String) toastShort(event.data as String)
            }
            ConvPresenter.Events.FORWARD_MESSAGES -> {
                if (event.data != null) mCoordinator.flow().withBundle(event.data as Bundle).goTo(EScreenList.CONVERSATION)
            }
            ConvPresenter.Events.START_CONVERSATION -> {
                if (event.data != null) mCoordinator.flow().withBundle(event.data as Bundle).goTo(EScreenList.CONVERSATION)
            }
            ConvPresenter.Events.MCU_ROOM_STATE_CHANGED -> {
                if (event.data != null) {
                    val isActive = event.data as Boolean
                    mMessageText.isEnabled = isActive
                    setupActionBar()
                }
            }
            ConvPresenter.Events.ACCOUNTS_CHANGED -> {
                val isSending = canSendMessage(false)
                mMessageText.isEnabled = isSending
                if (!isSending) {
                    mMessageText.clearFocus()
                }
                invalidateMenu()
                mSendFile.visibility = if (canSendFile()) VISIBLE else GONE
                if (!canSendFile()) {
                    sendFileContainerVisible = false
                }
                updateSendFileMenu()

            }
            ConvPresenter.Events.UPDATE_ACTION_BAR -> setupActionBar()
            ConvPresenter.Events.MESSAGES_FETCHED -> mSwipeRefresh.isRefreshing = false
            ConvPresenter.Events.ADD_MESSAGE_WORD -> {
                mMessageText.setText(mMessageText.text.toString() + " " + event.data)
            }
            ConvPresenter.Events.CANCELED_LOADING -> {
            }
            ConvPresenter.Events.CLEAR_SELECTION -> {
                adapter.clearSelection()
            }
            ConvPresenter.Events.UPDATE_PARTICIPANTS_BAR -> {
                updateParticipantsContainer()
            }
            ConvPresenter.Events.SEND_TO_CLIPBOARD -> {
                val messages = event.data as String
                if (presenter.canCopy) {
                    val clipboardManager = activity.getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                    val clipData = ClipData.newPlainText("", messages)
                    clipboardManager.primaryClip = clipData
                }
                adapter.clearSelection()
                updateSelectionToolbar()
                toastShort(R.string.tMessagesCopiedToClipboard)
            }
        }
    }

    @MainThread
    override fun onResume() {
        super.onResume()
        presenter.validateConversation()
        //getActivity().getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE); // ???\
        //processVoiceInput()
        letterCounter.text = presenter.getSMSCounterValue(mMessageText.text.length)
        updateSelectionToolbar()
        if (!isInTabletMode && !isInsideCallScreen) {
            // onResume is called twice, after second requestFocus() call selection is changed
            EditTextUtils.requestFocusAndPreserveSelection(mMessageText)
        }

        updateScreen()
        if (presenter.isTyping()) {
            animateTyping(true)
        }

        adapter.parseHyperlinks(presenter.shouldParseHyperlinks())
    }

    @MainThread
    override fun onSaveState(stateBundle: Bundle) {
        super.onSaveState(stateBundle)
        stateBundle.putParcelable(KEY_LAYOUT_STATE_LIST, layoutManager.onSaveInstanceState())
        stateBundle.putLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, presenter.conversationData.getId())
        stateBundle.putBoolean(KEY_LAYOUT_STATE_NEW_MESSAGES_FLAG, mMoreMessagesContainer.visibility == View.VISIBLE)
        stateBundle.putBoolean(KEY_FILE_CONTAINER_VISIBLE, sendFileContainerVisible)
        stateBundle.putBoolean(KEY_LAYOUT_STATE_FOCUSED, mMessageText.isFocusable)
        if (mMessageText.isEnabled) {
            stateBundle.putInt(MESSAGE_SELECTION_START_KEY, mMessageText.selectionStart)
            stateBundle.putInt(MESSAGE_SELECTION_END_KEY, mMessageText.selectionEnd)
        }
    }

    private var lastListState: Bundle? = null

    @MainThread
    override fun onRestoreState(stateBundle: Bundle) {
        super.onRestoreState(stateBundle)
        val layoutState = stateBundle.getParcelable<Parcelable>(KEY_LAYOUT_STATE_LIST)
        if (layoutState != null) {
            lastListState = Bundle()
            lastListState?.putParcelable(KEY_LAYOUT_STATE_LIST, layoutState)
            lastListState?.putLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, stateBundle.getLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, -1))
            mMoreMessagesContainer.visibility = if (stateBundle.getBoolean(KEY_LAYOUT_STATE_NEW_MESSAGES_FLAG, false)) View.VISIBLE else View.GONE
            sendFileContainerVisible = stateBundle.getBoolean(KEY_FILE_CONTAINER_VISIBLE, false)
            updateSendFileMenu()
            //mMoreMessagesContainer.setVisibility(stateBundle.getBoolean(KEY_LAYOUT_STATE_FOCUSED, false) ? View.VISIBLE : View.GONE);
        }
        val selectionStart = stateBundle.getInt(MESSAGE_SELECTION_START_KEY, mMessageText.length())
        val selectionEnd = stateBundle.getInt(MESSAGE_SELECTION_END_KEY, mMessageText.length())
        mMessageText.setSelection(selectionStart, selectionEnd)
    }

    fun canSendFile(): Boolean = presenter.isFileTransferEnabled() && canSendMessage(false)

    private val clickListener = object : IOnRecyclerViewItemEventsListener {
        override fun onItemClick(item: View?, index: Int) {
            if (adapter.getSelectedMessageCount() > 0) {
                adapter.toggleSelectionByIndex(index)
            }
            updateSelectionToolbar()
        }

        override fun onItemLongClick(item: View?, index: Int) {
            if (adapter.getSelectedMessageCount() == 0) {
                adapter.toggleSelectionByIndex(index)
                updateSelectionToolbar()
            } else {
                onItemClick(item, index)
            }
        }

    }

    private fun updateSelectionToolbar() {
        mSelectForward.visibility = if (isInsideCallScreen) View.GONE else View.VISIBLE
        mSelectNumber.text = adapter.getSelectedMessageCount().toString()
        if (adapter.getSelectedMessageCount() == 0) {
            mSelectToolbar.visibility = View.GONE
            mFakeToolbar.visibility = View.VISIBLE
        } else {
            mSelectToolbar.visibility = View.VISIBLE
            mFakeToolbar.visibility = View.GONE
        }
    }

    @MainThread
    override fun updateMenuItems(menu: Menu, updateKey: String?) {
        super.updateMenuItems(menu, updateKey)
        if (!presenter.canAddMoreParticipants() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_add_more)
        }

        if (!presenter.canCall() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_call)
        }
        if (!presenter.collabEnabled() || presenter.isConferenceLive() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_start_conf)
        }
        if (!presenter.collabEnabled() || !presenter.isConferenceHosted() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_invite_to_conf)
        }

        if (!presenter.cannedEnabled()) {
            menu.removeItem(R.id.mi_chat_canned)
        }

        if (!presenter.isDeleteMenuItemVisible() || isInsideCallScreen) {
            menu.removeItem(R.id.mi_chat_delete)
        }

        if (!presenter.canSaveBuddy()) {
            menu.removeItem(R.id.mi_chat_add_buddy)
        }

        if (!presenter.canSaveContact()) {
            menu.removeItem(R.id.mi_chat_add_contact)
        }

        if (presenter.getParticipantBundle() == null) {
            menu.removeItem(R.id.mi_chat_view_buddy)
        }

        if (!presenter.canAddToExisting()) {
            menu.removeItem(R.id.mi_chat_add_to_existing)
        }

        if (!presenter.showMenuChatRoomMembersItem()) {
            menu.removeItem(R.id.mi_chat_room_members)
        }
    }

    @MainThread
    override fun onMenuItemClick(menuItem: MenuItem): Boolean {
        if (menuItemClickEnabled) {
            menuItemClickEnabled = false
            post({ menuItemClickEnabled = true }, 500)

            when (menuItem.itemId) {
                R.id.mi_chat_add_more -> {
                    saveComposedText()
                    val bundle = Bundle()
                    bundle.putStringArrayList(BuddyPickerScreen.ALREADY_PRESENT_BUDDIES, presenter.getParticipantList())
                    showScreenForResult(EScreenList.BUDDY_PICKER, bundle)
                    return true
                }
                R.id.mi_chat_call -> {
                    presenter.call()
                    return true
                }
                R.id.mi_chat_start_conf -> {
                    presenter.startHostWithParticipantInvite()
                    return true
                }
                R.id.mi_invite_to_conf -> {
                    presenter.inviteParticipantToJoinConf()
                    return true
                }
                R.id.mi_chat_canned -> {
                    saveComposedText()
                    showScreenForResult(EScreenList.CANNED_IM)
                    return true
                }
                R.id.mi_chat_room_members -> {
                    mCoordinator.flow().goTo(EScreenList.CHAT_ROOM_MEMBERS_SCREEN);
                    return true
                }
                R.id.mi_chat_delete -> {
                    showDialog(DELETE_CONTACT_DIALOG)
                    return true
                }
                R.id.mi_chat_add_buddy -> {
                    if (presenter.conversationType != ImConversationData.IM_SIP ||
                            checkOrRequestContactsPermission(GlobalConstants.CP_PERMISSION_ADD_SIP_BUDDY)) {
                        presenter.handleSaveBuddy()
                    }
                    return true
                }
                R.id.mi_chat_add_contact -> {
                    if (checkOrRequestContactsPermission(GlobalConstants.CP_PERMISSION_ADD_CONTACT)) {
                        presenter.handleSaveContact()
                    }
                    return true
                }
                R.id.mi_chat_view_buddy -> {
                    Orion.get().showActivity(EPhoneActivityList.MAIN, GlobalConstants.INTENT_ACTION_SHOW_CONTACT,
                            presenter.getParticipantBundle(), false)
                    return true
                }
                R.id.mi_chat_add_to_existing -> {
                    if (checkOrRequestContactsPermission(GlobalConstants.CP_PERMISSION_UPDATE_CONTACT)) {
                        showContactPicker()
                    }
                    return true
                }
            }
        }

        return super.onMenuItemClick(menuItem)
    }

    @MainThread
    override fun createDialogForResult(whichScreen: IScreenEnum, data: Bundle): ScreenHolderDialog {
        val color = Coloring.get().decodeColor(presenter.toolbarColor)
        when (whichScreen) {
            EScreenList.BUDDY_PICKER, EScreenList.SMS_PICKER, EScreenList.CONTACT_PICKER -> {
                return if (isInTabletMode) {
                    ScreenHolderDialog
                            .builder(activity)
                            .screen(whichScreen)
                            .anchorView(toolbar!!.findViewById(R.id.conversation_list_new))
                            .gravity(Gravity.BOTTOM)
                            .arrowColor(color)
                            .backgroundColor(color)
                            .bundle(data)
                            .build()
                } else {
                    ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()
                }
            }
            EScreenList.CANNED_IM ->
                return ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()

            EScreenList.GALLERY ->
                return ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()
            EScreenList.FILE_PICKER ->
                return ScreenHolderDialog.builder(activity).screen(whichScreen).style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN).bundle(data).build()
        }
        return super.createDialogForResult(whichScreen, data)
    }

    private fun showContactPicker() {
        val bundle = Bundle(1)
        bundle.putInt(ContactPickerScreen.MAX_NUMBER_OF_CHOICES, 1)
        showScreenForResult(EScreenList.CONTACT_PICKER, bundle)
    }

    @MainThread
    override fun onStop(finishing: Boolean) {
        super.onStop(finishing)
        presenter.sendTypingNotification(false)
        presenter.unsubscribe()
        saveComposedText()
        mMessageText.setText("")
        activity.window.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_NOTHING)
        mMessageText.removeTextChangedListener(mMessageTextWatcher)
    }

    @MainThread
    override fun onPause(finishing: Boolean) {
        super.onPause(finishing)
        presenter.sendTypingNotification(false)
    }

    private fun saveComposedText() {
        if (mMessageText.isEnabled) {
            presenter.setLastUnsentMessage(mMessageText.text.toString())
        }
    }

    private fun setupActionBar() {
        mTitleView.text = presenter.getTitle()
        if (presenter.isPrivateChatRoom()) {
            mTitleView.setCompoundDrawablesWithIntrinsicBounds(com.bria.common.R.drawable.ic_lock_black_24dp, 0, 0, 0)
        } else {
            mTitleView.setCompoundDrawablesWithIntrinsicBounds(0, 0, 0, 0)
        }
        if (presenter.isPresenceIconVisible()) {
            val presenceIcon = presenter.getPresenceIcon()
            mPresenceIcon.setImageDrawable(presenceIcon)
            mPresenceIcon.visibility = View.VISIBLE
        } else {
            mPresenceIcon.visibility = GONE
        }
        mSubtitleView.visibility = if (presenter.isGroupChat || presenter.isSMS()) View.GONE else View.VISIBLE
        mSubtitleView.text = presenter.getPresenceNote()
    }

    private fun updateScreen() {
        setupActionBar()
        val isSending = canSendMessage(true)
        mMessageText.isEnabled = isSending

        if (!isSending) {
            mMessageText.clearFocus()
        }

        if (presenter.isChatRoom()) {
            mMessageText.setRawInputType(InputType.TYPE_TEXT_FLAG_AUTO_COMPLETE);
            mMessageText.setAdapter(chatRoomMentionsAdapter)
            mMessageText.setThreshold(0)
            mMessageText.setTokenizer(ChatRoomMentionsTokenizer())
            chatRoomMentionsAdapter.setBuddyKeys(presenter.getParticipantList().toSet())
        } else {
            mMessageText.setRawInputType(InputType.TYPE_CLASS_TEXT);
            mMessageText.setAdapter(null)
        }


        // debug
        mMessageText.setRawInputType(InputType.TYPE_TEXT_FLAG_AUTO_COMPLETE);
        mMessageText.setAdapter(chatRoomMentionsAdapter)
        mMessageText.setThreshold(0)
        mMessageText.setTokenizer(ChatRoomMentionsTokenizer())
        chatRoomMentionsAdapter.setBuddyKeys(Controllers.get().buddy.allBuddies.filter { it.buddyType == Buddy.EBuddyType.XMPP }.take(5).map { it.uniqueIdentifier }.toSet())
        // debug


        invalidateMenu()
        updateSwipeRefresh()
        if (presenter.remoteSyncEnabled()) {
            fetchSyncedMsgsStart()
        }
        if (MdmUtils.isCopyPasteRestricted(activity)) {
            MdmUtils.blockCopyPasteOnEditText(mMessageText)
        }
        updateSelectionToolbar()
        updateParticipantsContainer()
        mSendFile.visibility = if (canSendFile()) VISIBLE else GONE
        updateSendFileMenu()
    }

    private fun updateParticipantsContainer() {
        if (presenter.isGroupChat && presenter.hasParticipants() && !presenter.isCollab()) {
            mParticipantsContainer.visibility = VISIBLE
            mParticipantsPanelShortList.text = presenter.getParticipantNames()
            mParticipantsExpandedPanelText.text = presenter.getParticipantsText()
        } else {
            mParticipantsContainer.visibility = GONE
        }

    }

    private fun updateLastUnsentMessage() {
        val lastUnsentMsg = presenter.getLastUnsentMessage()
        if (!TextUtils.isEmpty(lastUnsentMsg)) {
            mMessageText.setText(lastUnsentMsg)
        } else {
            mMessageText.setText("")
        }
    }

    private fun publishActiveConversation(bundle: Bundle) {
        if (isInTabletMode && !isInsideCallScreen) {
            val convBundle = Bundle()
            convBundle.putInt(GlobalConstants.KEY_CONVERSATION_TYPE, presenter.conversationType)
            mCoordinator.flow().withBundle(convBundle).goTo(EScreenList.CONVERSATION_LIST)
            mHandler.postDelayed({ sendMessage(bundle, EScreenList.CONVERSATION_LIST) }, 300)
        }
    }

    private fun animateTyping(showAnimation: Boolean) {
        var visibility = View.VISIBLE

        if (showAnimation) {
            typingImage.visibility = visibility
            if (!animationDrawable!!.isRunning) {
                animationDrawable!!.start()
            }
        } else {
            animationDrawable?.stop()
            animationDrawable?.selectDrawable(0)
            visibility = View.GONE
            typingImage.visibility = visibility
        }
    }

    @MainThread
    override fun onClick(v: View) {
        when (v.id) {
            R.id.compose_message_send_message -> sendMessage()
            R.id.conversation_screen_participant_panel_details_button -> showExtendedParticipantsPanel()
            R.id.conversation_screen_expanded_participant_panel_hide_button -> hideExtendedParticipantsPanel()
            R.id.conversation_screen_msel_back -> {
                adapter.clearSelection()
                updateSelectionToolbar()
            }
            R.id.compose_message_text_voice -> {
                val intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
                intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
                intent.putExtra(RecognizerIntent.EXTRA_PROMPT, activity.getString(R.string.tWaitingForVoice))
                activity.startActivityForResult(intent, GlobalConstants.INTENT_CODE_VOICE_INPUT_IM)
            }
            R.id.conversation_screen_msel_del -> showDialog(DIALOG_DELETE_IMS)
            R.id.conversation_screen_msel_copy -> {
                copySelectedMessages()
            }
            R.id.conversation_screen_msel_forward -> {
                openForwardToDialog()
            }
            R.id.conversation_screen_more_messages -> {
                adapter.setCurrentVisibleMessages(layoutManager.findFirstVisibleItemPosition(), adapter.itemCount - 1)
                recyclerList.scrollToPosition(adapter.itemCount - 1)
            }
            R.id.compose_message_send_file -> {
                sendFileContainerVisible = !sendFileContainerVisible
                updateSendFileMenu()
            }
            R.id.compose_message_send_image -> {
                val bundle = Bundle()
                bundle.putString(GlobalConstants.KEY_SENDER_NAME, presenter.getTitle())
                showScreenForResult(EScreenList.GALLERY, bundle)
            }
            R.id.compose_message_send_doc -> {
                showScreenForResult(EScreenList.FILE_PICKER)
            }
        }
        super.onClick(v)
    }

    private fun updateSendFileMenu() {
        if (sendFileContainerVisible) {
            hideKeyboard()
        }

        mSendFileContainer.visibility = if (sendFileContainerVisible) VISIBLE else GONE
    }

    override fun onSoftKeyboardOpen(keyboardHeightPx: Int) {
        sendFileContainerVisible = false
        updateSendFileMenu()
    }

    private fun copySelectedMessages() {
        presenter.prepareSelectedMessagesForCopying()
    }

    private fun sendMessage(): Boolean {
        if (presenter.shouldShowWifiWarningDialog()) {
            showDialog(SHOW_WIFI_WARNING_DIALOG)
            return false;
        }
        if (!canSendMessage(true)) {
            return false
        }
        val success = presenter.sendMessage(mMessageText.text.toString())
        if (success) {
            mMessageText.setText("")
            presenter.setLastUnsentMessage("")
        }
        return success
    }

    private fun canSendMessage(showNotice: Boolean): Boolean {
        val errorMsgId = presenter.canSendMessages()
        if (showNotice && errorMsgId != 0) {
            if (isVisible) toastShort(errorMsgId)
        }
        return errorMsgId == 0
    }

    @MainThread
    override fun onDestroy(finishing: Boolean) {
        super.onDestroy(finishing)
        adapter.cleanup()
    }

    @MainThread
    override fun onNewMessage(message: Bundle, sender: IScreenEnum) {
        super.onNewMessage(message, sender)
        when (sender) {
            EScreenList.BUDDY_PICKER -> {
                invalidateMenu()
                if (message.containsKey(ConvPresenter.KEY_CHAT_ACTION)) {
                    if (message.getString(ConvPresenter.KEY_CHAT_ACTION) == ConvPresenter.ACTION_FORWARD) {
                        val buddyList = message.getStringArrayList(CHOSEN_BUDDY_IDS)
                        presenter.forwardSelectedMessagesTo(buddyList)
                    }
                } else {
                    val buddyList = message.getStringArrayList(CHOSEN_BUDDY_IDS)
                    presenter.addParticipants(buddyList)
                }
            }
            EScreenList.SMS_PICKER -> {
                invalidateMenu()
                if (message.containsKey(ConvPresenter.KEY_CHAT_ACTION)) {
                    if (ConvPresenter.ACTION_FORWARD == message.getString(ConvPresenter.KEY_CHAT_ACTION)) {
                        presenter.forwardSMSTo(message.getParcelableArrayList<PhoneNumber>(SmsPickerScreen.SELECTED_NUMBERS))
                    }
                }
            }
            EScreenList.CANNED_IM -> {
                parseCannedIm(message.getString(CannedImScreen.KEY_MESSAGE_TEXT, ""))
            }
            EScreenList.GALLERY -> {
                BIAnalytics.get().reportUIClick(" file-transfer-send")
                presenter.sendImages(message.getStringArray(GlobalConstants.KEY_IMAGES_TO_SEND))
            }
            EScreenList.FILE_PICKER -> {
                BIAnalytics.get().reportUIClick(" file-transfer-send")
                presenter.sendFiles(message.getStringArray(GlobalConstants.KEY_FILES_TO_SEND))
            }
            EScreenList.CONTACT_PICKER -> {
                val contactIdList = message.getStringArrayList(ContactPickerScreen.SELECTED_CONTACT_IDS)
                if (contactIdList != null && contactIdList.isNotEmpty()) {
                    presenter.handleAddToExisting(contactIdList[0])
                }
            }
        }
    }

    @MainThread
    override fun createDialog(which: Int, data: Bundle?): Dialog? {
        when (which) {
            DIALOG_DELETE_IMS -> {
                val builder = AlertDialog.Builder(activity)
                builder.setTitle(R.string.tDelete)
                        .setMessage(String.format(getString(R.string.tDeleteMessages), adapter.getSelectedMessageCount()))
                        .setIcon(R.drawable.btn_delete_call_recording__comm_log_details)
                        .setCancelable(true)
                        .setPositiveButton(R.string.tYes) { _, _ ->
                            presenter.deleteSelectedMessages()
                        }
                        .setNegativeButton(R.string.tNo) { dialogInterface, _ -> dialogInterface.dismiss() }
                return builder.create()
            }
            DELETE_CONTACT_DIALOG -> {
                val message = getString(R.string.tAreYouSureDialog)
                val builder1 = AlertDialog.Builder(activity)
                builder1.setMessage(message)
                        .setCancelable(false)
                        .setPositiveButton(getString(R.string.tYes)) { _, _ -> presenter.handleDeleteConversation() }
                        .setNegativeButton(getString(R.string.tNo)) { dialog, _ -> dialog.dismiss() }
                return builder1.create()
            }
            SHOW_WIFI_WARNING_DIALOG -> {
                val builder = AlertDialog.Builder(activity)
                builder.setTitle(getString(R.string.tNotConnectedToWiFi)).setMessage(LocalString.getBrandedString(activity, getString(R.string
                        .tWiFiWarning)))
                        .setCancelable(true)
                        .setPositiveButton(R.string.tOk) { dialog, which1 -> dialog.dismiss() }
                return builder.create()

            }
            NUMBER_CHOOSER_DIALOG -> {
                val numberChooser = Dialog(activity)
                numberChooser.requestWindowFeature(Window.FEATURE_NO_TITLE)
                val layout = View.inflate(activity, R.layout.im_xmpp_number_chooser, null) as ViewGroup
                val phoneList = presenter.getPhoneListForBuddy()
                for (i in phoneList.indices) {
                    // create text item view
                    val itemView = View.inflate(activity, com.bria.common.R.layout.dialog_list_item, null) as TextView
                    itemView.text = phoneList[i].number

                    // create a divider
                    val divider = View(activity)
                    val dividerH = activity.resources.getDimensionPixelSize(com.bria.common.R.dimen.divider_height)
                    val params = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, dividerH)
                    @Suppress("DEPRECATION")
                    divider.setBackgroundColor(activity.resources.getColor(R.color.primary_light))

                    // set click listener
                    itemView.setOnClickListener {
                        presenter.callXmppNumber(phoneList[i])
                        numberChooser.dismiss()
                    }

                    // add them to the layout (TextView has intrinsic layout params)
                    layout.addView(itemView)
                    layout.addView(divider, params)
                }

                numberChooser.setContentView(layout)
                return numberChooser
            }
            CONTACT_EDIT_OVERLAY -> return ScreenHolderDialog
                    .builder(activity)
                    .screen(EScreenList.CONTACT_EDIT)
                    .style(ScreenHolderDialog.DIALOG_MAX_SIZE_NO_MARGIN)
                    .bundle(data)
                    .build()

            else -> {
                return super.createDialog(which, data)
            }
        }
    }

    private fun parseCannedIm(message: String) {
        var typedMsg = mMessageText.text.toString()
        val selStart = mMessageText.selectionStart
        val selEnd = mMessageText.selectionEnd

        typedMsg = typedMsg.substring(0, selStart) + message + typedMsg.substring(selEnd)
        mMessageText.setText(typedMsg)
        var totalLength = selStart + message.length
        if (totalLength > GlobalConstants.MAX_IM_MESSAGE_LENGTH) {
            totalLength = GlobalConstants.MAX_IM_MESSAGE_LENGTH
            Toast.makeText(activity, R.string.tMaxMeassage, Toast.LENGTH_LONG).show()
        }
        mMessageText.setSelection(totalLength)
        mMessageText.requestFocus()
    }

    @Suppress("UNCHECKED_CAST")
    override fun getPresenterClass(): Class<Presenter> = ConvPresenter::class.java as Class<Presenter>

    override fun getTitle(): String? = ""

    private fun openForwardToDialog() {
        saveComposedText()
        val bundle = Bundle()
        bundle.putString(ConvPresenter.KEY_CHAT_ACTION, ConvPresenter.ACTION_FORWARD)
        if (presenter.isImType) {
            bundle.putBoolean("FORWARD_TO", true)
            if (presenter.dataProvider.hasTransferMessagesInSelection()) {
                bundle.putInt(SmsPickerScreen.MAX_NUMBER_OF_CHOICES, 1)
            }

            if (presenter.dataProvider.getSelectedFileTransferMessages().isNotEmpty()) {
                bundle.putSerializable(BuddyPickerScreen.BUDDY_TYPE, Buddy.EBuddyType.XMPP)
            }

            showScreenForResult(EScreenList.BUDDY_PICKER, bundle)
        } else {
            bundle.putInt(SmsPickerScreen.MAX_NUMBER_OF_CHOICES, 1)
            bundle.putStringArrayList(SmsPickerScreen.ALREADY_PRESENT_PHONES, presenter.getPhoneList())
            showScreenForResult(EScreenList.SMS_PICKER, bundle)
        }
    }

    private fun isScrolledToTheBottom(): Boolean = layoutManager.itemCount - 2 <= layoutManager.findLastVisibleItemPosition()

    private fun isScreenActive(): Boolean = state == AbstractScreen.EScreenState.RESUMED && isVisible

    private fun updateVisibleMessages() {
        if (isScreenActive()) {
            val first = layoutManager.findFirstVisibleItemPosition()
            val last = layoutManager.findLastVisibleItemPosition()
            adapter.setCurrentVisibleMessages(first, last)
        }
    }

    private fun setInputText(text: String) {
        if (!TextUtils.isEmpty(text)) {
            mMessageText.setText(text)
            mMessageText.setSelection(text.length)
        } else {
            mMessageText.setText("")
        }
    }

    private fun hideExtendedParticipantsPanel() {
        mParticipantsExpandedPanelContainer.visibility = View.GONE
        mParticipantsShortContainer.visibility = View.VISIBLE
    }

    private fun showExtendedParticipantsPanel() {
        mParticipantsExpandedPanelContainer.visibility = View.VISIBLE
        mParticipantsShortContainer.visibility = View.GONE
    }

    private fun updateSwipeRefresh() {
        mSwipeRefresh.setOnRefreshListener {
            Log.d(TAG, "onRefresh: ")
            adapter.loadMoreMessages()
            //fetchSyncedMsgsStart()
        }
        mSwipeRefresh.isRefreshing = false
    }

    private val mRunnableImSyncLoadComplete = Runnable { fetchSyncedMsgsEnd() }

    private fun fetchSyncedMsgsStart() {
        // See https://code.google.com/p/android/issues/detail?id=77712
        mSwipeRefresh.post { mSwipeRefresh.isRefreshing = true }

        mHandler.postDelayed(mRunnableImSyncLoadComplete, 5000)
        //nvtd
        //getPresenter().fetchRangeRevision();
    }

    private fun fetchSyncedMsgsEnd() {
        mHandler.removeCallbacks(mRunnableImSyncLoadComplete)
        mSwipeRefresh.isRefreshing = false
    }

    private val mMessageTextWatcher = object : TextWatcher {
        override fun onTextChanged(text: CharSequence, start: Int, before: Int, count: Int) {
            if (text.length >= presenter.getMessageMaxLength()) {
                toastLong(R.string.tMaxMeassage)
            }
        }

        override fun beforeTextChanged(text: CharSequence, start: Int, count: Int, after: Int) {}

        override fun afterTextChanged(text: Editable) {
            presenter.sendTypingNotification(!TextUtils.isEmpty(text))
            while (text.length > presenter.getMessageMaxLength()) {
                //if emoji is inserted last it's not sufficient to delete only last char
                // protobuffer will have issue otherwise
                if (ImpsUtils.isLast2CharEmoji(text)) {
                    text.delete(text.length - 2, text.length)
                } else {
                    text.delete(text.length - 1, text.length)
                }

            }
        }
    }

    @MainThread
    override fun onKey(v: View, keyCode: Int, event: KeyEvent): Boolean {
        if (event.action == KeyEvent.ACTION_DOWN && keyCode == KeyEvent.KEYCODE_BACK) {
            animateTyping(false)
        }
        return false
    }

    private val callback = object : RxInstantMessageListAdapter.IInstantMessageListAdapterCallback {
        override fun canSendMessage(): Boolean = canSendMessage(true)

        override fun openFile(filePath: String?) {
            filePath?.let {
                val file = File(filePath)
                val uri: Uri
                uri = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    FileProvider.getUriForFile(activity, activity.packageName, file);
                } else {
                    Uri.fromFile(file)
                }
                val intent = Intent(Intent.ACTION_VIEW)
                //intent.setDataAndType(uri, URLConnection.guessContentTypeFromName(uri.toString()))
                intent.setData(uri)
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                try {
                    activity.startActivity(intent);
                } catch (e: ActivityNotFoundException) {
                    toastLong("No handler for this type of file.")
                }

            } ?: toastLong("File could not be found")
        }

        override fun listUpdated(desiredPosition: Int?) {
            updateSelectionToolbar()
            mSwipeRefresh.isRefreshing = false
            if (lastListState != null) {
                Log.d(TAG, "Loading previous screen state")
                val layoutState = lastListState?.getParcelable<Parcelable>(KEY_LAYOUT_STATE_LIST)
                val conversationID = lastListState?.getLong(KEY_LAYOUT_STATE_SAVED_CONVERSATION_ID, -1)
                if (conversationID == presenter.conversationData.getId()) {
                    Log.d(TAG, "Confirmed same Id")
                    recyclerList.layoutManager.onRestoreInstanceState(layoutState)
                }
                lastListState = null
            } else {
                Log.d(TAG, "scroll to: $desiredPosition")
                desiredPosition?.let {
                    recyclerList.scrollToPosition(desiredPosition)
                }

            }
        }

        override fun setMoreMessagesVisibility(visible: Boolean) {
            mMoreMessagesContainer.visibility = if (visible) VISIBLE else GONE
        }

        override fun checkWriteStoragePermission(): Boolean {
            if (!checkPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                requestPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE, GlobalConstants.CP_PERMISSION_WRITE_EXTERNAL_STORAGE,
                        getString(R.string.tFileWritePermissionRequestExplanation))
                return false
            }
            return true
        }
    }

    private fun checkOrRequestContactsPermission(requestCode: Int): Boolean {
        if (!checkPermission(Manifest.permission.WRITE_CONTACTS)) {
            requestPermission(Manifest.permission.WRITE_CONTACTS, requestCode, getString(R.string.tPermissionContacts))
            return false
        }

        return true
    }

    @MainThread
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        if (grantResults.isEmpty()) {
            return;
        }

        if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            when (requestCode) {
                GlobalConstants.CP_PERMISSION_ADD_SIP_BUDDY -> presenter.handleSaveBuddy()
                GlobalConstants.CP_PERMISSION_ADD_CONTACT -> presenter.handleSaveContact()
                GlobalConstants.CP_PERMISSION_UPDATE_CONTACT -> showContactPicker()
                GlobalConstants.CP_PERMISSION_WRITE_EXTERNAL_STORAGE -> adapter.notifyDataSetChanged()
            }
        }
    }

}
