package com.bria.voip.ui.main.im

import android.graphics.drawable.Drawable
import android.text.TextUtils
import com.bria.common.analytics.BIAnalytics
import com.bria.common.controller.contacts.genband.GenbandFriendDataObject
import com.bria.common.controller.presence.refactoring.EPresenceStatus
import com.bria.common.controller.settings.branding.EAccountType
import com.bria.common.sdkwrapper.CallData
import com.bria.common.util.Log
import com.bria.common.util.genband.FriendUtils
import com.bria.voip.R

class GenbandConvPresenter : ConvPresenter() {
    val TAG = "GenbandConvPresenter"
    private var directory_list_index: Int = 0

    override fun getPresenceIcon(): Drawable {
        if (conversationData.isSmsType()) {
            @Suppress("DEPRECATION")
            return resources.getDrawable(R.drawable.im_sms_icon)
        }
        val drawable = getConversationFriend()?.presence?.status?.getIcon(context)
        return if (drawable != null) drawable else super.getPresenceIcon()
    }

    override fun getPresenceNote(): String {
        if (!conversationData.isSmsType() && getConversationFriend() != null && getConversationFriend()!!.presence != null) {
            return getConversationFriend()!!.presence.getPresenceNote(context)
        }
        Log.i(TAG, "Presence status is  null")
        return ""
    }

    override fun canCall(): Boolean {
        val activeSipAccounts = mControllers.accounts.getActiveAccounts(EAccountType.Sip)
        if (!conversationData.isCreated() || activeSipAccounts == null || activeSipAccounts.isEmpty()) {
            return false
        }
        return conversationData.isSipType() || conversationData.isSmsType()
    }

    override fun callSipNumber() {
        if (getConversationFriend() != null) {
            val contact = getConversationFriend()?.primaryContact
            val nickName = getConversationFriend()?.accountId
            if (contact != null && nickName != null) {
                val contactDisplayName = getConversationFriend()!!.name
                BIAnalytics.get().reportUIClick("dial-from-IM");
                if (!mControllers.phone.call(contact, nickName, contactDisplayName, CallData.ECallType.Generic)) {
                    val error = mControllers.phone.lastError
                    firePresenterEvent(ConvPresenter.Events.ON_IM_ERROR, error)
                }
            }
        }
    }

    private fun getConversationFriend(): GenbandFriendDataObject? {
        val buddyKey = conversationData.participantSet.firstParticipantKey
        var friendDataObject: GenbandFriendDataObject?
        if (TextUtils.isEmpty(buddyKey)) {
            return null
        }
        friendDataObject = FriendUtils.getFriendByNewKey(buddyKey)
        if (friendDataObject != null) {
            return friendDataObject
        }

        //Directory friend
        try {

            friendDataObject = mControllers.genbandContact.directoryDataProvider.getItemAt(directory_list_index)
            if (friendDataObject != null) {
                return friendDataObject
            }
        } catch (e: IndexOutOfBoundsException) {
            android.util.Log.i(TAG, "getConversationFriend: faild to get friend")
            e.printStackTrace()
            return null
        }

        return null

    }

    internal fun setDirectoryIndex(index: Int) {
        directory_list_index = index
    }

    override fun onRemoteEndPresenceChanged(accId: String?, remoteAddr: String?, presenceStatus: EPresenceStatus?, presenceMsg: String?) {
        if (conversationData.isCreated()) { // TODO : Add filter per remote address so it will not update the screen on every remote presence update
            firePresenterEvent(ConvPresenter.Events.UPDATE_ACTION_BAR)
        }
    }
}
