package com.bria.voip.ui.main.im.chatroom

import com.bria.common.controller.contacts.ContactsSortBy
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.BuddyComparator
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.im.refactoring.BuddyKeyUtils
import com.bria.common.uireusable.dataprovider.AbstractFilterableListDataProvider
import com.bria.common.uireusable.dataprovider.FilterQueryMatcher

class ChatRoomMembersDataProvider(
        private val buddyController: IBuddyCtrlEvents,
        private val contactsSortBy: ContactsSortBy
) : AbstractFilterableListDataProvider<Buddy>() {

    private var buddyKeys: Set<String> = setOf()
    private var fullDataSet: MutableList<Buddy>? = null

    fun setBuddyKeys(buddyKeys: Set<String>) {
        if (buddyKeys != this.buddyKeys) {
            fullDataSet = null
            this.buddyKeys = buddyKeys
        }

        refilter()
    }

    override fun provideFullDataSet(): MutableList<Buddy> {
        if (fullDataSet == null) {
            fullDataSet = buddyController
                .allBuddies
                .filter { it.buddyType == Buddy.EBuddyType.XMPP }
                .filter { buddyKeys.contains(BuddyKeyUtils.getNewBuddyKey(it)) }
                .sortedWith(BuddyComparator(contactsSortBy))
                .toMutableList()
        }

        return fullDataSet!!
    }

    override fun provideFilteredDataSet(fullDataSet: MutableList<Buddy>, query: String): MutableList<Buddy> {
        val filterQueryMatcher = FilterQueryMatcher(query)

        return fullDataSet
            .filter { filterQueryMatcher.isMatch(it.firstName, it.lastName) }
            .toMutableList()
    }
}
