package com.bria.voip.ui.main.im.chatroom

import android.os.Bundle
import android.os.Parcelable
import android.support.design.widget.FloatingActionButton
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.MenuItem
import android.view.View
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.settings.ESetting
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.uireusable.CustomSearchView
import com.bria.common.util.Log
import com.bria.voip.R
import java.util.*

@Layout(R.layout.chat_room_members_screen)
@com.bria.common.uiframework.annotations.Menu(R.menu.chat_room_members_menu)
class ChatRoomMembersScreen : AbstractScreen<ChatRoomMembersPresenter>() {

    private companion object {
        private const val TAG = "ChatRoomMembersScreen"

        private const val KEY_LINEAR_LAYOUT_STATE = "LINEAR_LAYOUT_STATE"

        public const val CHAT_ROOM_ID = "CHAT_ROOM_ID"
    }

    @Inject(id = com.bria.voip.R.id.chat_room_members_screen_search_view, col = false)
    private lateinit var mSearchView: CustomSearchView

    @Inject(R.id.chat_room_members_screen_recycler_view)
    private lateinit var mList: RecyclerView

    @Clickable
    @Inject(id = R.id.chat_room_members_screen_add_member, back = ESetting.ColorBrandDefault)
    private lateinit var mAddMember: FloatingActionButton

    private lateinit var mLinearLayoutManager: LinearLayoutManager
    private lateinit var mAdapter: ChatRoomMembersAdapter


    override fun getPresenterClass(): Class<ChatRoomMembersPresenter> = ChatRoomMembersPresenter::class.java

    override fun getTitle(): String? = presenter.title


    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)

        presenter.subscribe(this)
        presenter.init(bundle?.getLong(CHAT_ROOM_ID) ?: Random().nextLong())

        mLinearLayoutManager = LinearLayoutManager(activity)
        mList.setLayoutManager(mLinearLayoutManager)

        mAdapter = ChatRoomMembersAdapter(presenter.settings, mList, ActionsHandlerImpl(), presenter.dataProvider, presenter.adapterState)
        mList.setAdapter(mAdapter)

        if (bundle != null) {
            val parcelable = bundle.getParcelable<Parcelable>(KEY_LINEAR_LAYOUT_STATE)
            if (parcelable != null) {
                mLinearLayoutManager.onRestoreInstanceState(parcelable)
            }
        }

        mSearchView.setEventHandler(object : CustomSearchView.EventHandler {
            override fun searchFor(searchString: String?) {
                presenter.filterBy(searchString)
            }

            override fun onClosed() {}
            override fun onOpened() {}
            override fun onClear() {}
        })

        mSearchView.setDelayedSearchInterval(0)
        mSearchView.setCleanSearchOnClose(true)
    }

    override fun onDestroy(finishing: Boolean) {
        presenter.unsubscribe()

        super.onDestroy(finishing)
    }

    override fun onStart(bundle: Bundle?) {
        super.onStart(bundle)
    }

    override fun onStop(finishing: Boolean) {
        if (finishing) {
            mSearchView.close()
            mSearchView.setEventHandler(null)
        }

        super.onStop(finishing)
    }

    override fun onSaveState(stateBundle: Bundle) {
        super.onSaveState(stateBundle)

        stateBundle.putParcelable(KEY_LINEAR_LAYOUT_STATE, mLinearLayoutManager.onSaveInstanceState())
    }

    override fun onResume() {
        super.onResume()

    }

    override fun onPause(finishing: Boolean) {
        super.onPause(finishing)
    }

    override fun onMenuItemClick(menuItem: MenuItem?): Boolean {
        when (menuItem?.itemId) {

            R.id.chat_room_members_mi_search -> {
                mSearchView.open()
                return true
            }
        }
        return super.onMenuItemClick(menuItem)

    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.chat_room_members_screen_add_member -> {

                AlertDialog.Builder(activity)
                    .setMessage("What to do, what to do...")
                    .setNeutralButton("OK", null)
                    .create()
                    .show()
            }
        }

        super.onClick(v)
    }

    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
        val type = event.type as ChatRoomMembersPresenter.Events
        Log.d(TAG, "OnPresenterEvent ${type.name}")

        when (type) {
            ChatRoomMembersPresenter.Events.BZZZ -> {

            }
        }
    }

    inner class ActionsHandlerImpl : ChatRoomMembersAdapter.ActionsHandler {
        override fun onCallClicked(buddy: Buddy) {
            AlertDialog.Builder(activity)
                .setMessage("Don't know how to call xmpp buddy")
                .setNeutralButton("OK", null)
                .create()
                .show()
        }

        override fun onMessageClicked(buddy: Buddy) {
            presenter.sendMessageTo(buddy)
        }

        override fun onRemoveClicked(buddy: Buddy) {
            presenter.removeFromChatRoom(buddy)
        }

    }
}

