package com.bria.voip.ui.main.im.chatroom

import android.text.SpannableString
import android.text.Spanned
import android.text.TextUtils
import android.widget.MultiAutoCompleteTextView

class ChatRoomMentionsTokenizer : MultiAutoCompleteTextView.Tokenizer {

    /**
     * Returns the start of the token that ends at offset
     * <code>cursor</code> within <code>text</code>.
     */
    override fun findTokenStart(text: CharSequence, cursor: Int): Int {
        var i = cursor

        // find space to the left
        while (i > 0 && text[i - 1] != ' ') {
            i--
        }

        // fin non space to the right
        while (i < cursor && text[i] == ' ') {
            i++
        }

        return i
    }

    /**
     * Returns the end of the token (index after last char)
     * that begins at offset <code>cursor</code> within <code>text</code>.
     */
    override fun findTokenEnd(text: CharSequence, cursor: Int): Int {
        var i = cursor
        val len = text.length

        while (i < len) {
            if (text[i] == ' ') {
                return i
            } else {
                i++
            }
        }

        return len
    }

    /**
     * Returns <code>text</code>, modified, if necessary, to ensure that
     * it ends with a token terminator (for example a space or comma).
     */
    override fun terminateToken(text: CharSequence): CharSequence {
        if (text is Spanned) {
            val sp = SpannableString(text.toString() + " ")
            TextUtils.copySpansFrom(text, 0, text.length, Any::class.java, sp, 0)
            return sp
        } else {
            return text.toString() + " "
        }
    }
}
