package com.bria.voip.ui.main.im.chatroom

import android.graphics.Bitmap
import android.graphics.Typeface
import android.support.v7.content.res.AppCompatResources
import android.support.v7.widget.RecyclerView
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.bria.common.R
import com.bria.common.controller.Controllers
import com.bria.common.controller.im.refactoring.roomdb.LocalRoomInfo
import com.bria.common.controller.im.refactoring.roomdb.entities.getInitials
import com.bria.common.controller.settings.ESetting
import com.bria.common.controller.settings.core.Settings
import com.bria.common.ui.util.SearchQueryColorizer
import com.bria.common.uiframework.coloring.Coloring
import com.bria.common.uiframework.lists.recycler.AbstractMultiSelectRecyclerAdapter
import com.bria.common.uiframework.lists.recycler.AbstractRecyclerAdapter
import com.bria.common.util.AndroidUtils

class JoinRoomAdapter(
        private val settings: Settings,
        recyclerView: RecyclerView,
        private val joinRoomsDataProvider: JoinRoomsDataProvider
        ) : AbstractMultiSelectRecyclerAdapter<LocalRoomInfo, JoinRoomAdapter.ViewHolder>(
        recyclerView,
        joinRoomsDataProvider,
        R.layout.v2_im_session_item) {

    inner class ViewHolder internal constructor(itemView: View) : AbstractRecyclerAdapter.AbstractViewHolder(itemView) {
        val firstName: TextView = itemView.findViewById(R.id.v2_im_session_item_first_name)
        var lastName: TextView = itemView.findViewById(R.id.v2_im_session_item_last_name)
        var noOfUnreadMsgs: TextView = itemView.findViewById(R.id.v2_im_session_item_unread_messages)
         var lastMessage: TextView = itemView.findViewById(R.id.v2_im_session_item_last_message)
         var lastMessageDate: TextView = itemView.findViewById(R.id.v2_im_session_item_last_message_date)
         var initials: TextView = itemView.findViewById(R.id.v2_im_session_item_initials)
         var avatar: ImageView = itemView.findViewById(R.id.v2_im_session_item_avatar)
         var squaredAvatar: ImageView = itemView.findViewById(R.id.v2_im_session_item_squared_avatar)
         var presence: ImageView = itemView.findViewById(R.id.v2_im_session_item_presence)
    }

    private val mCheckMark = createSquaredCheckMark(mContext.get()?.getResources()?.getDimensionPixelSize(R.dimen.list_item_size_half) ?: 0)

    private var searchQueryColorizer = SearchQueryColorizer(Coloring.get().decodeColor(settings.getStr(ESetting.ColorBrandTint)))


    override fun provideNewViewHolder(inflatedView: View, viewType: Int): JoinRoomAdapter.ViewHolder {
        return ViewHolder(inflatedView)
    }

    override fun onNewViewHolderReady(viewHolder: JoinRoomAdapter.ViewHolder?, viewType: Int) {}

    override fun onUpdateNeeded(viewHolder: JoinRoomAdapter.ViewHolder, position: Int) {
        val roomInfo = joinRoomsDataProvider.getItemAt(position)
        viewHolder.firstName.typeface = Typeface.create(viewHolder.firstName.typeface, Typeface.NORMAL)
        viewHolder.lastName.typeface = Typeface.create(viewHolder.firstName.typeface, Typeface.NORMAL)
        viewHolder.firstName.text = roomInfo.name
        searchQueryColorizer.colorize(viewHolder.firstName, listOf(joinRoomsDataProvider.searchQuery))
        viewHolder.lastName.text = ""
        viewHolder.avatar.visibility = View.INVISIBLE
        if (isSelected(position)) {
            viewHolder.squaredAvatar.setImageBitmap(mCheckMark)
            viewHolder.squaredAvatar.visibility = View.VISIBLE
            viewHolder.initials.visibility = View.GONE

        } else {
            viewHolder.squaredAvatar.visibility = View.GONE
            viewHolder.initials.visibility = View.VISIBLE
        }
        viewHolder.initials.text = roomInfo.name.getInitials()
        viewHolder.presence.visibility = View.GONE
        viewHolder.noOfUnreadMsgs.visibility = View.GONE
        viewHolder.lastMessage.visibility = View.GONE
        viewHolder.lastMessageDate.visibility = View.GONE

    }

    protected fun createSquaredCheckMark(size: Int): Bitmap {
        val brandTint = Coloring.get().decodeColor(Controllers.get().settings.getStr(ESetting.ColorBrandTint))
        var checkMark = AppCompatResources.getDrawable(mContext.get()!!, R.drawable.ic_chatroom_select)
        checkMark = Coloring.get().colorDrawable(checkMark, brandTint)
        return AndroidUtils.drawableToBitmap(checkMark, size, size)
    }

}
