package com.bria.voip.ui.main.im.chatroom

import com.bria.common.controller.Controllers
import com.bria.common.controller.accounts.IAccountsFilter
import com.bria.common.controller.settings.core.Settings
import com.bria.common.uiframework.presenters.AbstractPresenter
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.voip.R
import io.reactivex.disposables.CompositeDisposable

class JoinRoomPresenter : AbstractPresenter() {
    var disposable = CompositeDisposable()
    var title: String = getString(R.string.tJoinRooms)

    val settings get() = Settings.get(context)
    val dataProvider = JoinRoomsDataProvider(Controllers.get().im.imData.chatRoomRepo)

    var waitingForRoomCreation = ""

    enum class Events : IPresenterEventTypeEnum {
        DATA_LOADED,
        DATA_LOADING,
        GO_UP,
        GO_TO;
        override fun getType(): IPresenterEventTypeEnum = this
    }

    init {
        disposable.add(Controllers.get().im.imData.chatRoomRepo.getOnServerRoomListChangedObservable().subscribe ({
            dataProvider.refilter()
        },{}))

        dataProvider.attachListener(object : IFilterableDataProvider.DataProviderListener {
            override fun onDataLoaded(isEmpty: Boolean) {
                firePresenterEvent(Events.DATA_LOADED, isEmpty)
            }

            override fun onDataLoading() {
                firePresenterEvent(Events.DATA_LOADING)
            }

        })
    }

    fun filterBy(searchString: String?) {
        dataProvider.filter(searchString)
    }

    fun joinSelectedRooms(selectedRooms: List<String>) {

        Controllers.get().accounts.getAccounts(IAccountsFilter.XMPP).firstOrNull()?.let {
            Controllers.get().im.chatApi.getXmppAccount(it)?.let { xmppAccount ->
                Controllers.get().im.chatApi.chatRoomApi.joinRooms(selectedRooms, xmppAccount)
                //if one go to chat room session else go up
                if (selectedRooms.size > 1) {
                    firePresenterEvent(Events.GO_UP)
                } else {
                    disposable.add(Controllers.get().im.imData.chatRoomRepo.getOnRoomAddedObservable().firstOrError().subscribe({
                        if (it.isNotEmpty()) {
                            firePresenterEvent(Events.GO_TO, it[0].id)
                        } else {
                            firePresenterEvent(Events.GO_UP)
                        }
                    },{
                        firePresenterEvent(Events.GO_UP)
                    }))
                }
            }
        }

    }

}
