package com.bria.voip.ui.main.im.filetransfer

import android.Manifest
import android.content.pm.PackageManager
import android.os.Bundle
import android.support.annotation.MainThread
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.View
import android.widget.*
import com.bria.common.controller.im.refactoring.adapter.IDataProviderEvents
import com.bria.common.controller.im.refactoring.dataprovider.GalleryListAdapter
import com.bria.common.uiframework.annotations.Clickable
import com.bria.common.uiframework.annotations.Inject
import com.bria.common.uiframework.annotations.Layout
import com.bria.common.uiframework.lists.recycler.IOnRecyclerViewItemEventsListener
import com.bria.common.uiframework.presenters.IPresenterEvent
import com.bria.common.uiframework.screens.AbstractScreen
import com.bria.common.util.GlobalConstants
import com.bria.voip.R
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions

@Layout(R.layout.gallery_screen)
class GalleryScreen : AbstractScreen<GalleryPresenter>(), IOnRecyclerViewItemEventsListener {

    @Inject(R.id.gallery_screen_list)
    lateinit var recyclerView: RecyclerView

    @Inject(R.id.gallery_screen_toolbar_count)
    lateinit var count: TextView

//    @Inject(R.id.gallery_screen_toolbar_folder)
//    lateinit var folderName: TextView

    @Clickable
    @Inject(R.id.gallery_list_item_check)
    lateinit var check: ImageView

    @Inject(R.id.gallery_screen_no_permission)
    lateinit var noPermissionContainer: LinearLayout

    @Inject(R.id.gallery_screen_no_images)
    lateinit var noImages: TextView

    @Clickable
    @Inject(R.id.missing_permission_error_state_request_button)
    lateinit var requestPermissionButton: Button

    @Inject(R.id.missing_permission_error_state_text_view)
    lateinit var requestPermissionText: TextView

    lateinit var adapter: GalleryListAdapter

    @MainThread
    override fun onCreate(bundle: Bundle?) {
        super.onCreate(bundle)
        val name = bundle?.getString(GlobalConstants.KEY_SENDER_NAME, null)
        name?.let { presenter.senderName = it}
        val options = RequestOptions()
            .diskCacheStrategy(DiskCacheStrategy.RESOURCE).override(160, 160).skipMemoryCache(true).error(R.drawable.ic_error_red_24dp)
        val glide = Glide.with(activity)
        val builder = glide.asBitmap()
        recyclerView.layoutManager = GridLayoutManager(activity, 2)
        recyclerView.setHasFixedSize(true)
        adapter = GalleryListAdapter(presenter.dataProvider, options, builder, eventsListener = this, dataProviderEvents =  dataEvents, context = activity)
        recyclerView.adapter = adapter
        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrollStateChanged(recyclerView: RecyclerView?, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                when (newState) {
                    RecyclerView.SCROLL_STATE_IDLE -> glide.resumeRequests()
                    AbsListView.OnScrollListener.SCROLL_STATE_TOUCH_SCROLL, AbsListView.OnScrollListener.SCROLL_STATE_FLING -> glide.pauseRequests()
                }
            }
        })
        updateToolbar()
        updateList()
    }

    @MainThread
    override fun onPresenterEvent(event: IPresenterEvent<*, *>) {
    }

    @Suppress("UNCHECKED_CAST")
    override fun getPresenterClass(): Class<GalleryPresenter> = GalleryPresenter::class.java

    override fun getTitle(): String? = presenter.getTitle()

    @MainThread
    override fun onItemClick(item: View?, index: Int) {
        if (adapter.dataProvider.isInRootState()) {
            val folder = adapter.galleryList[index].name
            adapter.dataProvider.moveToFolder(folder)
            //folderName.text = folder
        } else {
            if (adapter.getSelectedCount() > 0) {
                adapter.toggleSelectionByIndex(index)
            }
        }
        updateToolbar()
    }

    private fun updateToolbar() {
        toolbar.visibility = if (adapter.getSelectedCount() > 0) View.GONE else View.VISIBLE
        count.visibility = if (adapter.getSelectedCount() > 0) View.VISIBLE else View.GONE
        count.text = activity.resources.getQuantityString(R.plurals.files_selected, adapter.getSelectedCount(), adapter.getSelectedCount())
        check.visibility = if (adapter.getSelectedCount() > 0) View.VISIBLE else View.GONE
        //folderName.text = if (adapter.getSelectedCount() > 0) presenter.getTitle() else adapter.dataProvider.activeFolder
        updateTitle()
    }

    @MainThread
    override fun onItemLongClick(item: View?, index: Int) {
        if (!adapter.dataProvider.isInRootState()) {
            adapter.toggleSelectionByIndex(index)
        }
        updateToolbar()
    }

    @MainThread
    override fun onBackPressed(willGoToParent: Boolean): Boolean {
        if (!adapter.dataProvider.isInRootState()) {
            adapter.dataProvider.moveToRootState()
            updateToolbar()
            return true
        }
        return false
    }

    @MainThread
    override fun onClick(v: View) {
        when (v.id) {
            R.id.gallery_list_item_check -> {
                val bundle = Bundle()
                bundle.putStringArray(GlobalConstants.KEY_IMAGES_TO_SEND, adapter.dataProvider.selectedItems.map { it -> it.path }.toTypedArray())
                publishResult(bundle)
                presenter.dataProvider.moveToRootState()
                if (isInsideDialog) {
                    dismissScreenHolderDialog()
                }
            }
            R.id.missing_permission_error_state_request_button -> {
                requestPermission(Manifest.permission.READ_EXTERNAL_STORAGE, GlobalConstants.CP_PERMISSION_READ_EXTERNAL_STORAGE, getString(R
                    .string.tGalleryPermissionRequestExplanation))
            }
        }
        super.onClick(v)
    }

    fun updateList() {
        if (!checkPermission(android.Manifest.permission.READ_EXTERNAL_STORAGE)) {
            recyclerView.visibility = View.GONE
            requestPermissionText.text = getString(R.string.tGalleryPermissionError)
            noPermissionContainer.visibility = View.VISIBLE
            noImages.visibility = View.GONE
        } else if (adapter.itemCount == 0) {
            recyclerView.visibility = View.GONE
            noPermissionContainer.visibility = View.GONE
            noImages.visibility = View.VISIBLE
        } else{
            recyclerView.visibility = View.VISIBLE
            noPermissionContainer.visibility = View.GONE
            noImages.visibility = View.GONE
        }
    }

    val dataEvents = object : IDataProviderEvents {
        override fun onListLoading() {
        }

        override fun onListLoaded(size: Int) {
            updateList()
        }

    }

    @MainThread
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        val permissionGranted = grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED
        if (permissionGranted && requestCode == GlobalConstants.CP_PERMISSION_READ_EXTERNAL_STORAGE) {
            adapter.reload()
            recyclerView.visibility = View.VISIBLE
            noPermissionContainer.visibility = View.GONE
        }
    }

}
