package com.bria.voip.ui.shared.pickers

import android.os.Bundle
import android.support.annotation.MainThread
import com.bria.common.controller.contacts.buddy.Buddy
import com.bria.common.controller.contacts.buddy.IBuddyCtrlEvents
import com.bria.common.controller.contacts.buddy.IBuddyCtrlObserver
import com.bria.common.controller.im.ImController
import com.bria.common.modules.BriaGraph
import com.bria.common.network.INetworkObserver
import com.bria.common.network.NetworkModule
import com.bria.common.uiframework.presenters.IPresenterEventTypeEnum
import com.bria.common.uireusable.dataprovider.IFilterableDataProvider
import com.bria.voip.ui.shared.pickers.BuddyPickerScreen.ALREADY_PRESENT_BUDDIES
import com.bria.voip.ui.shared.pickers.generic.AbstractAdvancedPickerPresenter
import com.bria.voip.ui.shared.pickers.generic.AbstractPickerScreen

class ChatRoomMembersPickerPresenter : AbstractAdvancedPickerPresenter<Buddy>() {

    enum class ChatRoomMembersPickerPresenterEvents : IPresenterEventTypeEnum {
        NETWORK_CONNECTION_LOST,
        PRESENCE_CHANGED;

        override fun getType(): IPresenterEventTypeEnum {
            return this
        }
    }

    private val mNetworkModule: NetworkModule = BriaGraph.networkModule

    private val mBuddyCtrlObserver = BuddyCtrlObserver()
    private val mNetworkObserver = NetworkObserver()

    @MainThread
    override fun onSubscribe() {
        super.onSubscribe()

        mObservables.buddy.attachObserver(mBuddyCtrlObserver)
        mNetworkModule.attachObserver(mNetworkObserver)
    }

    @MainThread
    override fun onUnsubscribe() {
        super.onUnsubscribe()

        mObservables.buddy.detachObserver(mBuddyCtrlObserver)
        mNetworkModule.detachObserver(mNetworkObserver)
    }

    override fun createDataProvider(): IFilterableDataProvider<Buddy>? {
        return ChatRoomMembersPickerProvider()
    }

    override fun processDataInBackground(data: MutableList<Buddy>): Any {
        return data
    }

    private val specificDataProvider: ChatRoomMembersPickerProvider
        get() = dataProvider as ChatRoomMembersPickerProvider

    fun setOnlineStatusFilter(onlineStatus: IBuddyCtrlEvents.EBuddyFilterType) {
        specificDataProvider.onlineStatus = onlineStatus
    }

    fun setMembers(fixedMembers: Set<String>) {
        specificDataProvider.setAlreadyPresentBuddyIds(fixedMembers)
    }

    fun calculateParticipantLimit(bundle: Bundle?): Int {
        if (bundle?.containsKey(AbstractPickerScreen.MAX_NUMBER_OF_CHOICES) == true) {
            return bundle.getInt(AbstractPickerScreen.MAX_NUMBER_OF_CHOICES)
        }

        val presentBuddies = bundle?.getStringArrayList(ALREADY_PRESENT_BUDDIES) // buddies already in chat

        return if (presentBuddies != null) {
            // todo vi verify these constraints
            ImController.MAX_GROUP_CHAT_PARTICIPANTS - presentBuddies.size
        } else {
            ImController.MAX_GROUP_CHAT_PARTICIPANTS
        }
    }

    fun isNetworkAvailable(): Boolean = mNetworkModule.connectionType != INetworkObserver.ENetworkType.NONE


    inner class NetworkObserver : INetworkObserver {
        override fun onNetworkConnected(conType: INetworkObserver.ENetworkType?, cellType: INetworkObserver.EMobileType?) {
            refilter()
        }

        override fun onNetworkDisconnected() {
            refilter()
        }

        override fun onPhoneStateChanged(status: INetworkObserver.EPhoneMonitoring?) {}
    }

    inner class BuddyCtrlObserver : IBuddyCtrlObserver {

        override fun onBuddyListUpdated() {
            refilter();
        }

        override fun onBuddyPresenceChanged(buddy: Buddy) {
            if (specificDataProvider.onlineStatus == IBuddyCtrlEvents.EBuddyFilterType.ONLINE) {
                refilter()
            } else {
                for (i in 0 until dataProvider.itemsCount) {
                    val localBuddy = dataProvider.getItemAt(i)
                    if (buddy.getUniqueIdentifier() == localBuddy.uniqueIdentifier) {
                        firePresenterEvent(ChatRoomMembersPickerPresenterEvents.PRESENCE_CHANGED, Integer.valueOf(i))
                    }
                }
            }
        }

        override fun onSubscriptionRequest() {}
        override fun onBuddyFilterChanged(aBuddyFilterType: IBuddyCtrlEvents.EBuddyFilterType?) {}
    }
}
